#ifndef __NDR_IREMOTEWINSPOOL_SCOMPAT_H__
#define __NDR_IREMOTEWINSPOOL_SCOMPAT_H__

struct pipes_struct;
struct dcesrv_endpoint_server;
struct dcesrv_call_state;

const struct dcesrv_endpoint_server *iremotewinspool_get_ep_server(void);

NTSTATUS iremotewinspool__op_ndr_pull(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_pull *pull, void **r);
struct winspool_AsyncOpenPrinter;
WERROR _winspool_AsyncOpenPrinter(struct pipes_struct *p, struct winspool_AsyncOpenPrinter *r);
struct winspool_AsyncAddPrinter;
WERROR _winspool_AsyncAddPrinter(struct pipes_struct *p, struct winspool_AsyncAddPrinter *r);
struct winspool_AsyncSetJob;
WERROR _winspool_AsyncSetJob(struct pipes_struct *p, struct winspool_AsyncSetJob *r);
struct winspool_AsyncGetJob;
WERROR _winspool_AsyncGetJob(struct pipes_struct *p, struct winspool_AsyncGetJob *r);
struct winspool_AsyncEnumJobs;
WERROR _winspool_AsyncEnumJobs(struct pipes_struct *p, struct winspool_AsyncEnumJobs *r);
struct winspool_AsyncAddJob;
WERROR _winspool_AsyncAddJob(struct pipes_struct *p, struct winspool_AsyncAddJob *r);
struct winspool_AsyncScheduleJob;
WERROR _winspool_AsyncScheduleJob(struct pipes_struct *p, struct winspool_AsyncScheduleJob *r);
struct winspool_AsyncDeletePrinter;
WERROR _winspool_AsyncDeletePrinter(struct pipes_struct *p, struct winspool_AsyncDeletePrinter *r);
struct winspool_AsyncSetPrinter;
WERROR _winspool_AsyncSetPrinter(struct pipes_struct *p, struct winspool_AsyncSetPrinter *r);
struct winspool_AsyncGetPrinter;
WERROR _winspool_AsyncGetPrinter(struct pipes_struct *p, struct winspool_AsyncGetPrinter *r);
struct winspool_AsyncStartDocPrinter;
WERROR _winspool_AsyncStartDocPrinter(struct pipes_struct *p, struct winspool_AsyncStartDocPrinter *r);
struct winspool_AsyncStartPagePrinter;
WERROR _winspool_AsyncStartPagePrinter(struct pipes_struct *p, struct winspool_AsyncStartPagePrinter *r);
struct winspool_AsyncWritePrinter;
WERROR _winspool_AsyncWritePrinter(struct pipes_struct *p, struct winspool_AsyncWritePrinter *r);
struct winspool_AsyncEndPagePrinter;
WERROR _winspool_AsyncEndPagePrinter(struct pipes_struct *p, struct winspool_AsyncEndPagePrinter *r);
struct winspool_AsyncEndDocPrinter;
WERROR _winspool_AsyncEndDocPrinter(struct pipes_struct *p, struct winspool_AsyncEndDocPrinter *r);
struct winspool_AsyncAbortPrinter;
WERROR _winspool_AsyncAbortPrinter(struct pipes_struct *p, struct winspool_AsyncAbortPrinter *r);
struct winspool_AsyncGetPrinterData;
WERROR _winspool_AsyncGetPrinterData(struct pipes_struct *p, struct winspool_AsyncGetPrinterData *r);
struct winspool_AsyncGetPrinterDataEx;
WERROR _winspool_AsyncGetPrinterDataEx(struct pipes_struct *p, struct winspool_AsyncGetPrinterDataEx *r);
struct winspool_AsyncSetPrinterData;
WERROR _winspool_AsyncSetPrinterData(struct pipes_struct *p, struct winspool_AsyncSetPrinterData *r);
struct winspool_AsyncSetPrinterDataEx;
WERROR _winspool_AsyncSetPrinterDataEx(struct pipes_struct *p, struct winspool_AsyncSetPrinterDataEx *r);
struct winspool_AsyncClosePrinter;
WERROR _winspool_AsyncClosePrinter(struct pipes_struct *p, struct winspool_AsyncClosePrinter *r);
struct winspool_AsyncAddForm;
WERROR _winspool_AsyncAddForm(struct pipes_struct *p, struct winspool_AsyncAddForm *r);
struct winspool_AsyncDeleteForm;
WERROR _winspool_AsyncDeleteForm(struct pipes_struct *p, struct winspool_AsyncDeleteForm *r);
struct winspool_AsyncGetForm;
WERROR _winspool_AsyncGetForm(struct pipes_struct *p, struct winspool_AsyncGetForm *r);
struct winspool_AsyncSetForm;
WERROR _winspool_AsyncSetForm(struct pipes_struct *p, struct winspool_AsyncSetForm *r);
struct winspool_AsyncEnumForms;
WERROR _winspool_AsyncEnumForms(struct pipes_struct *p, struct winspool_AsyncEnumForms *r);
struct winspool_AsyncGetPrinterDriver;
WERROR _winspool_AsyncGetPrinterDriver(struct pipes_struct *p, struct winspool_AsyncGetPrinterDriver *r);
struct winspool_AsyncEnumPrinterData;
WERROR _winspool_AsyncEnumPrinterData(struct pipes_struct *p, struct winspool_AsyncEnumPrinterData *r);
struct winspool_AsyncEnumPrinterDataEx;
WERROR _winspool_AsyncEnumPrinterDataEx(struct pipes_struct *p, struct winspool_AsyncEnumPrinterDataEx *r);
struct winspool_AsyncEnumPrinterKey;
WERROR _winspool_AsyncEnumPrinterKey(struct pipes_struct *p, struct winspool_AsyncEnumPrinterKey *r);
struct winspool_AsyncDeletePrinterData;
WERROR _winspool_AsyncDeletePrinterData(struct pipes_struct *p, struct winspool_AsyncDeletePrinterData *r);
struct winspool_AsyncDeletePrinterDataEx;
WERROR _winspool_AsyncDeletePrinterDataEx(struct pipes_struct *p, struct winspool_AsyncDeletePrinterDataEx *r);
struct winspool_AsyncDeletePrinterKey;
WERROR _winspool_AsyncDeletePrinterKey(struct pipes_struct *p, struct winspool_AsyncDeletePrinterKey *r);
struct winspool_AsyncXcvData;
WERROR _winspool_AsyncXcvData(struct pipes_struct *p, struct winspool_AsyncXcvData *r);
struct winspool_AsyncSendRecvBidiData;
WERROR _winspool_AsyncSendRecvBidiData(struct pipes_struct *p, struct winspool_AsyncSendRecvBidiData *r);
struct winspool_AsyncCreatePrinterIC;
WERROR _winspool_AsyncCreatePrinterIC(struct pipes_struct *p, struct winspool_AsyncCreatePrinterIC *r);
struct winspool_AsyncPlayGdiScriptOnPrinterIC;
WERROR _winspool_AsyncPlayGdiScriptOnPrinterIC(struct pipes_struct *p, struct winspool_AsyncPlayGdiScriptOnPrinterIC *r);
struct winspool_AsyncDeletePrinterIC;
WERROR _winspool_AsyncDeletePrinterIC(struct pipes_struct *p, struct winspool_AsyncDeletePrinterIC *r);
struct winspool_AsyncEnumPrinters;
WERROR _winspool_AsyncEnumPrinters(struct pipes_struct *p, struct winspool_AsyncEnumPrinters *r);
struct winspool_AsyncAddPrinterDriver;
WERROR _winspool_AsyncAddPrinterDriver(struct pipes_struct *p, struct winspool_AsyncAddPrinterDriver *r);
struct winspool_AsyncEnumPrinterDrivers;
WERROR _winspool_AsyncEnumPrinterDrivers(struct pipes_struct *p, struct winspool_AsyncEnumPrinterDrivers *r);
struct winspool_AsyncGetPrinterDriverDirectory;
WERROR _winspool_AsyncGetPrinterDriverDirectory(struct pipes_struct *p, struct winspool_AsyncGetPrinterDriverDirectory *r);
struct winspool_AsyncDeletePrinterDriver;
WERROR _winspool_AsyncDeletePrinterDriver(struct pipes_struct *p, struct winspool_AsyncDeletePrinterDriver *r);
struct winspool_AsyncDeletePrinterDriverEx;
WERROR _winspool_AsyncDeletePrinterDriverEx(struct pipes_struct *p, struct winspool_AsyncDeletePrinterDriverEx *r);
struct winspool_AsyncAddPrintProcessor;
WERROR _winspool_AsyncAddPrintProcessor(struct pipes_struct *p, struct winspool_AsyncAddPrintProcessor *r);
struct winspool_AsyncEnumPrintProcessors;
WERROR _winspool_AsyncEnumPrintProcessors(struct pipes_struct *p, struct winspool_AsyncEnumPrintProcessors *r);
struct winspool_AsyncGetPrintProcessorDirectory;
WERROR _winspool_AsyncGetPrintProcessorDirectory(struct pipes_struct *p, struct winspool_AsyncGetPrintProcessorDirectory *r);
struct winspool_AsyncEnumPorts;
WERROR _winspool_AsyncEnumPorts(struct pipes_struct *p, struct winspool_AsyncEnumPorts *r);
struct winspool_AsyncEnumMonitors;
WERROR _winspool_AsyncEnumMonitors(struct pipes_struct *p, struct winspool_AsyncEnumMonitors *r);
struct winspool_AsyncAddPort;
WERROR _winspool_AsyncAddPort(struct pipes_struct *p, struct winspool_AsyncAddPort *r);
struct winspool_AsyncSetPort;
WERROR _winspool_AsyncSetPort(struct pipes_struct *p, struct winspool_AsyncSetPort *r);
struct winspool_AsyncAddMonitor;
WERROR _winspool_AsyncAddMonitor(struct pipes_struct *p, struct winspool_AsyncAddMonitor *r);
struct winspool_AsyncDeleteMonitor;
WERROR _winspool_AsyncDeleteMonitor(struct pipes_struct *p, struct winspool_AsyncDeleteMonitor *r);
struct winspool_AsyncDeletePrintProcessor;
WERROR _winspool_AsyncDeletePrintProcessor(struct pipes_struct *p, struct winspool_AsyncDeletePrintProcessor *r);
struct winspool_AsyncEnumPrintProcessorDatatypes;
WERROR _winspool_AsyncEnumPrintProcessorDatatypes(struct pipes_struct *p, struct winspool_AsyncEnumPrintProcessorDatatypes *r);
struct winspool_AsyncAddPerMachineConnection;
WERROR _winspool_AsyncAddPerMachineConnection(struct pipes_struct *p, struct winspool_AsyncAddPerMachineConnection *r);
struct winspool_AsyncDeletePerMachineConnection;
WERROR _winspool_AsyncDeletePerMachineConnection(struct pipes_struct *p, struct winspool_AsyncDeletePerMachineConnection *r);
struct winspool_AsyncEnumPerMachineConnections;
WERROR _winspool_AsyncEnumPerMachineConnections(struct pipes_struct *p, struct winspool_AsyncEnumPerMachineConnections *r);
struct winspool_SyncRegisterForRemoteNotifications;
HRESULT _winspool_SyncRegisterForRemoteNotifications(struct pipes_struct *p, struct winspool_SyncRegisterForRemoteNotifications *r);
struct winspool_SyncUnRegisterForRemoteNotifications;
HRESULT _winspool_SyncUnRegisterForRemoteNotifications(struct pipes_struct *p, struct winspool_SyncUnRegisterForRemoteNotifications *r);
struct winspool_SyncRefreshRemoteNotifications;
HRESULT _winspool_SyncRefreshRemoteNotifications(struct pipes_struct *p, struct winspool_SyncRefreshRemoteNotifications *r);
struct winspool_AsyncGetRemoteNotifications;
HRESULT _winspool_AsyncGetRemoteNotifications(struct pipes_struct *p, struct winspool_AsyncGetRemoteNotifications *r);
struct winspool_AsyncInstallPrinterDriverFromPackage;
HRESULT _winspool_AsyncInstallPrinterDriverFromPackage(struct pipes_struct *p, struct winspool_AsyncInstallPrinterDriverFromPackage *r);
struct winspool_AsyncUploadPrinterDriverPackage;
HRESULT _winspool_AsyncUploadPrinterDriverPackage(struct pipes_struct *p, struct winspool_AsyncUploadPrinterDriverPackage *r);
struct winspool_AsyncGetCorePrinterDrivers;
HRESULT _winspool_AsyncGetCorePrinterDrivers(struct pipes_struct *p, struct winspool_AsyncGetCorePrinterDrivers *r);
struct winspool_AsyncCorePrinterDriverInstalled;
HRESULT _winspool_AsyncCorePrinterDriverInstalled(struct pipes_struct *p, struct winspool_AsyncCorePrinterDriverInstalled *r);
struct winspool_AsyncGetPrinterDriverPackagePath;
HRESULT _winspool_AsyncGetPrinterDriverPackagePath(struct pipes_struct *p, struct winspool_AsyncGetPrinterDriverPackagePath *r);
struct winspool_AsyncDeletePrinterDriverPackage;
HRESULT _winspool_AsyncDeletePrinterDriverPackage(struct pipes_struct *p, struct winspool_AsyncDeletePrinterDriverPackage *r);
struct winspool_AsyncReadPrinter;
WERROR _winspool_AsyncReadPrinter(struct pipes_struct *p, struct winspool_AsyncReadPrinter *r);
struct winspool_AsyncResetPrinter;
WERROR _winspool_AsyncResetPrinter(struct pipes_struct *p, struct winspool_AsyncResetPrinter *r);
struct winspool_AsyncGetJobNamedPropertyValue;
WERROR _winspool_AsyncGetJobNamedPropertyValue(struct pipes_struct *p, struct winspool_AsyncGetJobNamedPropertyValue *r);
struct winspool_AsyncSetJobNamedProperty;
WERROR _winspool_AsyncSetJobNamedProperty(struct pipes_struct *p, struct winspool_AsyncSetJobNamedProperty *r);
struct winspool_AsyncDeleteJobNamedProperty;
WERROR _winspool_AsyncDeleteJobNamedProperty(struct pipes_struct *p, struct winspool_AsyncDeleteJobNamedProperty *r);
struct winspool_AsyncEnumJobNamedProperties;
WERROR _winspool_AsyncEnumJobNamedProperties(struct pipes_struct *p, struct winspool_AsyncEnumJobNamedProperties *r);
struct winspool_AsyncLogJobInfoForBranchOffice;
WERROR _winspool_AsyncLogJobInfoForBranchOffice(struct pipes_struct *p, struct winspool_AsyncLogJobInfoForBranchOffice *r);
NTSTATUS iremotewinspool__op_dispatch(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS iremotewinspool__op_reply(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS iremotewinspool__op_ndr_push(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_push *push, const void *r);
NTSTATUS iremotewinspool__op_local(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
#endif /* __NDR_IREMOTEWINSPOOL_SCOMPAT_H__ */
