/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_printcap
#define _PIDL_HEADER_printcap

#include <stdint.h>

#include "libcli/util/ntstatus.h"

#ifndef _HEADER_printcap
#define _HEADER_printcap

struct pcap_printer {
	const char *name;/* [charset(UTF8),unique] */
	const char *info;/* [charset(UTF8),unique] */
	const char *location;/* [charset(UTF8),unique] */
};

struct pcap_data {
	NTSTATUS status;
	uint32_t count;
	struct pcap_printer *printers;/* [size_is(count)] */
}/* [public] */;

#endif /* _HEADER_printcap */
#endif /* _PIDL_HEADER_printcap */
