/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_keysvc
#define _PIDL_HEADER_keysvc

#include <stdint.h>


#ifndef _HEADER_keysvc
#define _HEADER_keysvc


struct keysvc_Unknown0 {
	struct {
		WERROR result;
	} out;

};

#endif /* _HEADER_keysvc */
#endif /* _PIDL_HEADER_keysvc */
