/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/source3/librpc/gen_ndr/ads.h"

#ifndef _HEADER_NDR_ads
#define _HEADER_NDR_ads

extern const struct ndr_interface_table ndr_table_ads;
#define NDR_ADS_CALL_COUNT (0)
enum ndr_err_code ndr_push_ads_saslwrap_type(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum ads_saslwrap_type r);
enum ndr_err_code ndr_pull_ads_saslwrap_type(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum ads_saslwrap_type *r);
void ndr_print_ads_saslwrap_type(struct ndr_print *ndr, const char *name, enum ads_saslwrap_type r);
enum ndr_err_code ndr_push_ads_auth_flags(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint32_t r);
enum ndr_err_code ndr_pull_ads_auth_flags(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint32_t *r);
void ndr_print_ads_auth_flags(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_ads_server(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ads_server *r);
enum ndr_err_code ndr_pull_ads_server(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ads_server *r);
void ndr_print_ads_server(struct ndr_print *ndr, const char *name, const struct ads_server *r);
enum ndr_err_code ndr_push_ads_auth(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ads_auth *r);
enum ndr_err_code ndr_pull_ads_auth(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ads_auth *r);
void ndr_print_ads_auth(struct ndr_print *ndr, const char *name, const struct ads_auth *r);
enum ndr_err_code ndr_push_ads_config(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ads_config *r);
enum ndr_err_code ndr_pull_ads_config(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ads_config *r);
void ndr_print_ads_config(struct ndr_print *ndr, const char *name, const struct ads_config *r);
enum ndr_err_code ndr_push_ads_ldap_buf_in(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ads_ldap_buf_in *r);
enum ndr_err_code ndr_pull_ads_ldap_buf_in(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ads_ldap_buf_in *r);
void ndr_print_ads_ldap_buf_in(struct ndr_print *ndr, const char *name, const struct ads_ldap_buf_in *r);
enum ndr_err_code ndr_push_ads_ldap_buf_out(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ads_ldap_buf_out *r);
enum ndr_err_code ndr_pull_ads_ldap_buf_out(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ads_ldap_buf_out *r);
void ndr_print_ads_ldap_buf_out(struct ndr_print *ndr, const char *name, const struct ads_ldap_buf_out *r);
enum ndr_err_code ndr_push_ads_saslwrap(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ads_saslwrap *r);
enum ndr_err_code ndr_pull_ads_saslwrap(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ads_saslwrap *r);
void ndr_print_ads_saslwrap(struct ndr_print *ndr, const char *name, const struct ads_saslwrap *r);
enum ndr_err_code ndr_push_ads_tlswrap(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ads_tlswrap *r);
enum ndr_err_code ndr_pull_ads_tlswrap(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ads_tlswrap *r);
void ndr_print_ads_tlswrap(struct ndr_print *ndr, const char *name, const struct ads_tlswrap *r);
enum ndr_err_code ndr_push_ads_ldap(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ads_ldap *r);
enum ndr_err_code ndr_pull_ads_ldap(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ads_ldap *r);
void ndr_print_ads_ldap(struct ndr_print *ndr, const char *name, const struct ads_ldap *r);
enum ndr_err_code ndr_push_ads_struct(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ads_struct *r);
enum ndr_err_code ndr_pull_ads_struct(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ads_struct *r);
void ndr_print_ads_struct(struct ndr_print *ndr, const char *name, const struct ads_struct *r);
#endif /* _HEADER_NDR_ads */
