/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/cab.h"

#ifndef _HEADER_NDR_cab
#define _HEADER_NDR_cab

#include "../librpc/ndr/ndr_cab.h"
#define NDR_CAB_UUID "12345678-0000-0000-0000-00000000"
#define NDR_CAB_VERSION 0.0
#define NDR_CAB_NAME "cab"
#define NDR_CAB_HELPSTRING "Cabinet structure"
extern const struct ndr_interface_table ndr_table_cab;
#define NDR_CAB_CALL_COUNT (0)
void ndr_print_cf_flags(struct ndr_print *ndr, const char *name, uint16_t r);
enum ndr_err_code ndr_push_CFHEADER(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct CFHEADER *r);
enum ndr_err_code ndr_pull_CFHEADER(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct CFHEADER *r);
void ndr_print_CFHEADER(struct ndr_print *ndr, const char *name, const struct CFHEADER *r);
void ndr_print_cf_compress_type(struct ndr_print *ndr, const char *name, enum cf_compress_type r);
enum ndr_err_code ndr_push_CFFOLDER(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct CFFOLDER *r);
enum ndr_err_code ndr_pull_CFFOLDER(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct CFFOLDER *r);
void ndr_print_CFFOLDER(struct ndr_print *ndr, const char *name, const struct CFFOLDER *r);
void ndr_print_cf_attributes(struct ndr_print *ndr, const char *name, uint16_t r);
void ndr_print_cf_date(struct ndr_print *ndr, const char *name, const struct cf_date *r);
void ndr_print_cf_time(struct ndr_print *ndr, const char *name, const struct cf_time *r);
enum ndr_err_code ndr_push_CFFILE(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct CFFILE *r);
enum ndr_err_code ndr_pull_CFFILE(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct CFFILE *r);
void ndr_print_CFFILE(struct ndr_print *ndr, const char *name, const struct CFFILE *r);
size_t ndr_size_CFFILE(const struct CFFILE *r, libndr_flags flags);
enum ndr_err_code ndr_push_CFDATA(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct CFDATA *r);
enum ndr_err_code ndr_pull_CFDATA(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct CFDATA *r);
void ndr_print_CFDATA(struct ndr_print *ndr, const char *name, const struct CFDATA *r);
enum ndr_err_code ndr_push_cab_file(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct cab_file *r);
enum ndr_err_code ndr_pull_cab_file(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct cab_file *r);
void ndr_print_cab_file(struct ndr_print *ndr, const char *name, const struct cab_file *r);
#endif /* _HEADER_NDR_cab */
