/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/fscc.h"

#ifndef _HEADER_NDR_fscc
#define _HEADER_NDR_fscc

extern const struct ndr_interface_table ndr_table_fscc;
#define NDR_FSCC_CALL_COUNT (0)
enum ndr_err_code ndr_push_fscc_FileObjectIdBuffer_2(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fscc_FileObjectIdBuffer_2 *r);
enum ndr_err_code ndr_pull_fscc_FileObjectIdBuffer_2(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fscc_FileObjectIdBuffer_2 *r);
void ndr_print_fscc_FileObjectIdBuffer_2(struct ndr_print *ndr, const char *name, const struct fscc_FileObjectIdBuffer_2 *r);
void ndr_print_fscc_FileAttributes(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_fscc_FileNetworkOpenInformation(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fscc_FileNetworkOpenInformation *r);
enum ndr_err_code ndr_pull_fscc_FileNetworkOpenInformation(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fscc_FileNetworkOpenInformation *r);
void ndr_print_fscc_FileNetworkOpenInformation(struct ndr_print *ndr, const char *name, const struct fscc_FileNetworkOpenInformation *r);
#endif /* _HEADER_NDR_fscc */
