/* parser auto-generated by pidl */

#include "replace.h"
#include "bin/default/librpc/gen_ndr/ndr_msgsvc.h"

static enum ndr_err_code ndr_push_NetrMessageNameAdd(struct ndr_push *ndr, ndr_flags_type flags, const struct NetrMessageNameAdd *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_NetrMessageNameAdd(struct ndr_pull *ndr, ndr_flags_type flags, struct NetrMessageNameAdd *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_NetrMessageNameAdd(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetrMessageNameAdd *r)
{
	ndr_print_struct(ndr, name, "NetrMessageNameAdd");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "NetrMessageNameAdd");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "NetrMessageNameAdd");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_NetrMessageNameEnum(struct ndr_push *ndr, ndr_flags_type flags, const struct NetrMessageNameEnum *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_NetrMessageNameEnum(struct ndr_pull *ndr, ndr_flags_type flags, struct NetrMessageNameEnum *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_NetrMessageNameEnum(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetrMessageNameEnum *r)
{
	ndr_print_struct(ndr, name, "NetrMessageNameEnum");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "NetrMessageNameEnum");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "NetrMessageNameEnum");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_NetrMessageNameGetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetrMessageNameGetInfo *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_NetrMessageNameGetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetrMessageNameGetInfo *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_NetrMessageNameGetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetrMessageNameGetInfo *r)
{
	ndr_print_struct(ndr, name, "NetrMessageNameGetInfo");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "NetrMessageNameGetInfo");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "NetrMessageNameGetInfo");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_NetrMessageNameDel(struct ndr_push *ndr, ndr_flags_type flags, const struct NetrMessageNameDel *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_NetrMessageNameDel(struct ndr_pull *ndr, ndr_flags_type flags, struct NetrMessageNameDel *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_NetrMessageNameDel(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetrMessageNameDel *r)
{
	ndr_print_struct(ndr, name, "NetrMessageNameDel");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "NetrMessageNameDel");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "NetrMessageNameDel");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

#ifndef SKIP_NDR_TABLE_msgsvc
static const struct ndr_interface_public_struct msgsvc_public_structs[] = {
	{ .name = NULL }
};

static const struct ndr_interface_call msgsvc_calls[] = {
	{
		"NetrMessageNameAdd",
		sizeof(struct NetrMessageNameAdd),
		(ndr_push_flags_fn_t) ndr_push_NetrMessageNameAdd,
		(ndr_pull_flags_fn_t) ndr_pull_NetrMessageNameAdd,
		(ndr_print_function_t) ndr_print_NetrMessageNameAdd,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"NetrMessageNameEnum",
		sizeof(struct NetrMessageNameEnum),
		(ndr_push_flags_fn_t) ndr_push_NetrMessageNameEnum,
		(ndr_pull_flags_fn_t) ndr_pull_NetrMessageNameEnum,
		(ndr_print_function_t) ndr_print_NetrMessageNameEnum,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"NetrMessageNameGetInfo",
		sizeof(struct NetrMessageNameGetInfo),
		(ndr_push_flags_fn_t) ndr_push_NetrMessageNameGetInfo,
		(ndr_pull_flags_fn_t) ndr_pull_NetrMessageNameGetInfo,
		(ndr_print_function_t) ndr_print_NetrMessageNameGetInfo,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"NetrMessageNameDel",
		sizeof(struct NetrMessageNameDel),
		(ndr_push_flags_fn_t) ndr_push_NetrMessageNameDel,
		(ndr_pull_flags_fn_t) ndr_pull_NetrMessageNameDel,
		(ndr_print_function_t) ndr_print_NetrMessageNameDel,
		{ 0, NULL },
		{ 0, NULL },
	},
	{ .name = NULL }
};

static const char * const msgsvc_endpoint_strings[] = {
	"ncacn_np:[\\pipe\\msgsvc]", 
};

static const struct ndr_interface_string_array msgsvc_endpoints = {
	.count	= 1,
	.names	= msgsvc_endpoint_strings
};

static const char * const msgsvc_authservice_strings[] = {
	"host", 
};

static const struct ndr_interface_string_array msgsvc_authservices = {
	.count	= 1,
	.names	= msgsvc_authservice_strings
};


const struct ndr_interface_table ndr_table_msgsvc = {
	.name		= "msgsvc",
	.syntax_id	= {
		{0x17fdd703,0x1827,0x4e34,{0x79,0xd4},{0x24,0xa5,0x5c,0x53,0xbb,0x37}},
		NDR_MSGSVC_VERSION
	},
	.helpstring	= NDR_MSGSVC_HELPSTRING,
	.num_calls	= 4,
	.calls		= msgsvc_calls,
	.num_public_structs	= 0,
	.public_structs		= msgsvc_public_structs,
	.endpoints	= &msgsvc_endpoints,
	.authservices	= &msgsvc_authservices
};

#endif /* SKIP_NDR_TABLE_msgsvc */
static enum ndr_err_code ndr_push_NetrSendMessage(struct ndr_push *ndr, ndr_flags_type flags, const struct NetrSendMessage *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_NetrSendMessage(struct ndr_pull *ndr, ndr_flags_type flags, struct NetrSendMessage *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_NetrSendMessage(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetrSendMessage *r)
{
	ndr_print_struct(ndr, name, "NetrSendMessage");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "NetrSendMessage");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "NetrSendMessage");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

#ifndef SKIP_NDR_TABLE_msgsvcsend
static const struct ndr_interface_public_struct msgsvcsend_public_structs[] = {
	{ .name = NULL }
};

static const struct ndr_interface_call msgsvcsend_calls[] = {
	{
		"NetrSendMessage",
		sizeof(struct NetrSendMessage),
		(ndr_push_flags_fn_t) ndr_push_NetrSendMessage,
		(ndr_pull_flags_fn_t) ndr_pull_NetrSendMessage,
		(ndr_print_function_t) ndr_print_NetrSendMessage,
		{ 0, NULL },
		{ 0, NULL },
	},
	{ .name = NULL }
};

static const char * const msgsvcsend_endpoint_strings[] = {
	"ncacn_np:[\\pipe\\msgsvcsend]", 
};

static const struct ndr_interface_string_array msgsvcsend_endpoints = {
	.count	= 1,
	.names	= msgsvcsend_endpoint_strings
};

static const char * const msgsvcsend_authservice_strings[] = {
	"host", 
};

static const struct ndr_interface_string_array msgsvcsend_authservices = {
	.count	= 1,
	.names	= msgsvcsend_authservice_strings
};


const struct ndr_interface_table ndr_table_msgsvcsend = {
	.name		= "msgsvcsend",
	.syntax_id	= {
		{0x5a7b91f8,0xff00,0x11d0,{0xa9,0xb2},{0x00,0xc0,0x4f,0xb6,0xe6,0xfc}},
		NDR_MSGSVCSEND_VERSION
	},
	.helpstring	= NDR_MSGSVCSEND_HELPSTRING,
	.num_calls	= 1,
	.calls		= msgsvcsend_calls,
	.num_public_structs	= 0,
	.public_structs		= msgsvcsend_public_structs,
	.endpoints	= &msgsvcsend_endpoints,
	.authservices	= &msgsvcsend_authservices
};

#endif /* SKIP_NDR_TABLE_msgsvcsend */
