#ifndef __SOURCE4_TORTURE_LOCAL_PROTO_H__
#define __SOURCE4_TORTURE_LOCAL_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../lib/util/charset/tests/iconv.c  */

struct torture_suite *torture_local_iconv(TALLOC_CTX *mem_ctx);
struct torture_suite *torture_local_iconv(TALLOC_CTX *mem_ctx) ;

/* The following definitions come from ../../lib/talloc/testsuite.c  */

bool torture_local_talloc(struct torture_context *tctx);

/* The following definitions come from ../../source4/lib/messaging/tests/messaging.c  */

struct torture_suite *torture_local_messaging(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/lib/messaging/tests/irpc.c  */

struct torture_suite *torture_local_irpc(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/librpc/tests/binding_string.c  */

struct torture_suite *torture_local_binding_string(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/idtree.c  */

struct torture_suite *torture_local_idtree(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/dlinklist.c  */

struct torture_suite *torture_local_dlinklist(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/lib/socket/testsuite.c  */


/**
  basic testing of udp routines
*/
struct torture_suite *torture_local_socket(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/libcli/resolve/testsuite.c  */

struct torture_suite *torture_local_resolve(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/strlist.c  */

struct torture_suite *torture_local_util_strlist(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/binsearch.c  */

struct torture_suite *torture_local_util_binsearch(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/str.c  */

struct torture_suite *torture_local_util_str(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/time.c  */

struct torture_suite *torture_local_util_time(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/asn1_tests.c  */

struct torture_suite *torture_local_util_asn1(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/data_blob.c  */

struct torture_suite *torture_local_util_data_blob(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/file.c  */

struct torture_suite *torture_local_util_file(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/genrand.c  */

struct torture_suite *torture_local_genrand(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/charset/tests/charset.c  */

struct torture_suite *torture_local_charset(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/charset/tests/convert_string.c  */

struct torture_suite *torture_local_convert_string_handle(TALLOC_CTX *mem_ctx);
struct torture_suite *torture_local_string_case_handle(TALLOC_CTX *mem_ctx);
struct torture_suite *torture_local_convert_string(TALLOC_CTX *mem_ctx);
struct torture_suite *torture_local_string_case(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/charset/tests/util_unistr.c  */

struct torture_suite *torture_local_util_unistr(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/tdr/testsuite.c  */

struct torture_suite *torture_local_tdr(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/tevent/testsuite.c  */

struct torture_suite *torture_local_event(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/param/tests/share.c  */

struct torture_suite *torture_local_share(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/tevent/test_req.c  */

struct torture_suite *torture_local_tevent_req(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/param/tests/loadparm.c  */

struct torture_suite *torture_local_loadparm(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/local/local.c  */

NTSTATUS torture_local_init(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/local/dbspeed.c  */

struct torture_suite *torture_local_dbspeed(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/local/torture.c  */

struct torture_suite *torture_local_torture(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/ldb/ldb.c  */


/**
 * This is the hex code derived from the tdbdump for
 * "st/ad_dc/private/sam.ldb.d/DC=ADDC,DC=SAMBA,DC=EXAMPLE,DC=COM.ldb"
 * key "DN=CN=DDA1D01D-4BD7-4C49-A184-46F9241B560E,CN=OPERATIONS,CN=DOMAINUPDATES,CN=SYSTEM,DC=ADDC,DC=SAMBA,DC=EXAMPLE,DC=COM\00"
 *   -- adrianc
 */
struct torture_suite *torture_ldb(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/dsdb/common/tests/dsdb_dn.c  */

struct torture_suite *torture_dsdb_dn(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/dsdb/schema/tests/schema_syntax.c  */

DATA_BLOB hexstr_to_data_blob(TALLOC_CTX *mem_ctx, const char *string) ;

/**
 * DSDB-SYNTAX test suite creation
 */
struct torture_suite *torture_dsdb_syntax(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/anonymous_shared.c  */

struct torture_suite *torture_local_util_anonymous_shared(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/strv.c  */

struct torture_suite *torture_local_util_strv(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/strv_util.c  */

struct torture_suite *torture_local_util_strv_util(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/util.c  */

struct torture_suite *torture_local_util(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/util_str_escape.c  */

struct torture_suite *torture_local_util_str_escape(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../lib/util/tests/tfork.c  */

struct torture_suite *torture_local_tfork(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/local/verif_trailer.c  */

struct torture_suite *torture_local_verif_trailer(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/local/nss_tests.c  */

struct torture_suite *torture_local_nss(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/local/mdspkt.c  */

struct torture_suite *torture_local_mdspkt(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/local/fsrvp_state.c  */

struct torture_suite *torture_local_fsrvp(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/local/smbtorture_fullname.c  */

struct torture_suite *torture_local_smbtorture(TALLOC_CTX *ctx);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_TORTURE_LOCAL_PROTO_H__ */

