/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/dsbackup.h"

#ifndef _HEADER_NDR_ad_backup
#define _HEADER_NDR_ad_backup

#define NDR_AD_BACKUP_UUID "ecec0d70-a603-11d0-96b1-00a0c91ece30"
#define NDR_AD_BACKUP_VERSION 1.0
#define NDR_AD_BACKUP_NAME "ad_backup"
#define NDR_AD_BACKUP_HELPSTRING "Backup support for Active Directory"
extern const struct ndr_interface_table ndr_table_ad_backup;
#define NDR_HRRBACKUPPREPARE (0x00)

#define NDR_HRRBACKUPEND (0x01)

#define NDR_HRRBACKUPGETATTACHMENTINFORMATION (0x02)

#define NDR_HRRBACKUPOPENFILE (0x03)

#define NDR_HRRBACKUPREAD (0x04)

#define NDR_HRRBACKUPCLOSE (0x05)

#define NDR_HRRBACKUPGETBACKUPLOGS (0x06)

#define NDR_HRRBACKUPTRUNCATELOGS (0x07)

#define NDR_HRRBACKUPPING (0x08)

#define NDR_AD_BACKUP_CALL_COUNT (9)
void ndr_print_HrRBackupPrepare(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupPrepare *r);
void ndr_print_HrRBackupEnd(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupEnd *r);
void ndr_print_HrRBackupGetAttachmentInformation(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupGetAttachmentInformation *r);
void ndr_print_HrRBackupOpenFile(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupOpenFile *r);
void ndr_print_HrRBackupRead(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupRead *r);
void ndr_print_HrRBackupClose(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupClose *r);
void ndr_print_HrRBackupGetBackupLogs(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupGetBackupLogs *r);
void ndr_print_HrRBackupTruncateLogs(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupTruncateLogs *r);
void ndr_print_HrRBackupPing(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupPing *r);
#endif /* _HEADER_NDR_ad_backup */
#ifndef _HEADER_NDR_ad_restore
#define _HEADER_NDR_ad_restore

#define NDR_AD_RESTORE_UUID "16e0cf3a-a604-11d0-96b1-00a0c91ece30"
#define NDR_AD_RESTORE_VERSION 1.0
#define NDR_AD_RESTORE_NAME "ad_restore"
#define NDR_AD_RESTORE_HELPSTRING "Restoring Active Directory backups"
extern const struct ndr_interface_table ndr_table_ad_restore;
#define NDR_HRRISNTDSONLINE (0x00)

#define NDR_HRRRESTOREPREPARE (0x01)

#define NDR_HRRRESTOREREGISTER (0x02)

#define NDR_HRRRESTOREREGISTERCOMPLETE (0x03)

#define NDR_HRRRESTOREGETDATABASELOCATIONS (0x04)

#define NDR_HRRRESTOREEND (0x05)

#define NDR_HRRRESTORESETCURRENTLOGNUMBER (0x06)

#define NDR_HRRRESTORECHECKLOGSFORBACKUP (0x07)

#define NDR_AD_RESTORE_CALL_COUNT (8)
void ndr_print_HrRIsNTDSOnline(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRIsNTDSOnline *r);
void ndr_print_HrRRestorePrepare(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestorePrepare *r);
void ndr_print_HrRRestoreRegister(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreRegister *r);
void ndr_print_HrRRestoreRegisterComplete(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreRegisterComplete *r);
void ndr_print_HrRRestoreGetDatabaseLocations(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreGetDatabaseLocations *r);
void ndr_print_HrRRestoreEnd(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreEnd *r);
void ndr_print_HrRRestoreSetCurrentLogNumber(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreSetCurrentLogNumber *r);
void ndr_print_HrRRestoreCheckLogsForBackup(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreCheckLogsForBackup *r);
#endif /* _HEADER_NDR_ad_restore */
