#ifndef __SOURCE4_LIB_REGISTRY_TESTS_PROTO_H__
#define __SOURCE4_LIB_REGISTRY_TESTS_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../source4/lib/registry/tests/generic.c  */

struct torture_suite *torture_registry(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/lib/registry/tests/hive.c  */

struct torture_suite *torture_registry_hive(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/lib/registry/tests/diff.c  */

struct torture_suite *torture_registry_diff(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/lib/registry/tests/registry.c  */


/**
 * Test listing values
 */
struct torture_suite *torture_registry_registry(TALLOC_CTX *mem_ctx);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_LIB_REGISTRY_TESTS_PROTO_H__ */

