/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_trkwks
#define _PIDL_HEADER_trkwks

#include <stdint.h>


#ifndef _HEADER_trkwks
#define _HEADER_trkwks


struct trkwks_Unknown0 {
	struct {
		WERROR result;
	} out;

};

#endif /* _HEADER_trkwks */
#endif /* _PIDL_HEADER_trkwks */
