
/* Python wrapper functions auto-generated by pidl */
#define PY_SSIZE_T_CLEAN 1 /* We use Py_ssize_t for PyArg_ParseTupleAndKeywords */
#include "lib/replace/system/python.h"
#include "python/py3compat.h"
#include "includes.h"
#include "python/modules.h"
#include <pytalloc.h>
#include "librpc/rpc/pyrpc.h"
#include "librpc/rpc/pyrpc_util.h"
#include "bin/default/librpc/gen_ndr/ndr_windows_event_ids.h"


/*
 * Suppress compiler warnings if the generated code does not call these
 * functions
 */
#ifndef _MAYBE_UNUSED_
#ifdef __has_attribute
#if __has_attribute(unused)
#define _MAYBE_UNUSED_ __attribute__ ((unused))
#else
#define _MAYBE_UNUSED_
#endif
#endif
#endif
/*
 * These functions are here to ensure they can be optimized out by
 * the compiler based on the constant input values
 */

static inline unsigned long long ndr_sizeof2uintmax(size_t var_size)
{
	switch (var_size) {
	case 8:
		return UINT64_MAX;
	case 4:
		return UINT32_MAX;
	case 2:
		return UINT16_MAX;
	case 1:
		return UINT8_MAX;
	}

	return 0;
}

static inline _MAYBE_UNUSED_ long long ndr_sizeof2intmax(size_t var_size)
{
	switch (var_size) {
	case 8:
		return INT64_MAX;
	case 4:
		return INT32_MAX;
	case 2:
		return INT16_MAX;
	case 1:
		return INT8_MAX;
	}

	return 0;
}


static PyMethodDef windows_event_ids_methods[] = {
	{ NULL, NULL, 0, NULL }
};

static struct PyModuleDef moduledef = {
	PyModuleDef_HEAD_INIT,
	.m_name = "windows_event_ids",
	.m_doc = "windows_event_ids DCE/RPC",
	.m_size = -1,
	.m_methods = windows_event_ids_methods,
};
MODULE_INIT_FUNC(windows_event_ids)
{
	PyObject *m = NULL;


	m = PyModule_Create(&moduledef);
	if (m == NULL)
		goto out;

	PyModule_AddObject(m, "EVT_ID_NONE", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_NONE)));
	PyModule_AddObject(m, "EVT_ID_SUCCESSFUL_LOGON", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_SUCCESSFUL_LOGON)));
	PyModule_AddObject(m, "EVT_ID_UNSUCCESSFUL_LOGON", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_UNSUCCESSFUL_LOGON)));
	PyModule_AddObject(m, "EVT_ID_PASSWORD_CHANGE", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_PASSWORD_CHANGE)));
	PyModule_AddObject(m, "EVT_ID_PASSWORD_RESET", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_PASSWORD_RESET)));
	PyModule_AddObject(m, "EVT_ID_USER_ADDED_TO_GLOBAL_SEC_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_ADDED_TO_GLOBAL_SEC_GROUP)));
	PyModule_AddObject(m, "EVT_ID_USER_REMOVED_FROM_GLOBAL_SEC_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_REMOVED_FROM_GLOBAL_SEC_GROUP)));
	PyModule_AddObject(m, "EVT_ID_USER_ADDED_TO_LOCAL_SEC_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_ADDED_TO_LOCAL_SEC_GROUP)));
	PyModule_AddObject(m, "EVT_ID_USER_REMOVED_FROM_LOCAL_SEC_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_REMOVED_FROM_LOCAL_SEC_GROUP)));
	PyModule_AddObject(m, "EVT_ID_USER_ADDED_TO_LOCAL_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_ADDED_TO_LOCAL_GROUP)));
	PyModule_AddObject(m, "EVT_ID_USER_REMOVED_FROM_LOCAL_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_REMOVED_FROM_LOCAL_GROUP)));
	PyModule_AddObject(m, "EVT_ID_USER_ADDED_TO_GLOBAL_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_ADDED_TO_GLOBAL_GROUP)));
	PyModule_AddObject(m, "EVT_ID_USER_REMOVED_FROM_GLOBAL_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_REMOVED_FROM_GLOBAL_GROUP)));
	PyModule_AddObject(m, "EVT_ID_USER_ADDED_TO_UNIVERSAL_SEC_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_ADDED_TO_UNIVERSAL_SEC_GROUP)));
	PyModule_AddObject(m, "EVT_ID_USER_REMOVED_FROM_UNIVERSAL_SEC_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_REMOVED_FROM_UNIVERSAL_SEC_GROUP)));
	PyModule_AddObject(m, "EVT_ID_USER_ADDED_TO_UNIVERSAL_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_ADDED_TO_UNIVERSAL_GROUP)));
	PyModule_AddObject(m, "EVT_ID_USER_REMOVED_FROM_UNIVERSAL_GROUP", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_USER_REMOVED_FROM_UNIVERSAL_GROUP)));
	PyModule_AddObject(m, "EVT_ID_DIRECTORY_OBJECT_CHANGE", PyLong_FromUnsignedLongLong((uint32_t)(EVT_ID_DIRECTORY_OBJECT_CHANGE)));
	PyModule_AddObject(m, "AUTH_EVT_ID_NONE", PyLong_FromUnsignedLongLong((uint32_t)(AUTH_EVT_ID_NONE)));
	PyModule_AddObject(m, "AUTH_EVT_ID_NTLM_DEVICE_RESTRICTION", PyLong_FromUnsignedLongLong((uint32_t)(AUTH_EVT_ID_NTLM_DEVICE_RESTRICTION)));
	PyModule_AddObject(m, "AUTH_EVT_ID_KERBEROS_DEVICE_RESTRICTION", PyLong_FromUnsignedLongLong((uint32_t)(AUTH_EVT_ID_KERBEROS_DEVICE_RESTRICTION)));
	PyModule_AddObject(m, "AUTH_EVT_ID_KERBEROS_DEVICE_RESTRICTION_AUDIT", PyLong_FromUnsignedLongLong((uint32_t)(AUTH_EVT_ID_KERBEROS_DEVICE_RESTRICTION_AUDIT)));
	PyModule_AddObject(m, "AUTH_EVT_ID_KERBEROS_SERVER_RESTRICTION", PyLong_FromUnsignedLongLong((uint32_t)(AUTH_EVT_ID_KERBEROS_SERVER_RESTRICTION)));
	PyModule_AddObject(m, "AUTH_EVT_ID_KERBEROS_SERVER_RESTRICTION_AUDIT", PyLong_FromUnsignedLongLong((uint32_t)(AUTH_EVT_ID_KERBEROS_SERVER_RESTRICTION_AUDIT)));
	PyModule_AddObject(m, "EVT_LOGON_INTERACTIVE", PyLong_FromUnsignedLongLong((uint32_t)(EVT_LOGON_INTERACTIVE)));
	PyModule_AddObject(m, "EVT_LOGON_NETWORK", PyLong_FromUnsignedLongLong((uint32_t)(EVT_LOGON_NETWORK)));
	PyModule_AddObject(m, "EVT_LOGON_BATCH", PyLong_FromUnsignedLongLong((uint32_t)(EVT_LOGON_BATCH)));
	PyModule_AddObject(m, "EVT_LOGON_SERVICE", PyLong_FromUnsignedLongLong((uint32_t)(EVT_LOGON_SERVICE)));
	PyModule_AddObject(m, "EVT_LOGON_UNLOCK", PyLong_FromUnsignedLongLong((uint32_t)(EVT_LOGON_UNLOCK)));
	PyModule_AddObject(m, "EVT_LOGON_NETWORK_CLEAR_TEXT", PyLong_FromUnsignedLongLong((uint32_t)(EVT_LOGON_NETWORK_CLEAR_TEXT)));
	PyModule_AddObject(m, "EVT_LOGON_NEW_CREDENTIALS", PyLong_FromUnsignedLongLong((uint32_t)(EVT_LOGON_NEW_CREDENTIALS)));
	PyModule_AddObject(m, "EVT_LOGON_REMOTE_INTERACTIVE", PyLong_FromUnsignedLongLong((uint32_t)(EVT_LOGON_REMOTE_INTERACTIVE)));
	PyModule_AddObject(m, "EVT_LOGON_CACHED_INTERACTIVE", PyLong_FromUnsignedLongLong((uint32_t)(EVT_LOGON_CACHED_INTERACTIVE)));
#ifdef PY_MOD_WINDOWS_EVENT_IDS_PATCH
	PY_MOD_WINDOWS_EVENT_IDS_PATCH(m);
#endif
	out:
	return m;

}
