#ifndef __NDR_SPOOLSS_SCOMPAT_H__
#define __NDR_SPOOLSS_SCOMPAT_H__

struct pipes_struct;
struct dcesrv_endpoint_server;
struct dcesrv_call_state;

const struct dcesrv_endpoint_server *spoolss_get_ep_server(void);

NTSTATUS spoolss__op_ndr_pull(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_pull *pull, void **r);
struct spoolss_EnumPrinters;
WERROR _spoolss_EnumPrinters(struct pipes_struct *p, struct spoolss_EnumPrinters *r);
struct spoolss_OpenPrinter;
WERROR _spoolss_OpenPrinter(struct pipes_struct *p, struct spoolss_OpenPrinter *r);
struct spoolss_SetJob;
WERROR _spoolss_SetJob(struct pipes_struct *p, struct spoolss_SetJob *r);
struct spoolss_GetJob;
WERROR _spoolss_GetJob(struct pipes_struct *p, struct spoolss_GetJob *r);
struct spoolss_EnumJobs;
WERROR _spoolss_EnumJobs(struct pipes_struct *p, struct spoolss_EnumJobs *r);
struct spoolss_AddPrinter;
WERROR _spoolss_AddPrinter(struct pipes_struct *p, struct spoolss_AddPrinter *r);
struct spoolss_DeletePrinter;
WERROR _spoolss_DeletePrinter(struct pipes_struct *p, struct spoolss_DeletePrinter *r);
struct spoolss_SetPrinter;
WERROR _spoolss_SetPrinter(struct pipes_struct *p, struct spoolss_SetPrinter *r);
struct spoolss_GetPrinter;
WERROR _spoolss_GetPrinter(struct pipes_struct *p, struct spoolss_GetPrinter *r);
struct spoolss_AddPrinterDriver;
WERROR _spoolss_AddPrinterDriver(struct pipes_struct *p, struct spoolss_AddPrinterDriver *r);
struct spoolss_EnumPrinterDrivers;
WERROR _spoolss_EnumPrinterDrivers(struct pipes_struct *p, struct spoolss_EnumPrinterDrivers *r);
struct spoolss_GetPrinterDriver;
WERROR _spoolss_GetPrinterDriver(struct pipes_struct *p, struct spoolss_GetPrinterDriver *r);
struct spoolss_GetPrinterDriverDirectory;
WERROR _spoolss_GetPrinterDriverDirectory(struct pipes_struct *p, struct spoolss_GetPrinterDriverDirectory *r);
struct spoolss_DeletePrinterDriver;
WERROR _spoolss_DeletePrinterDriver(struct pipes_struct *p, struct spoolss_DeletePrinterDriver *r);
struct spoolss_AddPrintProcessor;
WERROR _spoolss_AddPrintProcessor(struct pipes_struct *p, struct spoolss_AddPrintProcessor *r);
struct spoolss_EnumPrintProcessors;
WERROR _spoolss_EnumPrintProcessors(struct pipes_struct *p, struct spoolss_EnumPrintProcessors *r);
struct spoolss_GetPrintProcessorDirectory;
WERROR _spoolss_GetPrintProcessorDirectory(struct pipes_struct *p, struct spoolss_GetPrintProcessorDirectory *r);
struct spoolss_StartDocPrinter;
WERROR _spoolss_StartDocPrinter(struct pipes_struct *p, struct spoolss_StartDocPrinter *r);
struct spoolss_StartPagePrinter;
WERROR _spoolss_StartPagePrinter(struct pipes_struct *p, struct spoolss_StartPagePrinter *r);
struct spoolss_WritePrinter;
WERROR _spoolss_WritePrinter(struct pipes_struct *p, struct spoolss_WritePrinter *r);
struct spoolss_EndPagePrinter;
WERROR _spoolss_EndPagePrinter(struct pipes_struct *p, struct spoolss_EndPagePrinter *r);
struct spoolss_AbortPrinter;
WERROR _spoolss_AbortPrinter(struct pipes_struct *p, struct spoolss_AbortPrinter *r);
struct spoolss_ReadPrinter;
WERROR _spoolss_ReadPrinter(struct pipes_struct *p, struct spoolss_ReadPrinter *r);
struct spoolss_EndDocPrinter;
WERROR _spoolss_EndDocPrinter(struct pipes_struct *p, struct spoolss_EndDocPrinter *r);
struct spoolss_AddJob;
WERROR _spoolss_AddJob(struct pipes_struct *p, struct spoolss_AddJob *r);
struct spoolss_ScheduleJob;
WERROR _spoolss_ScheduleJob(struct pipes_struct *p, struct spoolss_ScheduleJob *r);
struct spoolss_GetPrinterData;
WERROR _spoolss_GetPrinterData(struct pipes_struct *p, struct spoolss_GetPrinterData *r);
struct spoolss_SetPrinterData;
WERROR _spoolss_SetPrinterData(struct pipes_struct *p, struct spoolss_SetPrinterData *r);
struct spoolss_WaitForPrinterChange;
WERROR _spoolss_WaitForPrinterChange(struct pipes_struct *p, struct spoolss_WaitForPrinterChange *r);
struct spoolss_ClosePrinter;
WERROR _spoolss_ClosePrinter(struct pipes_struct *p, struct spoolss_ClosePrinter *r);
struct spoolss_AddForm;
WERROR _spoolss_AddForm(struct pipes_struct *p, struct spoolss_AddForm *r);
struct spoolss_DeleteForm;
WERROR _spoolss_DeleteForm(struct pipes_struct *p, struct spoolss_DeleteForm *r);
struct spoolss_GetForm;
WERROR _spoolss_GetForm(struct pipes_struct *p, struct spoolss_GetForm *r);
struct spoolss_SetForm;
WERROR _spoolss_SetForm(struct pipes_struct *p, struct spoolss_SetForm *r);
struct spoolss_EnumForms;
WERROR _spoolss_EnumForms(struct pipes_struct *p, struct spoolss_EnumForms *r);
struct spoolss_EnumPorts;
WERROR _spoolss_EnumPorts(struct pipes_struct *p, struct spoolss_EnumPorts *r);
struct spoolss_EnumMonitors;
WERROR _spoolss_EnumMonitors(struct pipes_struct *p, struct spoolss_EnumMonitors *r);
struct spoolss_AddPort;
WERROR _spoolss_AddPort(struct pipes_struct *p, struct spoolss_AddPort *r);
struct spoolss_ConfigurePort;
WERROR _spoolss_ConfigurePort(struct pipes_struct *p, struct spoolss_ConfigurePort *r);
struct spoolss_DeletePort;
WERROR _spoolss_DeletePort(struct pipes_struct *p, struct spoolss_DeletePort *r);
struct spoolss_CreatePrinterIC;
WERROR _spoolss_CreatePrinterIC(struct pipes_struct *p, struct spoolss_CreatePrinterIC *r);
struct spoolss_PlayGDIScriptOnPrinterIC;
WERROR _spoolss_PlayGDIScriptOnPrinterIC(struct pipes_struct *p, struct spoolss_PlayGDIScriptOnPrinterIC *r);
struct spoolss_DeletePrinterIC;
WERROR _spoolss_DeletePrinterIC(struct pipes_struct *p, struct spoolss_DeletePrinterIC *r);
struct spoolss_AddPrinterConnection;
WERROR _spoolss_AddPrinterConnection(struct pipes_struct *p, struct spoolss_AddPrinterConnection *r);
struct spoolss_DeletePrinterConnection;
WERROR _spoolss_DeletePrinterConnection(struct pipes_struct *p, struct spoolss_DeletePrinterConnection *r);
struct spoolss_PrinterMessageBox;
WERROR _spoolss_PrinterMessageBox(struct pipes_struct *p, struct spoolss_PrinterMessageBox *r);
struct spoolss_AddMonitor;
WERROR _spoolss_AddMonitor(struct pipes_struct *p, struct spoolss_AddMonitor *r);
struct spoolss_DeleteMonitor;
WERROR _spoolss_DeleteMonitor(struct pipes_struct *p, struct spoolss_DeleteMonitor *r);
struct spoolss_DeletePrintProcessor;
WERROR _spoolss_DeletePrintProcessor(struct pipes_struct *p, struct spoolss_DeletePrintProcessor *r);
struct spoolss_AddPrintProvidor;
WERROR _spoolss_AddPrintProvidor(struct pipes_struct *p, struct spoolss_AddPrintProvidor *r);
struct spoolss_DeletePrintProvidor;
WERROR _spoolss_DeletePrintProvidor(struct pipes_struct *p, struct spoolss_DeletePrintProvidor *r);
struct spoolss_EnumPrintProcessorDataTypes;
WERROR _spoolss_EnumPrintProcessorDataTypes(struct pipes_struct *p, struct spoolss_EnumPrintProcessorDataTypes *r);
struct spoolss_ResetPrinter;
WERROR _spoolss_ResetPrinter(struct pipes_struct *p, struct spoolss_ResetPrinter *r);
struct spoolss_GetPrinterDriver2;
WERROR _spoolss_GetPrinterDriver2(struct pipes_struct *p, struct spoolss_GetPrinterDriver2 *r);
struct spoolss_FindFirstPrinterChangeNotification;
WERROR _spoolss_FindFirstPrinterChangeNotification(struct pipes_struct *p, struct spoolss_FindFirstPrinterChangeNotification *r);
struct spoolss_FindNextPrinterChangeNotification;
WERROR _spoolss_FindNextPrinterChangeNotification(struct pipes_struct *p, struct spoolss_FindNextPrinterChangeNotification *r);
struct spoolss_FindClosePrinterNotify;
WERROR _spoolss_FindClosePrinterNotify(struct pipes_struct *p, struct spoolss_FindClosePrinterNotify *r);
struct spoolss_RouterFindFirstPrinterChangeNotificationOld;
WERROR _spoolss_RouterFindFirstPrinterChangeNotificationOld(struct pipes_struct *p, struct spoolss_RouterFindFirstPrinterChangeNotificationOld *r);
struct spoolss_ReplyOpenPrinter;
WERROR _spoolss_ReplyOpenPrinter(struct pipes_struct *p, struct spoolss_ReplyOpenPrinter *r);
struct spoolss_RouterReplyPrinter;
WERROR _spoolss_RouterReplyPrinter(struct pipes_struct *p, struct spoolss_RouterReplyPrinter *r);
struct spoolss_ReplyClosePrinter;
WERROR _spoolss_ReplyClosePrinter(struct pipes_struct *p, struct spoolss_ReplyClosePrinter *r);
struct spoolss_AddPortEx;
WERROR _spoolss_AddPortEx(struct pipes_struct *p, struct spoolss_AddPortEx *r);
struct spoolss_RouterFindFirstPrinterChangeNotification;
WERROR _spoolss_RouterFindFirstPrinterChangeNotification(struct pipes_struct *p, struct spoolss_RouterFindFirstPrinterChangeNotification *r);
struct spoolss_SpoolerInit;
WERROR _spoolss_SpoolerInit(struct pipes_struct *p, struct spoolss_SpoolerInit *r);
struct spoolss_ResetPrinterEx;
WERROR _spoolss_ResetPrinterEx(struct pipes_struct *p, struct spoolss_ResetPrinterEx *r);
struct spoolss_RemoteFindFirstPrinterChangeNotifyEx;
WERROR _spoolss_RemoteFindFirstPrinterChangeNotifyEx(struct pipes_struct *p, struct spoolss_RemoteFindFirstPrinterChangeNotifyEx *r);
struct spoolss_RouterReplyPrinterEx;
WERROR _spoolss_RouterReplyPrinterEx(struct pipes_struct *p, struct spoolss_RouterReplyPrinterEx *r);
struct spoolss_RouterRefreshPrinterChangeNotify;
WERROR _spoolss_RouterRefreshPrinterChangeNotify(struct pipes_struct *p, struct spoolss_RouterRefreshPrinterChangeNotify *r);
struct spoolss_44;
WERROR _spoolss_44(struct pipes_struct *p, struct spoolss_44 *r);
struct spoolss_OpenPrinterEx;
WERROR _spoolss_OpenPrinterEx(struct pipes_struct *p, struct spoolss_OpenPrinterEx *r);
struct spoolss_AddPrinterEx;
WERROR _spoolss_AddPrinterEx(struct pipes_struct *p, struct spoolss_AddPrinterEx *r);
struct spoolss_SetPort;
WERROR _spoolss_SetPort(struct pipes_struct *p, struct spoolss_SetPort *r);
struct spoolss_EnumPrinterData;
WERROR _spoolss_EnumPrinterData(struct pipes_struct *p, struct spoolss_EnumPrinterData *r);
struct spoolss_DeletePrinterData;
WERROR _spoolss_DeletePrinterData(struct pipes_struct *p, struct spoolss_DeletePrinterData *r);
struct spoolss_4a;
WERROR _spoolss_4a(struct pipes_struct *p, struct spoolss_4a *r);
struct spoolss_4b;
WERROR _spoolss_4b(struct pipes_struct *p, struct spoolss_4b *r);
struct spoolss_4c;
WERROR _spoolss_4c(struct pipes_struct *p, struct spoolss_4c *r);
struct spoolss_SetPrinterDataEx;
WERROR _spoolss_SetPrinterDataEx(struct pipes_struct *p, struct spoolss_SetPrinterDataEx *r);
struct spoolss_GetPrinterDataEx;
WERROR _spoolss_GetPrinterDataEx(struct pipes_struct *p, struct spoolss_GetPrinterDataEx *r);
struct spoolss_EnumPrinterDataEx;
WERROR _spoolss_EnumPrinterDataEx(struct pipes_struct *p, struct spoolss_EnumPrinterDataEx *r);
struct spoolss_EnumPrinterKey;
WERROR _spoolss_EnumPrinterKey(struct pipes_struct *p, struct spoolss_EnumPrinterKey *r);
struct spoolss_DeletePrinterDataEx;
WERROR _spoolss_DeletePrinterDataEx(struct pipes_struct *p, struct spoolss_DeletePrinterDataEx *r);
struct spoolss_DeletePrinterKey;
WERROR _spoolss_DeletePrinterKey(struct pipes_struct *p, struct spoolss_DeletePrinterKey *r);
struct spoolss_53;
WERROR _spoolss_53(struct pipes_struct *p, struct spoolss_53 *r);
struct spoolss_DeletePrinterDriverEx;
WERROR _spoolss_DeletePrinterDriverEx(struct pipes_struct *p, struct spoolss_DeletePrinterDriverEx *r);
struct spoolss_AddPerMachineConnection;
WERROR _spoolss_AddPerMachineConnection(struct pipes_struct *p, struct spoolss_AddPerMachineConnection *r);
struct spoolss_DeletePerMachineConnection;
WERROR _spoolss_DeletePerMachineConnection(struct pipes_struct *p, struct spoolss_DeletePerMachineConnection *r);
struct spoolss_EnumPerMachineConnections;
WERROR _spoolss_EnumPerMachineConnections(struct pipes_struct *p, struct spoolss_EnumPerMachineConnections *r);
struct spoolss_XcvData;
WERROR _spoolss_XcvData(struct pipes_struct *p, struct spoolss_XcvData *r);
struct spoolss_AddPrinterDriverEx;
WERROR _spoolss_AddPrinterDriverEx(struct pipes_struct *p, struct spoolss_AddPrinterDriverEx *r);
struct spoolss_5a;
WERROR _spoolss_5a(struct pipes_struct *p, struct spoolss_5a *r);
struct spoolss_5b;
WERROR _spoolss_5b(struct pipes_struct *p, struct spoolss_5b *r);
struct spoolss_5c;
WERROR _spoolss_5c(struct pipes_struct *p, struct spoolss_5c *r);
struct spoolss_5d;
WERROR _spoolss_5d(struct pipes_struct *p, struct spoolss_5d *r);
struct spoolss_5e;
WERROR _spoolss_5e(struct pipes_struct *p, struct spoolss_5e *r);
struct spoolss_5f;
WERROR _spoolss_5f(struct pipes_struct *p, struct spoolss_5f *r);
struct spoolss_60;
WERROR _spoolss_60(struct pipes_struct *p, struct spoolss_60 *r);
struct spoolss_SendRecvBidiData;
WERROR _spoolss_SendRecvBidiData(struct pipes_struct *p, struct spoolss_SendRecvBidiData *r);
struct spoolss_62;
WERROR _spoolss_62(struct pipes_struct *p, struct spoolss_62 *r);
struct spoolss_63;
WERROR _spoolss_63(struct pipes_struct *p, struct spoolss_63 *r);
struct spoolss_64;
WERROR _spoolss_64(struct pipes_struct *p, struct spoolss_64 *r);
struct spoolss_65;
WERROR _spoolss_65(struct pipes_struct *p, struct spoolss_65 *r);
struct spoolss_GetCorePrinterDrivers;
HRESULT _spoolss_GetCorePrinterDrivers(struct pipes_struct *p, struct spoolss_GetCorePrinterDrivers *r);
struct spoolss_67;
WERROR _spoolss_67(struct pipes_struct *p, struct spoolss_67 *r);
struct spoolss_GetPrinterDriverPackagePath;
HRESULT _spoolss_GetPrinterDriverPackagePath(struct pipes_struct *p, struct spoolss_GetPrinterDriverPackagePath *r);
struct spoolss_69;
WERROR _spoolss_69(struct pipes_struct *p, struct spoolss_69 *r);
struct spoolss_6a;
WERROR _spoolss_6a(struct pipes_struct *p, struct spoolss_6a *r);
struct spoolss_6b;
WERROR _spoolss_6b(struct pipes_struct *p, struct spoolss_6b *r);
struct spoolss_6c;
WERROR _spoolss_6c(struct pipes_struct *p, struct spoolss_6c *r);
struct spoolss_6d;
WERROR _spoolss_6d(struct pipes_struct *p, struct spoolss_6d *r);
struct spoolss_GetJobNamedPropertyValue;
WERROR _spoolss_GetJobNamedPropertyValue(struct pipes_struct *p, struct spoolss_GetJobNamedPropertyValue *r);
struct spoolss_SetJobNamedProperty;
WERROR _spoolss_SetJobNamedProperty(struct pipes_struct *p, struct spoolss_SetJobNamedProperty *r);
struct spoolss_DeleteJobNamedProperty;
WERROR _spoolss_DeleteJobNamedProperty(struct pipes_struct *p, struct spoolss_DeleteJobNamedProperty *r);
struct spoolss_EnumJobNamedProperties;
WERROR _spoolss_EnumJobNamedProperties(struct pipes_struct *p, struct spoolss_EnumJobNamedProperties *r);
struct spoolss_72;
WERROR _spoolss_72(struct pipes_struct *p, struct spoolss_72 *r);
struct spoolss_73;
WERROR _spoolss_73(struct pipes_struct *p, struct spoolss_73 *r);
struct spoolss_LogJobInfoForBranchOffice;
WERROR _spoolss_LogJobInfoForBranchOffice(struct pipes_struct *p, struct spoolss_LogJobInfoForBranchOffice *r);
NTSTATUS spoolss__op_dispatch(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS spoolss__op_reply(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS spoolss__op_ndr_push(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_push *push, const void *r);
NTSTATUS spoolss__op_local(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
#endif /* __NDR_SPOOLSS_SCOMPAT_H__ */
