/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/bcrypt_rsakey_blob.h"

#ifndef _HEADER_NDR_bcrypt_rsakey_blob
#define _HEADER_NDR_bcrypt_rsakey_blob

extern const struct ndr_interface_table ndr_table_bcrypt_rsakey_blob;
#define NDR_BCRYPT_RSAKEY_BLOB_CALL_COUNT (0)
enum ndr_err_code ndr_push_BCRYPT_RSAPUBLIC_BLOB(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct BCRYPT_RSAPUBLIC_BLOB *r);
enum ndr_err_code ndr_pull_BCRYPT_RSAPUBLIC_BLOB(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct BCRYPT_RSAPUBLIC_BLOB *r);
void ndr_print_BCRYPT_RSAPUBLIC_BLOB(struct ndr_print *ndr, const char *name, const struct BCRYPT_RSAPUBLIC_BLOB *r);
#endif /* _HEADER_NDR_bcrypt_rsakey_blob */
