/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/dssetup.h"

#ifndef _HEADER_NDR_dssetup
#define _HEADER_NDR_dssetup

#define NDR_DSSETUP_UUID "3919286a-b10c-11d0-9ba8-00c04fd92ef5"
#define NDR_DSSETUP_VERSION 0.0
#define NDR_DSSETUP_NAME "dssetup"
#define NDR_DSSETUP_HELPSTRING "Active Directory Setup"
extern const struct ndr_interface_table ndr_table_dssetup;
#define NDR_DSSETUP_DSROLEGETPRIMARYDOMAININFORMATION (0x00)

#define NDR_DSSETUP_DSROLEDNSNAMETOFLATNAME (0x01)

#define NDR_DSSETUP_DSROLEDCASDC (0x02)

#define NDR_DSSETUP_DSROLEDCASREPLICA (0x03)

#define NDR_DSSETUP_DSROLEDEMOTEDC (0x04)

#define NDR_DSSETUP_DSROLEGETDCOPERATIONPROGRESS (0x05)

#define NDR_DSSETUP_DSROLEGETDCOPERATIONRESULTS (0x06)

#define NDR_DSSETUP_DSROLECANCEL (0x07)

#define NDR_DSSETUP_DSROLESERVERSAVESTATEFORUPGRADE (0x08)

#define NDR_DSSETUP_DSROLEUPGRADEDOWNLEVELSERVER (0x09)

#define NDR_DSSETUP_DSROLEABORTDOWNLEVELSERVERUPGRADE (0x0a)

#define NDR_DSSETUP_CALL_COUNT (11)
void ndr_print_dssetup_DsRole(struct ndr_print *ndr, const char *name, enum dssetup_DsRole r);
void ndr_print_dssetup_DsRoleFlags(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_dssetup_DsRolePrimaryDomInfoBasic(struct ndr_print *ndr, const char *name, const struct dssetup_DsRolePrimaryDomInfoBasic *r);
void ndr_print_dssetup_DsUpgrade(struct ndr_print *ndr, const char *name, enum dssetup_DsUpgrade r);
void ndr_print_dssetup_DsPrevious(struct ndr_print *ndr, const char *name, enum dssetup_DsPrevious r);
void ndr_print_dssetup_DsRoleUpgradeStatus(struct ndr_print *ndr, const char *name, const struct dssetup_DsRoleUpgradeStatus *r);
void ndr_print_dssetup_DsRoleOp(struct ndr_print *ndr, const char *name, enum dssetup_DsRoleOp r);
void ndr_print_dssetup_DsRoleOpStatus(struct ndr_print *ndr, const char *name, const struct dssetup_DsRoleOpStatus *r);
void ndr_print_dssetup_DsRoleInfoLevel(struct ndr_print *ndr, const char *name, enum dssetup_DsRoleInfoLevel r);
void ndr_print_dssetup_DsRoleInfo(struct ndr_print *ndr, const char *name, const union dssetup_DsRoleInfo *r);
void ndr_print_dssetup_DsRoleGetPrimaryDomainInformation(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dssetup_DsRoleGetPrimaryDomainInformation *r);
void ndr_print_dssetup_DsRoleDnsNameToFlatName(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dssetup_DsRoleDnsNameToFlatName *r);
void ndr_print_dssetup_DsRoleDcAsDc(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dssetup_DsRoleDcAsDc *r);
void ndr_print_dssetup_DsRoleDcAsReplica(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dssetup_DsRoleDcAsReplica *r);
void ndr_print_dssetup_DsRoleDemoteDc(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dssetup_DsRoleDemoteDc *r);
void ndr_print_dssetup_DsRoleGetDcOperationProgress(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dssetup_DsRoleGetDcOperationProgress *r);
void ndr_print_dssetup_DsRoleGetDcOperationResults(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dssetup_DsRoleGetDcOperationResults *r);
void ndr_print_dssetup_DsRoleCancel(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dssetup_DsRoleCancel *r);
void ndr_print_dssetup_DsRoleServerSaveStateForUpgrade(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dssetup_DsRoleServerSaveStateForUpgrade *r);
void ndr_print_dssetup_DsRoleUpgradeDownlevelServer(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dssetup_DsRoleUpgradeDownlevelServer *r);
void ndr_print_dssetup_DsRoleAbortDownlevelServerUpgrade(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dssetup_DsRoleAbortDownlevelServerUpgrade *r);
#endif /* _HEADER_NDR_dssetup */
