#ifndef __SOURCE4_TORTURE_NDR_PROTO_H__
#define __SOURCE4_TORTURE_NDR_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../source4/torture/ndr/ndr.c  */

struct torture_suite *torture_local_ndr(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/ndr/dcerpc.c  */

struct torture_suite *ndr_dcerpc_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/winreg.c  */

struct torture_suite *ndr_winreg_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/atsvc.c  */

struct torture_suite *ndr_atsvc_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/lsa.c  */

struct torture_suite *ndr_lsa_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/epmap.c  */

struct torture_suite *ndr_epmap_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/dfs.c  */

struct torture_suite *ndr_dfs_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/netlogon.c  */

struct torture_suite *ndr_netlogon_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/drsuapi.c  */

struct torture_suite *ndr_drsuapi_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/spoolss.c  */

struct torture_suite *ndr_spoolss_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/ntprinting.c  */

struct torture_suite *ndr_ntprinting_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/samr.c  */

struct torture_suite *ndr_samr_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/dfsblob.c  */

struct torture_suite *ndr_dfsblob_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/drsblobs.c  */

struct torture_suite *ndr_drsblobs_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/dnsp.c  */

struct torture_suite *ndr_dnsp_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/nbt.c  */

struct torture_suite *ndr_nbt_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/ntlmssp.c  */

struct torture_suite *ndr_ntlmssp_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/string.c  */

struct torture_suite *ndr_string_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/backupkey.c  */

struct torture_suite *ndr_backupkey_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/witness.c  */

struct torture_suite *ndr_witness_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/clusapi.c  */

struct torture_suite *ndr_clusapi_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/negoex.c  */

struct torture_suite *ndr_negoex_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/krb5pac.c  */

struct torture_suite *ndr_krb5pac_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/winspool.c  */

struct torture_suite *ndr_winspool_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/cabinet.c  */

struct torture_suite *ndr_cabinet_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/charset.c  */

struct torture_suite *ndr_charset_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/svcctl.c  */

struct torture_suite *ndr_svcctl_suite(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/ndr/odj.c  */

struct torture_suite *ndr_ODJ_suite(TALLOC_CTX *ctx);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_TORTURE_NDR_PROTO_H__ */

