/* parser auto-generated by pidl */

#include "replace.h"
#include "bin/default/librpc/gen_ndr/ndr_wzcsvc.h"

static enum ndr_err_code ndr_push_wzcsvc_EnumInterfaces(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_EnumInterfaces *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_EnumInterfaces(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_EnumInterfaces *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_EnumInterfaces(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_EnumInterfaces *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_EnumInterfaces");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_EnumInterfaces");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_EnumInterfaces");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_QueryInterface(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_QueryInterface *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_QueryInterface(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_QueryInterface *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_QueryInterface(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_QueryInterface *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_QueryInterface");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_QueryInterface");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_QueryInterface");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_SetInterface(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_SetInterface *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_SetInterface(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_SetInterface *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_SetInterface(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_SetInterface *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_SetInterface");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_SetInterface");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_SetInterface");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_RefreshInterface(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_RefreshInterface *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_RefreshInterface(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_RefreshInterface *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_RefreshInterface(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_RefreshInterface *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_RefreshInterface");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_RefreshInterface");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_RefreshInterface");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_QueryContext(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_QueryContext *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_QueryContext(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_QueryContext *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_QueryContext(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_QueryContext *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_QueryContext");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_QueryContext");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_QueryContext");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_SetContext(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_SetContext *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_SetContext(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_SetContext *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_SetContext(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_SetContext *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_SetContext");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_SetContext");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_SetContext");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_EapolUIResponse(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_EapolUIResponse *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_EapolUIResponse(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_EapolUIResponse *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_EapolUIResponse(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_EapolUIResponse *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_EapolUIResponse");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_EapolUIResponse");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_EapolUIResponse");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_EapolGetCustomAuthData(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_EapolGetCustomAuthData *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_EapolGetCustomAuthData(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_EapolGetCustomAuthData *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_EapolGetCustomAuthData(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_EapolGetCustomAuthData *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_EapolGetCustomAuthData");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_EapolGetCustomAuthData");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_EapolGetCustomAuthData");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_EapolSetCustomAuthData(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_EapolSetCustomAuthData *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_EapolSetCustomAuthData(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_EapolSetCustomAuthData *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_EapolSetCustomAuthData(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_EapolSetCustomAuthData *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_EapolSetCustomAuthData");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_EapolSetCustomAuthData");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_EapolSetCustomAuthData");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_EapolGetInterfaceParams(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_EapolGetInterfaceParams *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_EapolGetInterfaceParams(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_EapolGetInterfaceParams *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_EapolGetInterfaceParams(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_EapolGetInterfaceParams *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_EapolGetInterfaceParams");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_EapolGetInterfaceParams");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_EapolGetInterfaceParams");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_EapolSetInterfaceParams(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_EapolSetInterfaceParams *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_EapolSetInterfaceParams(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_EapolSetInterfaceParams *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_EapolSetInterfaceParams(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_EapolSetInterfaceParams *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_EapolSetInterfaceParams");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_EapolSetInterfaceParams");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_EapolSetInterfaceParams");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_EapolReAuthenticateInterface(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_EapolReAuthenticateInterface *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_EapolReAuthenticateInterface(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_EapolReAuthenticateInterface *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_EapolReAuthenticateInterface(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_EapolReAuthenticateInterface *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_EapolReAuthenticateInterface");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_EapolReAuthenticateInterface");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_EapolReAuthenticateInterface");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_EapolQueryInterfaceState(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_EapolQueryInterfaceState *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_EapolQueryInterfaceState(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_EapolQueryInterfaceState *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_EapolQueryInterfaceState(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_EapolQueryInterfaceState *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_EapolQueryInterfaceState");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_EapolQueryInterfaceState");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_EapolQueryInterfaceState");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_OpenWZCDbLogSession(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_OpenWZCDbLogSession *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_OpenWZCDbLogSession(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_OpenWZCDbLogSession *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_OpenWZCDbLogSession(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_OpenWZCDbLogSession *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_OpenWZCDbLogSession");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_OpenWZCDbLogSession");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_OpenWZCDbLogSession");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_CloseWZCDbLogSession(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_CloseWZCDbLogSession *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_CloseWZCDbLogSession(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_CloseWZCDbLogSession *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_CloseWZCDbLogSession(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_CloseWZCDbLogSession *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_CloseWZCDbLogSession");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_CloseWZCDbLogSession");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_CloseWZCDbLogSession");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_EnumWZCDbLogRecords(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_EnumWZCDbLogRecords *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_EnumWZCDbLogRecords(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_EnumWZCDbLogRecords *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_EnumWZCDbLogRecords(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_EnumWZCDbLogRecords *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_EnumWZCDbLogRecords");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_EnumWZCDbLogRecords");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_EnumWZCDbLogRecords");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_FlushWZCdbLog(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_FlushWZCdbLog *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_FlushWZCdbLog(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_FlushWZCdbLog *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_FlushWZCdbLog(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_FlushWZCdbLog *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_FlushWZCdbLog");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_FlushWZCdbLog");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_FlushWZCdbLog");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_wzcsvc_GetWZCDbLogRecord(struct ndr_push *ndr, ndr_flags_type flags, const struct wzcsvc_GetWZCDbLogRecord *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_wzcsvc_GetWZCDbLogRecord(struct ndr_pull *ndr, ndr_flags_type flags, struct wzcsvc_GetWZCDbLogRecord *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_wzcsvc_GetWZCDbLogRecord(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct wzcsvc_GetWZCDbLogRecord *r)
{
	ndr_print_struct(ndr, name, "wzcsvc_GetWZCDbLogRecord");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "wzcsvc_GetWZCDbLogRecord");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "wzcsvc_GetWZCDbLogRecord");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

#ifndef SKIP_NDR_TABLE_wzcsvc
static const struct ndr_interface_public_struct wzcsvc_public_structs[] = {
	{ .name = NULL }
};

static const struct ndr_interface_call wzcsvc_calls[] = {
	{
		"wzcsvc_EnumInterfaces",
		sizeof(struct wzcsvc_EnumInterfaces),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_EnumInterfaces,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_EnumInterfaces,
		(ndr_print_function_t) ndr_print_wzcsvc_EnumInterfaces,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_QueryInterface",
		sizeof(struct wzcsvc_QueryInterface),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_QueryInterface,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_QueryInterface,
		(ndr_print_function_t) ndr_print_wzcsvc_QueryInterface,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_SetInterface",
		sizeof(struct wzcsvc_SetInterface),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_SetInterface,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_SetInterface,
		(ndr_print_function_t) ndr_print_wzcsvc_SetInterface,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_RefreshInterface",
		sizeof(struct wzcsvc_RefreshInterface),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_RefreshInterface,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_RefreshInterface,
		(ndr_print_function_t) ndr_print_wzcsvc_RefreshInterface,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_QueryContext",
		sizeof(struct wzcsvc_QueryContext),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_QueryContext,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_QueryContext,
		(ndr_print_function_t) ndr_print_wzcsvc_QueryContext,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_SetContext",
		sizeof(struct wzcsvc_SetContext),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_SetContext,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_SetContext,
		(ndr_print_function_t) ndr_print_wzcsvc_SetContext,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_EapolUIResponse",
		sizeof(struct wzcsvc_EapolUIResponse),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_EapolUIResponse,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_EapolUIResponse,
		(ndr_print_function_t) ndr_print_wzcsvc_EapolUIResponse,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_EapolGetCustomAuthData",
		sizeof(struct wzcsvc_EapolGetCustomAuthData),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_EapolGetCustomAuthData,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_EapolGetCustomAuthData,
		(ndr_print_function_t) ndr_print_wzcsvc_EapolGetCustomAuthData,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_EapolSetCustomAuthData",
		sizeof(struct wzcsvc_EapolSetCustomAuthData),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_EapolSetCustomAuthData,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_EapolSetCustomAuthData,
		(ndr_print_function_t) ndr_print_wzcsvc_EapolSetCustomAuthData,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_EapolGetInterfaceParams",
		sizeof(struct wzcsvc_EapolGetInterfaceParams),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_EapolGetInterfaceParams,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_EapolGetInterfaceParams,
		(ndr_print_function_t) ndr_print_wzcsvc_EapolGetInterfaceParams,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_EapolSetInterfaceParams",
		sizeof(struct wzcsvc_EapolSetInterfaceParams),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_EapolSetInterfaceParams,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_EapolSetInterfaceParams,
		(ndr_print_function_t) ndr_print_wzcsvc_EapolSetInterfaceParams,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_EapolReAuthenticateInterface",
		sizeof(struct wzcsvc_EapolReAuthenticateInterface),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_EapolReAuthenticateInterface,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_EapolReAuthenticateInterface,
		(ndr_print_function_t) ndr_print_wzcsvc_EapolReAuthenticateInterface,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_EapolQueryInterfaceState",
		sizeof(struct wzcsvc_EapolQueryInterfaceState),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_EapolQueryInterfaceState,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_EapolQueryInterfaceState,
		(ndr_print_function_t) ndr_print_wzcsvc_EapolQueryInterfaceState,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_OpenWZCDbLogSession",
		sizeof(struct wzcsvc_OpenWZCDbLogSession),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_OpenWZCDbLogSession,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_OpenWZCDbLogSession,
		(ndr_print_function_t) ndr_print_wzcsvc_OpenWZCDbLogSession,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_CloseWZCDbLogSession",
		sizeof(struct wzcsvc_CloseWZCDbLogSession),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_CloseWZCDbLogSession,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_CloseWZCDbLogSession,
		(ndr_print_function_t) ndr_print_wzcsvc_CloseWZCDbLogSession,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_EnumWZCDbLogRecords",
		sizeof(struct wzcsvc_EnumWZCDbLogRecords),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_EnumWZCDbLogRecords,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_EnumWZCDbLogRecords,
		(ndr_print_function_t) ndr_print_wzcsvc_EnumWZCDbLogRecords,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_FlushWZCdbLog",
		sizeof(struct wzcsvc_FlushWZCdbLog),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_FlushWZCdbLog,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_FlushWZCdbLog,
		(ndr_print_function_t) ndr_print_wzcsvc_FlushWZCdbLog,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"wzcsvc_GetWZCDbLogRecord",
		sizeof(struct wzcsvc_GetWZCDbLogRecord),
		(ndr_push_flags_fn_t) ndr_push_wzcsvc_GetWZCDbLogRecord,
		(ndr_pull_flags_fn_t) ndr_pull_wzcsvc_GetWZCDbLogRecord,
		(ndr_print_function_t) ndr_print_wzcsvc_GetWZCDbLogRecord,
		{ 0, NULL },
		{ 0, NULL },
	},
	{ .name = NULL }
};

static const char * const wzcsvc_endpoint_strings[] = {
	"ncacn_np:[\\pipe\\wzcsvc]", 
};

static const struct ndr_interface_string_array wzcsvc_endpoints = {
	.count	= 1,
	.names	= wzcsvc_endpoint_strings
};

static const char * const wzcsvc_authservice_strings[] = {
	"host", 
};

static const struct ndr_interface_string_array wzcsvc_authservices = {
	.count	= 1,
	.names	= wzcsvc_authservice_strings
};


const struct ndr_interface_table ndr_table_wzcsvc = {
	.name		= "wzcsvc",
	.syntax_id	= {
		{0x621dff68,0x3c39,0x4c6c,{0xaa,0xe3},{0xe6,0x8e,0x2c,0x65,0x03,0xad}},
		NDR_WZCSVC_VERSION
	},
	.helpstring	= NDR_WZCSVC_HELPSTRING,
	.num_calls	= 18,
	.calls		= wzcsvc_calls,
	.num_public_structs	= 0,
	.public_structs		= wzcsvc_public_structs,
	.endpoints	= &wzcsvc_endpoints,
	.authservices	= &wzcsvc_authservices
};

#endif /* SKIP_NDR_TABLE_wzcsvc */
