/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/ioctl.h"

#ifndef _HEADER_NDR_copychunk
#define _HEADER_NDR_copychunk

extern const struct ndr_interface_table ndr_table_copychunk;
#define NDR_COPYCHUNK_CALL_COUNT (0)
enum ndr_err_code ndr_push_req_resume_key_rsp(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct req_resume_key_rsp *r);
enum ndr_err_code ndr_pull_req_resume_key_rsp(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct req_resume_key_rsp *r);
void ndr_print_req_resume_key_rsp(struct ndr_print *ndr, const char *name, const struct req_resume_key_rsp *r);
void ndr_print_srv_copychunk(struct ndr_print *ndr, const char *name, const struct srv_copychunk *r);
enum ndr_err_code ndr_push_srv_copychunk_copy(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct srv_copychunk_copy *r);
enum ndr_err_code ndr_pull_srv_copychunk_copy(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct srv_copychunk_copy *r);
void ndr_print_srv_copychunk_copy(struct ndr_print *ndr, const char *name, const struct srv_copychunk_copy *r);
enum ndr_err_code ndr_push_srv_copychunk_rsp(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct srv_copychunk_rsp *r);
enum ndr_err_code ndr_pull_srv_copychunk_rsp(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct srv_copychunk_rsp *r);
void ndr_print_srv_copychunk_rsp(struct ndr_print *ndr, const char *name, const struct srv_copychunk_rsp *r);
enum ndr_err_code ndr_push_device_copy_offload_descriptor(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct device_copy_offload_descriptor *r);
enum ndr_err_code ndr_pull_device_copy_offload_descriptor(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct device_copy_offload_descriptor *r);
void ndr_print_device_copy_offload_descriptor(struct ndr_print *ndr, const char *name, const struct device_copy_offload_descriptor *r);
enum ndr_err_code ndr_push_storage_offload_token(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct storage_offload_token *r);
enum ndr_err_code ndr_pull_storage_offload_token(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct storage_offload_token *r);
void ndr_print_storage_offload_token(struct ndr_print *ndr, const char *name, const struct storage_offload_token *r);
enum ndr_err_code ndr_push_fsctl_offload_read_input(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_offload_read_input *r);
enum ndr_err_code ndr_pull_fsctl_offload_read_input(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_offload_read_input *r);
void ndr_print_fsctl_offload_read_input(struct ndr_print *ndr, const char *name, const struct fsctl_offload_read_input *r);
enum ndr_err_code ndr_push_offload_flags(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint32_t r);
enum ndr_err_code ndr_pull_offload_flags(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint32_t *r);
void ndr_print_offload_flags(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_fsctl_offload_read_output(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_offload_read_output *r);
enum ndr_err_code ndr_pull_fsctl_offload_read_output(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_offload_read_output *r);
void ndr_print_fsctl_offload_read_output(struct ndr_print *ndr, const char *name, const struct fsctl_offload_read_output *r);
enum ndr_err_code ndr_push_fsctl_offload_write_input(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_offload_write_input *r);
enum ndr_err_code ndr_pull_fsctl_offload_write_input(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_offload_write_input *r);
void ndr_print_fsctl_offload_write_input(struct ndr_print *ndr, const char *name, const struct fsctl_offload_write_input *r);
enum ndr_err_code ndr_push_fsctl_offload_write_output(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_offload_write_output *r);
enum ndr_err_code ndr_pull_fsctl_offload_write_output(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_offload_write_output *r);
void ndr_print_fsctl_offload_write_output(struct ndr_print *ndr, const char *name, const struct fsctl_offload_write_output *r);
enum ndr_err_code ndr_push_fsctl_dup_extents_to_file(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_dup_extents_to_file *r);
enum ndr_err_code ndr_pull_fsctl_dup_extents_to_file(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_dup_extents_to_file *r);
void ndr_print_fsctl_dup_extents_to_file(struct ndr_print *ndr, const char *name, const struct fsctl_dup_extents_to_file *r);
#endif /* _HEADER_NDR_copychunk */
#ifndef _HEADER_NDR_compression
#define _HEADER_NDR_compression

extern const struct ndr_interface_table ndr_table_compression;
#define NDR_COMPRESSION_CALL_COUNT (0)
enum ndr_err_code ndr_push_compression_state(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct compression_state *r);
enum ndr_err_code ndr_pull_compression_state(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct compression_state *r);
void ndr_print_compression_state(struct ndr_print *ndr, const char *name, const struct compression_state *r);
#endif /* _HEADER_NDR_compression */
#ifndef _HEADER_NDR_netinterface
#define _HEADER_NDR_netinterface

extern const struct ndr_interface_table ndr_table_netinterface;
#define NDR_NETINTERFACE_CALL_COUNT (0)
void ndr_print_fsctl_net_iface_capability(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_fsctl_sockaddr_af(struct ndr_print *ndr, const char *name, enum fsctl_sockaddr_af r);
void ndr_print_fsctl_sockaddr_in(struct ndr_print *ndr, const char *name, const struct fsctl_sockaddr_in *r);
void ndr_print_fsctl_sockaddr_in6(struct ndr_print *ndr, const char *name, const struct fsctl_sockaddr_in6 *r);
void ndr_print_fsctl_sockaddr_union(struct ndr_print *ndr, const char *name, const union fsctl_sockaddr_union *r);
void ndr_print_fsctl_sockaddr_storage(struct ndr_print *ndr, const char *name, const struct fsctl_sockaddr_storage *r);
enum ndr_err_code ndr_push_fsctl_net_iface_info(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_net_iface_info *r);
enum ndr_err_code ndr_pull_fsctl_net_iface_info(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_net_iface_info *r);
void ndr_print_fsctl_net_iface_info(struct ndr_print *ndr, const char *name, const struct fsctl_net_iface_info *r);
#endif /* _HEADER_NDR_netinterface */
#ifndef _HEADER_NDR_sparse
#define _HEADER_NDR_sparse

extern const struct ndr_interface_table ndr_table_sparse;
#define NDR_SPARSE_CALL_COUNT (0)
enum ndr_err_code ndr_push_file_alloced_range_buf(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct file_alloced_range_buf *r);
enum ndr_err_code ndr_pull_file_alloced_range_buf(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct file_alloced_range_buf *r);
void ndr_print_file_alloced_range_buf(struct ndr_print *ndr, const char *name, const struct file_alloced_range_buf *r);
enum ndr_err_code ndr_push_fsctl_query_alloced_ranges_req(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_query_alloced_ranges_req *r);
enum ndr_err_code ndr_pull_fsctl_query_alloced_ranges_req(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_query_alloced_ranges_req *r);
void ndr_print_fsctl_query_alloced_ranges_req(struct ndr_print *ndr, const char *name, const struct fsctl_query_alloced_ranges_req *r);
enum ndr_err_code ndr_push_fsctl_query_alloced_ranges_rsp(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_query_alloced_ranges_rsp *r);
enum ndr_err_code ndr_pull_fsctl_query_alloced_ranges_rsp(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_query_alloced_ranges_rsp *r);
void ndr_print_fsctl_query_alloced_ranges_rsp(struct ndr_print *ndr, const char *name, const struct fsctl_query_alloced_ranges_rsp *r);
enum ndr_err_code ndr_push_file_zero_data_info(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct file_zero_data_info *r);
enum ndr_err_code ndr_pull_file_zero_data_info(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct file_zero_data_info *r);
void ndr_print_file_zero_data_info(struct ndr_print *ndr, const char *name, const struct file_zero_data_info *r);
enum ndr_err_code ndr_push_fsctl_set_zero_data_req(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_set_zero_data_req *r);
enum ndr_err_code ndr_pull_fsctl_set_zero_data_req(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_set_zero_data_req *r);
void ndr_print_fsctl_set_zero_data_req(struct ndr_print *ndr, const char *name, const struct fsctl_set_zero_data_req *r);
#endif /* _HEADER_NDR_sparse */
#ifndef _HEADER_NDR_resiliency
#define _HEADER_NDR_resiliency

extern const struct ndr_interface_table ndr_table_resiliency;
#define NDR_RESILIENCY_CALL_COUNT (0)
enum ndr_err_code ndr_push_network_resiliency_request(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct network_resiliency_request *r);
enum ndr_err_code ndr_pull_network_resiliency_request(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct network_resiliency_request *r);
void ndr_print_network_resiliency_request(struct ndr_print *ndr, const char *name, const struct network_resiliency_request *r);
#endif /* _HEADER_NDR_resiliency */
#ifndef _HEADER_NDR_trim
#define _HEADER_NDR_trim

extern const struct ndr_interface_table ndr_table_trim;
#define NDR_TRIM_CALL_COUNT (0)
enum ndr_err_code ndr_push_file_level_trim_range(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct file_level_trim_range *r);
enum ndr_err_code ndr_pull_file_level_trim_range(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct file_level_trim_range *r);
void ndr_print_file_level_trim_range(struct ndr_print *ndr, const char *name, const struct file_level_trim_range *r);
enum ndr_err_code ndr_push_fsctl_file_level_trim_req(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_file_level_trim_req *r);
enum ndr_err_code ndr_pull_fsctl_file_level_trim_req(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_file_level_trim_req *r);
void ndr_print_fsctl_file_level_trim_req(struct ndr_print *ndr, const char *name, const struct fsctl_file_level_trim_req *r);
enum ndr_err_code ndr_push_fsctl_file_level_trim_rsp(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_file_level_trim_rsp *r);
enum ndr_err_code ndr_pull_fsctl_file_level_trim_rsp(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_file_level_trim_rsp *r);
void ndr_print_fsctl_file_level_trim_rsp(struct ndr_print *ndr, const char *name, const struct fsctl_file_level_trim_rsp *r);
#endif /* _HEADER_NDR_trim */
#ifndef _HEADER_NDR_fsctl
#define _HEADER_NDR_fsctl

extern const struct ndr_interface_table ndr_table_fsctl;
#define NDR_FSCTL_CALL_COUNT (0)
enum ndr_err_code ndr_push_fsctl_pipe_wait(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct fsctl_pipe_wait *r);
enum ndr_err_code ndr_pull_fsctl_pipe_wait(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct fsctl_pipe_wait *r);
void ndr_print_fsctl_pipe_wait(struct ndr_print *ndr, const char *name, const struct fsctl_pipe_wait *r);
#endif /* _HEADER_NDR_fsctl */
