#ifndef __SOURCE4_TORTURE_NBT_PROTO_H__
#define __SOURCE4_TORTURE_NBT_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../source4/torture/nbt/query.c  */

struct torture_suite *torture_bench_nbt(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/nbt/register.c  */

struct torture_suite *torture_nbt_register(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/nbt/wins.c  */

struct torture_suite *torture_nbt_wins(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/nbt/winsbench.c  */

struct torture_suite *torture_bench_wins(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/nbt/winsreplication.c  */

struct torture_suite *torture_nbt_winsreplication(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/nbt/dgram.c  */

struct torture_suite *torture_nbt_dgram(TALLOC_CTX *mem_ctx);

/* The following definitions come from ../../source4/torture/nbt/nbt.c  */

struct nbt_name_socket *torture_init_nbt_socket(struct torture_context *tctx);
bool torture_nbt_get_name(struct torture_context *tctx, 
			  struct nbt_name *name, 
			  const char **address);
NTSTATUS torture_nbt_init(TALLOC_CTX *ctx);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_TORTURE_NBT_PROTO_H__ */

