/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/dnsserver.h"

#ifndef _HEADER_NDR_dnsserver
#define _HEADER_NDR_dnsserver

#include "../librpc/ndr/ndr_dnsserver.h"
#define NDR_DNSSERVER_UUID "50abc2a4-574d-40b3-9d66-ee4fd5fba076"
#define NDR_DNSSERVER_VERSION 5.0
#define NDR_DNSSERVER_NAME "dnsserver"
#define NDR_DNSSERVER_HELPSTRING "DNS Management Server"
extern const struct ndr_interface_table ndr_table_dnsserver;
#define NDR_DNSSRVOPERATION (0x00)

#define NDR_DNSSRVQUERY (0x01)

#define NDR_DNSSRVCOMPLEXOPERATION (0x02)

#define NDR_DNSSRVENUMRECORDS (0x03)

#define NDR_DNSSRVUPDATERECORD (0x04)

#define NDR_DNSSRVOPERATION2 (0x05)

#define NDR_DNSSRVQUERY2 (0x06)

#define NDR_DNSSRVCOMPLEXOPERATION2 (0x07)

#define NDR_DNSSRVENUMRECORDS2 (0x08)

#define NDR_DNSSRVUPDATERECORD2 (0x09)

#define NDR_DNSSERVER_CALL_COUNT (10)
void ndr_print_DNS_RPC_CLIENT_VERSION(struct ndr_print *ndr, const char *name, enum DNS_RPC_CLIENT_VERSION r);
void ndr_print_DNS_RPC_BUFFER(struct ndr_print *ndr, const char *name, const struct DNS_RPC_BUFFER *r);
void ndr_print_DNS_RPC_UTF8_STRING_LIST(struct ndr_print *ndr, const char *name, const struct DNS_RPC_UTF8_STRING_LIST *r);
void ndr_print_DNS_RPC_NAME_AND_PARAM(struct ndr_print *ndr, const char *name, const struct DNS_RPC_NAME_AND_PARAM *r);
enum ndr_err_code ndr_push_DNS_RPC_NAME(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct DNS_RPC_NAME *r);
enum ndr_err_code ndr_pull_DNS_RPC_NAME(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct DNS_RPC_NAME *r);
void ndr_print_DNS_RPC_NAME(struct ndr_print *ndr, const char *name, const struct DNS_RPC_NAME *r);
size_t ndr_size_DNS_RPC_NAME(const struct DNS_RPC_NAME *r, libndr_flags flags);
void ndr_print_DNS_RPC_RECORD_SOA(struct ndr_print *ndr, const char *name, const struct DNS_RPC_RECORD_SOA *r);
void ndr_print_DNS_RPC_RECORD_NAME_PREFERENCE(struct ndr_print *ndr, const char *name, const struct DNS_RPC_RECORD_NAME_PREFERENCE *r);
enum ndr_err_code ndr_push_DNS_RPC_RECORD_STRING(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct DNS_RPC_RECORD_STRING *r);
enum ndr_err_code ndr_pull_DNS_RPC_RECORD_STRING(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct DNS_RPC_RECORD_STRING *r);
void ndr_print_DNS_RPC_RECORD_STRING(struct ndr_print *ndr, const char *name, const struct DNS_RPC_RECORD_STRING *r);
void ndr_print_DNS_RPC_RECORD_SRV(struct ndr_print *ndr, const char *name, const struct DNS_RPC_RECORD_SRV *r);
void ndr_print_DNS_RPC_RECORD_DATA(struct ndr_print *ndr, const char *name, const union DNS_RPC_RECORD_DATA *r);
enum ndr_err_code ndr_push_DNS_RPC_RECORD(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct DNS_RPC_RECORD *r);
enum ndr_err_code ndr_pull_DNS_RPC_RECORD(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct DNS_RPC_RECORD *r);
void ndr_print_DNS_RPC_RECORD(struct ndr_print *ndr, const char *name, const struct DNS_RPC_RECORD *r);
void ndr_print_DNS_RPC_RECORD_BUF(struct ndr_print *ndr, const char *name, const struct DNS_RPC_RECORD_BUF *r);
void ndr_print_IP4_ARRAY(struct ndr_print *ndr, const char *name, const struct IP4_ARRAY *r);
void ndr_print_DNS_ADDR(struct ndr_print *ndr, const char *name, const struct DNS_ADDR *r);
void ndr_print_DNS_ADDR_ARRAY(struct ndr_print *ndr, const char *name, const struct DNS_ADDR_ARRAY *r);
void ndr_print_DNS_RPC_IP_VALIDATE(struct ndr_print *ndr, const char *name, const struct DNS_RPC_IP_VALIDATE *r);
void ndr_print_DNS_BOOT_METHOD(struct ndr_print *ndr, const char *name, enum DNS_BOOT_METHOD r);
void ndr_print_DNS_NAME_CHECK_FLAG(struct ndr_print *ndr, const char *name, enum DNS_NAME_CHECK_FLAG r);
void ndr_print_DNS_RPC_SERVER_INFO_W2K(struct ndr_print *ndr, const char *name, const struct DNS_RPC_SERVER_INFO_W2K *r);
void ndr_print_DNS_EXTENSION(struct ndr_print *ndr, const char *name, const struct DNS_EXTENSION *r);
enum ndr_err_code ndr_push_DNS_RPC_SERVER_INFO_DOTNET(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct DNS_RPC_SERVER_INFO_DOTNET *r);
enum ndr_err_code ndr_pull_DNS_RPC_SERVER_INFO_DOTNET(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct DNS_RPC_SERVER_INFO_DOTNET *r);
void ndr_print_DNS_RPC_SERVER_INFO_DOTNET(struct ndr_print *ndr, const char *name, const struct DNS_RPC_SERVER_INFO_DOTNET *r);
enum ndr_err_code ndr_push_DNS_RPC_SERVER_INFO_LONGHORN(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct DNS_RPC_SERVER_INFO_LONGHORN *r);
enum ndr_err_code ndr_pull_DNS_RPC_SERVER_INFO_LONGHORN(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct DNS_RPC_SERVER_INFO_LONGHORN *r);
void ndr_print_DNS_RPC_SERVER_INFO_LONGHORN(struct ndr_print *ndr, const char *name, const struct DNS_RPC_SERVER_INFO_LONGHORN *r);
void ndr_print_DNS_RPC_DP_FLAGS(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_DNS_RPC_DP_REPLICA(struct ndr_print *ndr, const char *name, const struct DNS_RPC_DP_REPLICA *r);
void ndr_print_DNS_DP_STATE(struct ndr_print *ndr, const char *name, enum DNS_DP_STATE r);
void ndr_print_DNS_RPC_DP_INFO(struct ndr_print *ndr, const char *name, const struct DNS_RPC_DP_INFO *r);
void ndr_print_DNS_RPC_DP_ENUM(struct ndr_print *ndr, const char *name, const struct DNS_RPC_DP_ENUM *r);
void ndr_print_DNS_RPC_DP_LIST(struct ndr_print *ndr, const char *name, const struct DNS_RPC_DP_LIST *r);
void ndr_print_DNS_RPC_ENLIST_DP(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ENLIST_DP *r);
void ndr_print_DNS_RPC_ZONE_CHANGE_DP(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_CHANGE_DP *r);
void ndr_print_DNS_ZONE_SECONDARY_SECURITY(struct ndr_print *ndr, const char *name, enum DNS_ZONE_SECONDARY_SECURITY r);
void ndr_print_DNS_ZONE_NOTIFY_LEVEL(struct ndr_print *ndr, const char *name, enum DNS_ZONE_NOTIFY_LEVEL r);
void ndr_print_DNS_RPC_ZONE_FLAGS(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_DNS_RPC_ZONE_W2K(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_W2K *r);
void ndr_print_DNS_RPC_ZONE_DOTNET(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_DOTNET *r);
void ndr_print_DNS_RPC_ZONE_LIST_W2K(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_LIST_W2K *r);
void ndr_print_DNS_RPC_ZONE_LIST_DOTNET(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_LIST_DOTNET *r);
void ndr_print_DNS_RPC_ZONE_INFO_W2K(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_INFO_W2K *r);
void ndr_print_DNS_RPC_ZONE_INFO_DOTNET(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_INFO_DOTNET *r);
void ndr_print_DNS_RPC_ZONE_INFO_LONGHORN(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_INFO_LONGHORN *r);
void ndr_print_DNS_RPC_ZONE_SECONDARIES_W2K(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_SECONDARIES_W2K *r);
void ndr_print_DNS_RPC_ZONE_SECONDARIES_DOTNET(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_SECONDARIES_DOTNET *r);
void ndr_print_DNS_RPC_ZONE_SECONDARIES_LONGHORN(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_SECONDARIES_LONGHORN *r);
void ndr_print_DNS_RPC_ZONE_DATABASE_W2K(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_DATABASE_W2K *r);
void ndr_print_DNS_RPC_ZONE_DATABASE_DOTNET(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_DATABASE_DOTNET *r);
void ndr_print_DNS_RPC_ZONE_CREATE_INFO_W2K(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_CREATE_INFO_W2K *r);
void ndr_print_DNS_RPC_ZONE_CREATE_INFO_DOTNET(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_CREATE_INFO_DOTNET *r);
void ndr_print_DNS_RPC_ZONE_CREATE_INFO_LONGHORN(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_CREATE_INFO_LONGHORN *r);
void ndr_print_DNS_RPC_ZONE_EXPORT_INFO(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ZONE_EXPORT_INFO *r);
void ndr_print_DNS_RPC_ENUM_ZONES_FILTER(struct ndr_print *ndr, const char *name, const struct DNS_RPC_ENUM_ZONES_FILTER *r);
void ndr_print_DNS_RPC_FORWARDERS_W2K(struct ndr_print *ndr, const char *name, const struct DNS_RPC_FORWARDERS_W2K *r);
void ndr_print_DNS_RPC_FORWARDERS_DOTNET(struct ndr_print *ndr, const char *name, const struct DNS_RPC_FORWARDERS_DOTNET *r);
void ndr_print_DNS_RPC_FORWARDERS_LONGHORN(struct ndr_print *ndr, const char *name, const struct DNS_RPC_FORWARDERS_LONGHORN *r);
void ndr_print_DNS_RPC_AUTOCONFIG(struct ndr_print *ndr, const char *name, enum DNS_RPC_AUTOCONFIG r);
void ndr_print_DNS_RPC_AUTOCONFIGURE(struct ndr_print *ndr, const char *name, const struct DNS_RPC_AUTOCONFIGURE *r);
void ndr_print_DNS_SELECT_FLAGS(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_DNSSRV_STAT_HEADER(struct ndr_print *ndr, const char *name, const struct DNSSRV_STAT_HEADER *r);
void ndr_print_DNSSRV_STAT(struct ndr_print *ndr, const char *name, const struct DNSSRV_STAT *r);
void ndr_print_DNS_RPC_TYPEID(struct ndr_print *ndr, const char *name, enum DNS_RPC_TYPEID r);
void ndr_print_DNSSRV_RPC_UNION(struct ndr_print *ndr, const char *name, const union DNSSRV_RPC_UNION *r);
enum ndr_err_code ndr_push_DNS_RPC_RECORDS(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct DNS_RPC_RECORDS *r);
enum ndr_err_code ndr_pull_DNS_RPC_RECORDS(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct DNS_RPC_RECORDS *r);
void ndr_print_DNS_RPC_RECORDS(struct ndr_print *ndr, const char *name, const struct DNS_RPC_RECORDS *r);
size_t ndr_size_DNS_RPC_RECORDS(const struct DNS_RPC_RECORDS *r, libndr_flags flags);
enum ndr_err_code ndr_push_DNS_RPC_RECORDS_ARRAY(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct DNS_RPC_RECORDS_ARRAY *r);
enum ndr_err_code ndr_pull_DNS_RPC_RECORDS_ARRAY(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct DNS_RPC_RECORDS_ARRAY *r);
void ndr_print_DNS_RPC_RECORDS_ARRAY(struct ndr_print *ndr, const char *name, const struct DNS_RPC_RECORDS_ARRAY *r);
size_t ndr_size_DNS_RPC_RECORDS_ARRAY(const struct DNS_RPC_RECORDS_ARRAY *r, libndr_flags flags);
void ndr_print_DnssrvOperation(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct DnssrvOperation *r);
void ndr_print_DnssrvQuery(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct DnssrvQuery *r);
void ndr_print_DnssrvComplexOperation(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct DnssrvComplexOperation *r);
void ndr_print_DnssrvEnumRecords(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct DnssrvEnumRecords *r);
void ndr_print_DnssrvUpdateRecord(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct DnssrvUpdateRecord *r);
void ndr_print_DnssrvOperation2(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct DnssrvOperation2 *r);
void ndr_print_DnssrvQuery2(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct DnssrvQuery2 *r);
void ndr_print_DnssrvComplexOperation2(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct DnssrvComplexOperation2 *r);
void ndr_print_DnssrvEnumRecords2(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct DnssrvEnumRecords2 *r);
void ndr_print_DnssrvUpdateRecord2(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct DnssrvUpdateRecord2 *r);
#endif /* _HEADER_NDR_dnsserver */
