/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_wzcsvc
#define _PIDL_HEADER_wzcsvc

#include <stdint.h>


#ifndef _HEADER_wzcsvc
#define _HEADER_wzcsvc


struct wzcsvc_EnumInterfaces {
	int _dummy_element;
};


struct wzcsvc_QueryInterface {
	int _dummy_element;
};


struct wzcsvc_SetInterface {
	int _dummy_element;
};


struct wzcsvc_RefreshInterface {
	int _dummy_element;
};


struct wzcsvc_QueryContext {
	int _dummy_element;
};


struct wzcsvc_SetContext {
	int _dummy_element;
};


struct wzcsvc_EapolUIResponse {
	int _dummy_element;
};


struct wzcsvc_EapolGetCustomAuthData {
	int _dummy_element;
};


struct wzcsvc_EapolSetCustomAuthData {
	int _dummy_element;
};


struct wzcsvc_EapolGetInterfaceParams {
	int _dummy_element;
};


struct wzcsvc_EapolSetInterfaceParams {
	int _dummy_element;
};


struct wzcsvc_EapolReAuthenticateInterface {
	int _dummy_element;
};


struct wzcsvc_EapolQueryInterfaceState {
	int _dummy_element;
};


struct wzcsvc_OpenWZCDbLogSession {
	int _dummy_element;
};


struct wzcsvc_CloseWZCDbLogSession {
	int _dummy_element;
};


struct wzcsvc_EnumWZCDbLogRecords {
	int _dummy_element;
};


struct wzcsvc_FlushWZCdbLog {
	int _dummy_element;
};


struct wzcsvc_GetWZCDbLogRecord {
	int _dummy_element;
};

#endif /* _HEADER_wzcsvc */
#endif /* _PIDL_HEADER_wzcsvc */
