/* parser auto-generated by pidl */

#include "replace.h"
#include "bin/default/librpc/gen_ndr/ndr_quota.h"

#include "librpc/gen_ndr/ndr_security.h"
_PUBLIC_ enum ndr_err_code ndr_push_file_get_quota_info(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct file_get_quota_info *r)
{
	NDR_PUSH_CHECK_FLAGS(ndr, ndr_flags);
	if (ndr_flags & NDR_SCALARS) {
		NDR_CHECK(ndr_push_align(ndr, 4));
		NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, r->next_entry_offset));
		NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, r->sid_length));
		NDR_CHECK(ndr_push_dom_sid(ndr, NDR_SCALARS, &r->sid));
		NDR_CHECK(ndr_push_trailer_align(ndr, 4));
	}
	if (ndr_flags & NDR_BUFFERS) {
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ enum ndr_err_code ndr_pull_file_get_quota_info(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct file_get_quota_info *r)
{
	NDR_PULL_CHECK_FLAGS(ndr, ndr_flags);
	if (ndr_flags & NDR_SCALARS) {
		NDR_CHECK(ndr_pull_align(ndr, 4));
		NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &r->next_entry_offset));
		NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &r->sid_length));
		NDR_CHECK(ndr_pull_dom_sid(ndr, NDR_SCALARS, &r->sid));
		NDR_CHECK(ndr_pull_trailer_align(ndr, 4));
	}
	if (ndr_flags & NDR_BUFFERS) {
	}
	return NDR_ERR_SUCCESS;
}

static void ndr_print_flags_file_get_quota_info(struct ndr_print *ndr, const char *name, ndr_flags_type unused, const struct file_get_quota_info *r)
{
	ndr_print_file_get_quota_info(ndr, name, r);
}

_PUBLIC_ void ndr_print_file_get_quota_info(struct ndr_print *ndr, const char *name, const struct file_get_quota_info *r)
{
	ndr_print_struct(ndr, name, "file_get_quota_info");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	ndr_print_uint32(ndr, "next_entry_offset", r->next_entry_offset);
	ndr_print_uint32(ndr, "sid_length", r->sid_length);
	ndr_print_dom_sid(ndr, "sid", &r->sid);
	ndr->depth--;
}

_PUBLIC_ enum ndr_err_code ndr_push_file_quota_information(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct file_quota_information *r)
{
	NDR_PUSH_CHECK_FLAGS(ndr, ndr_flags);
	if (ndr_flags & NDR_SCALARS) {
		NDR_CHECK(ndr_push_align(ndr, 8));
		NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, r->next_entry_offset));
		NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, r->sid_length));
		NDR_CHECK(ndr_push_hyper(ndr, NDR_SCALARS, r->change_time));
		NDR_CHECK(ndr_push_hyper(ndr, NDR_SCALARS, r->quota_used));
		NDR_CHECK(ndr_push_hyper(ndr, NDR_SCALARS, r->quota_threshold));
		NDR_CHECK(ndr_push_hyper(ndr, NDR_SCALARS, r->quota_limit));
		NDR_CHECK(ndr_push_dom_sid(ndr, NDR_SCALARS, &r->sid));
		NDR_CHECK(ndr_push_trailer_align(ndr, 8));
	}
	if (ndr_flags & NDR_BUFFERS) {
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ enum ndr_err_code ndr_pull_file_quota_information(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct file_quota_information *r)
{
	NDR_PULL_CHECK_FLAGS(ndr, ndr_flags);
	if (ndr_flags & NDR_SCALARS) {
		NDR_CHECK(ndr_pull_align(ndr, 8));
		NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &r->next_entry_offset));
		NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &r->sid_length));
		NDR_CHECK(ndr_pull_hyper(ndr, NDR_SCALARS, &r->change_time));
		NDR_CHECK(ndr_pull_hyper(ndr, NDR_SCALARS, &r->quota_used));
		NDR_CHECK(ndr_pull_hyper(ndr, NDR_SCALARS, &r->quota_threshold));
		NDR_CHECK(ndr_pull_hyper(ndr, NDR_SCALARS, &r->quota_limit));
		NDR_CHECK(ndr_pull_dom_sid(ndr, NDR_SCALARS, &r->sid));
		NDR_CHECK(ndr_pull_trailer_align(ndr, 8));
	}
	if (ndr_flags & NDR_BUFFERS) {
	}
	return NDR_ERR_SUCCESS;
}

static void ndr_print_flags_file_quota_information(struct ndr_print *ndr, const char *name, ndr_flags_type unused, const struct file_quota_information *r)
{
	ndr_print_file_quota_information(ndr, name, r);
}

_PUBLIC_ void ndr_print_file_quota_information(struct ndr_print *ndr, const char *name, const struct file_quota_information *r)
{
	ndr_print_struct(ndr, name, "file_quota_information");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	ndr_print_uint32(ndr, "next_entry_offset", r->next_entry_offset);
	ndr_print_uint32(ndr, "sid_length", r->sid_length);
	ndr_print_hyper(ndr, "change_time", r->change_time);
	ndr_print_hyper(ndr, "quota_used", r->quota_used);
	ndr_print_hyper(ndr, "quota_threshold", r->quota_threshold);
	ndr_print_hyper(ndr, "quota_limit", r->quota_limit);
	ndr_print_dom_sid(ndr, "sid", &r->sid);
	ndr->depth--;
}

#ifndef SKIP_NDR_TABLE_file_quota
static const struct ndr_interface_public_struct file_quota_public_structs[] = {
	{
		.name = "file_get_quota_info",
		.struct_size = sizeof(struct file_get_quota_info ),
		.ndr_push = (ndr_push_flags_fn_t) ndr_push_file_get_quota_info,
		.ndr_pull = (ndr_pull_flags_fn_t) ndr_pull_file_get_quota_info,
		.ndr_print = (ndr_print_function_t) ndr_print_flags_file_get_quota_info,
	},
	{
		.name = "file_quota_information",
		.struct_size = sizeof(struct file_quota_information ),
		.ndr_push = (ndr_push_flags_fn_t) ndr_push_file_quota_information,
		.ndr_pull = (ndr_pull_flags_fn_t) ndr_pull_file_quota_information,
		.ndr_print = (ndr_print_function_t) ndr_print_flags_file_quota_information,
	},
	{ .name = NULL }
};

static const struct ndr_interface_call file_quota_calls[] = {
	{ .name = NULL }
};

static const char * const file_quota_endpoint_strings[] = {
	"ncacn_np:[\\pipe\\file_quota]", 
};

static const struct ndr_interface_string_array file_quota_endpoints = {
	.count	= 1,
	.names	= file_quota_endpoint_strings
};

static const char * const file_quota_authservice_strings[] = {
	"host", 
};

static const struct ndr_interface_string_array file_quota_authservices = {
	.count	= 1,
	.names	= file_quota_authservice_strings
};


const struct ndr_interface_table ndr_table_file_quota = {
	.name		= "file_quota",
	.num_calls	= 0,
	.calls		= file_quota_calls,
	.num_public_structs	= 2,
	.public_structs		= file_quota_public_structs,
	.endpoints	= &file_quota_endpoints,
	.authservices	= &file_quota_authservices
};

#endif /* SKIP_NDR_TABLE_file_quota */
_PUBLIC_ enum ndr_err_code ndr_push_smb2_query_quota_info(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct smb2_query_quota_info *r)
{
	NDR_PUSH_CHECK_FLAGS(ndr, ndr_flags);
	if (ndr_flags & NDR_SCALARS) {
		NDR_CHECK(ndr_push_align(ndr, 4));
		NDR_CHECK(ndr_push_uint8(ndr, NDR_SCALARS, r->return_single));
		NDR_CHECK(ndr_push_uint8(ndr, NDR_SCALARS, r->restart_scan));
		NDR_CHECK(ndr_push_uint16(ndr, NDR_SCALARS, r->reserved));
		NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, r->sid_list_length));
		NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, r->start_sid_length));
		NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, r->start_sid_offset));
		NDR_CHECK(ndr_push_trailer_align(ndr, 4));
	}
	if (ndr_flags & NDR_BUFFERS) {
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ enum ndr_err_code ndr_pull_smb2_query_quota_info(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct smb2_query_quota_info *r)
{
	NDR_PULL_CHECK_FLAGS(ndr, ndr_flags);
	if (ndr_flags & NDR_SCALARS) {
		NDR_CHECK(ndr_pull_align(ndr, 4));
		NDR_CHECK(ndr_pull_uint8(ndr, NDR_SCALARS, &r->return_single));
		NDR_CHECK(ndr_pull_uint8(ndr, NDR_SCALARS, &r->restart_scan));
		NDR_CHECK(ndr_pull_uint16(ndr, NDR_SCALARS, &r->reserved));
		NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &r->sid_list_length));
		NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &r->start_sid_length));
		NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &r->start_sid_offset));
		NDR_CHECK(ndr_pull_trailer_align(ndr, 4));
	}
	if (ndr_flags & NDR_BUFFERS) {
	}
	return NDR_ERR_SUCCESS;
}

static void ndr_print_flags_smb2_query_quota_info(struct ndr_print *ndr, const char *name, ndr_flags_type unused, const struct smb2_query_quota_info *r)
{
	ndr_print_smb2_query_quota_info(ndr, name, r);
}

_PUBLIC_ void ndr_print_smb2_query_quota_info(struct ndr_print *ndr, const char *name, const struct smb2_query_quota_info *r)
{
	ndr_print_struct(ndr, name, "smb2_query_quota_info");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	ndr_print_uint8(ndr, "return_single", r->return_single);
	ndr_print_uint8(ndr, "restart_scan", r->restart_scan);
	ndr_print_uint16(ndr, "reserved", r->reserved);
	ndr_print_uint32(ndr, "sid_list_length", r->sid_list_length);
	ndr_print_uint32(ndr, "start_sid_length", r->start_sid_length);
	ndr_print_uint32(ndr, "start_sid_offset", r->start_sid_offset);
	ndr->depth--;
}

#ifndef SKIP_NDR_TABLE_smb2_query_quota
static const struct ndr_interface_public_struct smb2_query_quota_public_structs[] = {
	{
		.name = "smb2_query_quota_info",
		.struct_size = sizeof(struct smb2_query_quota_info ),
		.ndr_push = (ndr_push_flags_fn_t) ndr_push_smb2_query_quota_info,
		.ndr_pull = (ndr_pull_flags_fn_t) ndr_pull_smb2_query_quota_info,
		.ndr_print = (ndr_print_function_t) ndr_print_flags_smb2_query_quota_info,
	},
	{ .name = NULL }
};

static const struct ndr_interface_call smb2_query_quota_calls[] = {
	{ .name = NULL }
};

static const char * const smb2_query_quota_endpoint_strings[] = {
	"ncacn_np:[\\pipe\\smb2_query_quota]", 
};

static const struct ndr_interface_string_array smb2_query_quota_endpoints = {
	.count	= 1,
	.names	= smb2_query_quota_endpoint_strings
};

static const char * const smb2_query_quota_authservice_strings[] = {
	"host", 
};

static const struct ndr_interface_string_array smb2_query_quota_authservices = {
	.count	= 1,
	.names	= smb2_query_quota_authservice_strings
};


const struct ndr_interface_table ndr_table_smb2_query_quota = {
	.name		= "smb2_query_quota",
	.num_calls	= 0,
	.calls		= smb2_query_quota_calls,
	.num_public_structs	= 1,
	.public_structs		= smb2_query_quota_public_structs,
	.endpoints	= &smb2_query_quota_endpoints,
	.authservices	= &smb2_query_quota_authservices
};

#endif /* SKIP_NDR_TABLE_smb2_query_quota */
_PUBLIC_ enum ndr_err_code ndr_push_nttrans_query_quota_params(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct nttrans_query_quota_params *r)
{
	NDR_PUSH_CHECK_FLAGS(ndr, ndr_flags);
	if (ndr_flags & NDR_SCALARS) {
		NDR_CHECK(ndr_push_align(ndr, 4));
		NDR_CHECK(ndr_push_uint16(ndr, NDR_SCALARS, r->fid));
		NDR_CHECK(ndr_push_uint8(ndr, NDR_SCALARS, r->return_single_entry));
		NDR_CHECK(ndr_push_uint8(ndr, NDR_SCALARS, r->restart_scan));
		NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, r->sid_list_length));
		NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, r->start_sid_length));
		NDR_CHECK(ndr_push_uint32(ndr, NDR_SCALARS, r->start_sid_offset));
		NDR_CHECK(ndr_push_trailer_align(ndr, 4));
	}
	if (ndr_flags & NDR_BUFFERS) {
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ enum ndr_err_code ndr_pull_nttrans_query_quota_params(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct nttrans_query_quota_params *r)
{
	NDR_PULL_CHECK_FLAGS(ndr, ndr_flags);
	if (ndr_flags & NDR_SCALARS) {
		NDR_CHECK(ndr_pull_align(ndr, 4));
		NDR_CHECK(ndr_pull_uint16(ndr, NDR_SCALARS, &r->fid));
		NDR_CHECK(ndr_pull_uint8(ndr, NDR_SCALARS, &r->return_single_entry));
		NDR_CHECK(ndr_pull_uint8(ndr, NDR_SCALARS, &r->restart_scan));
		NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &r->sid_list_length));
		NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &r->start_sid_length));
		NDR_CHECK(ndr_pull_uint32(ndr, NDR_SCALARS, &r->start_sid_offset));
		NDR_CHECK(ndr_pull_trailer_align(ndr, 4));
	}
	if (ndr_flags & NDR_BUFFERS) {
	}
	return NDR_ERR_SUCCESS;
}

static void ndr_print_flags_nttrans_query_quota_params(struct ndr_print *ndr, const char *name, ndr_flags_type unused, const struct nttrans_query_quota_params *r)
{
	ndr_print_nttrans_query_quota_params(ndr, name, r);
}

_PUBLIC_ void ndr_print_nttrans_query_quota_params(struct ndr_print *ndr, const char *name, const struct nttrans_query_quota_params *r)
{
	ndr_print_struct(ndr, name, "nttrans_query_quota_params");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	ndr_print_uint16(ndr, "fid", r->fid);
	ndr_print_uint8(ndr, "return_single_entry", r->return_single_entry);
	ndr_print_uint8(ndr, "restart_scan", r->restart_scan);
	ndr_print_uint32(ndr, "sid_list_length", r->sid_list_length);
	ndr_print_uint32(ndr, "start_sid_length", r->start_sid_length);
	ndr_print_uint32(ndr, "start_sid_offset", r->start_sid_offset);
	ndr->depth--;
}

#ifndef SKIP_NDR_TABLE_smb1_nt_transact_query_quota
static const struct ndr_interface_public_struct smb1_nt_transact_query_quota_public_structs[] = {
	{
		.name = "nttrans_query_quota_params",
		.struct_size = sizeof(struct nttrans_query_quota_params ),
		.ndr_push = (ndr_push_flags_fn_t) ndr_push_nttrans_query_quota_params,
		.ndr_pull = (ndr_pull_flags_fn_t) ndr_pull_nttrans_query_quota_params,
		.ndr_print = (ndr_print_function_t) ndr_print_flags_nttrans_query_quota_params,
	},
	{ .name = NULL }
};

static const struct ndr_interface_call smb1_nt_transact_query_quota_calls[] = {
	{ .name = NULL }
};

static const char * const smb1_nt_transact_query_quota_endpoint_strings[] = {
	"ncacn_np:[\\pipe\\smb1_nt_transact_query_quota]", 
};

static const struct ndr_interface_string_array smb1_nt_transact_query_quota_endpoints = {
	.count	= 1,
	.names	= smb1_nt_transact_query_quota_endpoint_strings
};

static const char * const smb1_nt_transact_query_quota_authservice_strings[] = {
	"host", 
};

static const struct ndr_interface_string_array smb1_nt_transact_query_quota_authservices = {
	.count	= 1,
	.names	= smb1_nt_transact_query_quota_authservice_strings
};


const struct ndr_interface_table ndr_table_smb1_nt_transact_query_quota = {
	.name		= "smb1_nt_transact_query_quota",
	.num_calls	= 0,
	.calls		= smb1_nt_transact_query_quota_calls,
	.num_public_structs	= 1,
	.public_structs		= smb1_nt_transact_query_quota_public_structs,
	.endpoints	= &smb1_nt_transact_query_quota_endpoints,
	.authservices	= &smb1_nt_transact_query_quota_authservices
};

#endif /* SKIP_NDR_TABLE_smb1_nt_transact_query_quota */
