/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/quota.h"

#ifndef _HEADER_NDR_file_quota
#define _HEADER_NDR_file_quota

extern const struct ndr_interface_table ndr_table_file_quota;
#define NDR_FILE_QUOTA_CALL_COUNT (0)
enum ndr_err_code ndr_push_file_get_quota_info(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct file_get_quota_info *r);
enum ndr_err_code ndr_pull_file_get_quota_info(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct file_get_quota_info *r);
void ndr_print_file_get_quota_info(struct ndr_print *ndr, const char *name, const struct file_get_quota_info *r);
enum ndr_err_code ndr_push_file_quota_information(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct file_quota_information *r);
enum ndr_err_code ndr_pull_file_quota_information(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct file_quota_information *r);
void ndr_print_file_quota_information(struct ndr_print *ndr, const char *name, const struct file_quota_information *r);
#endif /* _HEADER_NDR_file_quota */
#ifndef _HEADER_NDR_smb2_query_quota
#define _HEADER_NDR_smb2_query_quota

extern const struct ndr_interface_table ndr_table_smb2_query_quota;
#define NDR_SMB2_QUERY_QUOTA_CALL_COUNT (0)
enum ndr_err_code ndr_push_smb2_query_quota_info(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct smb2_query_quota_info *r);
enum ndr_err_code ndr_pull_smb2_query_quota_info(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct smb2_query_quota_info *r);
void ndr_print_smb2_query_quota_info(struct ndr_print *ndr, const char *name, const struct smb2_query_quota_info *r);
#endif /* _HEADER_NDR_smb2_query_quota */
#ifndef _HEADER_NDR_smb1_nt_transact_query_quota
#define _HEADER_NDR_smb1_nt_transact_query_quota

extern const struct ndr_interface_table ndr_table_smb1_nt_transact_query_quota;
#define NDR_SMB1_NT_TRANSACT_QUERY_QUOTA_CALL_COUNT (0)
enum ndr_err_code ndr_push_nttrans_query_quota_params(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct nttrans_query_quota_params *r);
enum ndr_err_code ndr_pull_nttrans_query_quota_params(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct nttrans_query_quota_params *r);
void ndr_print_nttrans_query_quota_params(struct ndr_print *ndr, const char *name, const struct nttrans_query_quota_params *r);
#endif /* _HEADER_NDR_smb1_nt_transact_query_quota */
