#ifndef __NDR_SAMR_SCOMPAT_H__
#define __NDR_SAMR_SCOMPAT_H__

struct pipes_struct;
struct dcesrv_endpoint_server;
struct dcesrv_call_state;

const struct dcesrv_endpoint_server *samr_get_ep_server(void);

NTSTATUS samr__op_ndr_pull(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_pull *pull, void **r);
struct samr_Connect;
NTSTATUS _samr_Connect(struct pipes_struct *p, struct samr_Connect *r);
struct samr_Close;
NTSTATUS _samr_Close(struct pipes_struct *p, struct samr_Close *r);
struct samr_SetSecurity;
NTSTATUS _samr_SetSecurity(struct pipes_struct *p, struct samr_SetSecurity *r);
struct samr_QuerySecurity;
NTSTATUS _samr_QuerySecurity(struct pipes_struct *p, struct samr_QuerySecurity *r);
struct samr_Shutdown;
NTSTATUS _samr_Shutdown(struct pipes_struct *p, struct samr_Shutdown *r);
struct samr_LookupDomain;
NTSTATUS _samr_LookupDomain(struct pipes_struct *p, struct samr_LookupDomain *r);
struct samr_EnumDomains;
NTSTATUS _samr_EnumDomains(struct pipes_struct *p, struct samr_EnumDomains *r);
struct samr_OpenDomain;
NTSTATUS _samr_OpenDomain(struct pipes_struct *p, struct samr_OpenDomain *r);
struct samr_QueryDomainInfo;
NTSTATUS _samr_QueryDomainInfo(struct pipes_struct *p, struct samr_QueryDomainInfo *r);
struct samr_SetDomainInfo;
NTSTATUS _samr_SetDomainInfo(struct pipes_struct *p, struct samr_SetDomainInfo *r);
struct samr_CreateDomainGroup;
NTSTATUS _samr_CreateDomainGroup(struct pipes_struct *p, struct samr_CreateDomainGroup *r);
struct samr_EnumDomainGroups;
NTSTATUS _samr_EnumDomainGroups(struct pipes_struct *p, struct samr_EnumDomainGroups *r);
struct samr_CreateUser;
NTSTATUS _samr_CreateUser(struct pipes_struct *p, struct samr_CreateUser *r);
struct samr_EnumDomainUsers;
NTSTATUS _samr_EnumDomainUsers(struct pipes_struct *p, struct samr_EnumDomainUsers *r);
struct samr_CreateDomAlias;
NTSTATUS _samr_CreateDomAlias(struct pipes_struct *p, struct samr_CreateDomAlias *r);
struct samr_EnumDomainAliases;
NTSTATUS _samr_EnumDomainAliases(struct pipes_struct *p, struct samr_EnumDomainAliases *r);
struct samr_GetAliasMembership;
NTSTATUS _samr_GetAliasMembership(struct pipes_struct *p, struct samr_GetAliasMembership *r);
struct samr_LookupNames;
NTSTATUS _samr_LookupNames(struct pipes_struct *p, struct samr_LookupNames *r);
struct samr_LookupRids;
NTSTATUS _samr_LookupRids(struct pipes_struct *p, struct samr_LookupRids *r);
struct samr_OpenGroup;
NTSTATUS _samr_OpenGroup(struct pipes_struct *p, struct samr_OpenGroup *r);
struct samr_QueryGroupInfo;
NTSTATUS _samr_QueryGroupInfo(struct pipes_struct *p, struct samr_QueryGroupInfo *r);
struct samr_SetGroupInfo;
NTSTATUS _samr_SetGroupInfo(struct pipes_struct *p, struct samr_SetGroupInfo *r);
struct samr_AddGroupMember;
NTSTATUS _samr_AddGroupMember(struct pipes_struct *p, struct samr_AddGroupMember *r);
struct samr_DeleteDomainGroup;
NTSTATUS _samr_DeleteDomainGroup(struct pipes_struct *p, struct samr_DeleteDomainGroup *r);
struct samr_DeleteGroupMember;
NTSTATUS _samr_DeleteGroupMember(struct pipes_struct *p, struct samr_DeleteGroupMember *r);
struct samr_QueryGroupMember;
NTSTATUS _samr_QueryGroupMember(struct pipes_struct *p, struct samr_QueryGroupMember *r);
struct samr_SetMemberAttributesOfGroup;
NTSTATUS _samr_SetMemberAttributesOfGroup(struct pipes_struct *p, struct samr_SetMemberAttributesOfGroup *r);
struct samr_OpenAlias;
NTSTATUS _samr_OpenAlias(struct pipes_struct *p, struct samr_OpenAlias *r);
struct samr_QueryAliasInfo;
NTSTATUS _samr_QueryAliasInfo(struct pipes_struct *p, struct samr_QueryAliasInfo *r);
struct samr_SetAliasInfo;
NTSTATUS _samr_SetAliasInfo(struct pipes_struct *p, struct samr_SetAliasInfo *r);
struct samr_DeleteDomAlias;
NTSTATUS _samr_DeleteDomAlias(struct pipes_struct *p, struct samr_DeleteDomAlias *r);
struct samr_AddAliasMember;
NTSTATUS _samr_AddAliasMember(struct pipes_struct *p, struct samr_AddAliasMember *r);
struct samr_DeleteAliasMember;
NTSTATUS _samr_DeleteAliasMember(struct pipes_struct *p, struct samr_DeleteAliasMember *r);
struct samr_GetMembersInAlias;
NTSTATUS _samr_GetMembersInAlias(struct pipes_struct *p, struct samr_GetMembersInAlias *r);
struct samr_OpenUser;
NTSTATUS _samr_OpenUser(struct pipes_struct *p, struct samr_OpenUser *r);
struct samr_DeleteUser;
NTSTATUS _samr_DeleteUser(struct pipes_struct *p, struct samr_DeleteUser *r);
struct samr_QueryUserInfo;
NTSTATUS _samr_QueryUserInfo(struct pipes_struct *p, struct samr_QueryUserInfo *r);
struct samr_SetUserInfo;
NTSTATUS _samr_SetUserInfo(struct pipes_struct *p, struct samr_SetUserInfo *r);
struct samr_ChangePasswordUser;
NTSTATUS _samr_ChangePasswordUser(struct pipes_struct *p, struct samr_ChangePasswordUser *r);
struct samr_GetGroupsForUser;
NTSTATUS _samr_GetGroupsForUser(struct pipes_struct *p, struct samr_GetGroupsForUser *r);
struct samr_QueryDisplayInfo;
NTSTATUS _samr_QueryDisplayInfo(struct pipes_struct *p, struct samr_QueryDisplayInfo *r);
struct samr_GetDisplayEnumerationIndex;
NTSTATUS _samr_GetDisplayEnumerationIndex(struct pipes_struct *p, struct samr_GetDisplayEnumerationIndex *r);
struct samr_TestPrivateFunctionsDomain;
NTSTATUS _samr_TestPrivateFunctionsDomain(struct pipes_struct *p, struct samr_TestPrivateFunctionsDomain *r);
struct samr_TestPrivateFunctionsUser;
NTSTATUS _samr_TestPrivateFunctionsUser(struct pipes_struct *p, struct samr_TestPrivateFunctionsUser *r);
struct samr_GetUserPwInfo;
NTSTATUS _samr_GetUserPwInfo(struct pipes_struct *p, struct samr_GetUserPwInfo *r);
struct samr_RemoveMemberFromForeignDomain;
NTSTATUS _samr_RemoveMemberFromForeignDomain(struct pipes_struct *p, struct samr_RemoveMemberFromForeignDomain *r);
struct samr_QueryDomainInfo2;
NTSTATUS _samr_QueryDomainInfo2(struct pipes_struct *p, struct samr_QueryDomainInfo2 *r);
struct samr_QueryUserInfo2;
NTSTATUS _samr_QueryUserInfo2(struct pipes_struct *p, struct samr_QueryUserInfo2 *r);
struct samr_QueryDisplayInfo2;
NTSTATUS _samr_QueryDisplayInfo2(struct pipes_struct *p, struct samr_QueryDisplayInfo2 *r);
struct samr_GetDisplayEnumerationIndex2;
NTSTATUS _samr_GetDisplayEnumerationIndex2(struct pipes_struct *p, struct samr_GetDisplayEnumerationIndex2 *r);
struct samr_CreateUser2;
NTSTATUS _samr_CreateUser2(struct pipes_struct *p, struct samr_CreateUser2 *r);
struct samr_QueryDisplayInfo3;
NTSTATUS _samr_QueryDisplayInfo3(struct pipes_struct *p, struct samr_QueryDisplayInfo3 *r);
struct samr_AddMultipleMembersToAlias;
NTSTATUS _samr_AddMultipleMembersToAlias(struct pipes_struct *p, struct samr_AddMultipleMembersToAlias *r);
struct samr_RemoveMultipleMembersFromAlias;
NTSTATUS _samr_RemoveMultipleMembersFromAlias(struct pipes_struct *p, struct samr_RemoveMultipleMembersFromAlias *r);
struct samr_OemChangePasswordUser2;
NTSTATUS _samr_OemChangePasswordUser2(struct pipes_struct *p, struct samr_OemChangePasswordUser2 *r);
struct samr_ChangePasswordUser2;
NTSTATUS _samr_ChangePasswordUser2(struct pipes_struct *p, struct samr_ChangePasswordUser2 *r);
struct samr_GetDomPwInfo;
NTSTATUS _samr_GetDomPwInfo(struct pipes_struct *p, struct samr_GetDomPwInfo *r);
struct samr_Connect2;
NTSTATUS _samr_Connect2(struct pipes_struct *p, struct samr_Connect2 *r);
struct samr_SetUserInfo2;
NTSTATUS _samr_SetUserInfo2(struct pipes_struct *p, struct samr_SetUserInfo2 *r);
struct samr_SetBootKeyInformation;
NTSTATUS _samr_SetBootKeyInformation(struct pipes_struct *p, struct samr_SetBootKeyInformation *r);
struct samr_GetBootKeyInformation;
NTSTATUS _samr_GetBootKeyInformation(struct pipes_struct *p, struct samr_GetBootKeyInformation *r);
struct samr_Connect3;
NTSTATUS _samr_Connect3(struct pipes_struct *p, struct samr_Connect3 *r);
struct samr_Connect4;
NTSTATUS _samr_Connect4(struct pipes_struct *p, struct samr_Connect4 *r);
struct samr_ChangePasswordUser3;
NTSTATUS _samr_ChangePasswordUser3(struct pipes_struct *p, struct samr_ChangePasswordUser3 *r);
struct samr_Connect5;
NTSTATUS _samr_Connect5(struct pipes_struct *p, struct samr_Connect5 *r);
struct samr_RidToSid;
NTSTATUS _samr_RidToSid(struct pipes_struct *p, struct samr_RidToSid *r);
struct samr_SetDsrmPassword;
NTSTATUS _samr_SetDsrmPassword(struct pipes_struct *p, struct samr_SetDsrmPassword *r);
struct samr_ValidatePassword;
NTSTATUS _samr_ValidatePassword(struct pipes_struct *p, struct samr_ValidatePassword *r);
struct samr_Opnum68NotUsedOnWire;
void _samr_Opnum68NotUsedOnWire(struct pipes_struct *p, struct samr_Opnum68NotUsedOnWire *r);
struct samr_Opnum69NotUsedOnWire;
void _samr_Opnum69NotUsedOnWire(struct pipes_struct *p, struct samr_Opnum69NotUsedOnWire *r);
struct samr_Opnum70NotUsedOnWire;
void _samr_Opnum70NotUsedOnWire(struct pipes_struct *p, struct samr_Opnum70NotUsedOnWire *r);
struct samr_Opnum71NotUsedOnWire;
void _samr_Opnum71NotUsedOnWire(struct pipes_struct *p, struct samr_Opnum71NotUsedOnWire *r);
struct samr_Opnum72NotUsedOnWire;
void _samr_Opnum72NotUsedOnWire(struct pipes_struct *p, struct samr_Opnum72NotUsedOnWire *r);
struct samr_ChangePasswordUser4;
NTSTATUS _samr_ChangePasswordUser4(struct pipes_struct *p, struct samr_ChangePasswordUser4 *r);
NTSTATUS samr__op_dispatch(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS samr__op_reply(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS samr__op_ndr_push(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_push *push, const void *r);
NTSTATUS samr__op_local(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
#endif /* __NDR_SAMR_SCOMPAT_H__ */
