/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/winstation.h"

#ifndef _HEADER_NDR_winstation
#define _HEADER_NDR_winstation

#define NDR_WINSTATION_UUID "5ca4a760-ebb1-11cf-8611-00a0245420ed"
#define NDR_WINSTATION_VERSION 1.0
#define NDR_WINSTATION_NAME "winstation"
#define NDR_WINSTATION_HELPSTRING "Terminal Services remote management"
extern const struct ndr_interface_table ndr_table_winstation;
#define NDR_WINSTATION_FOO (0x00)

#define NDR_WINSTATION_CALL_COUNT (1)
void ndr_print_winstation_foo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct winstation_foo *r);
#endif /* _HEADER_NDR_winstation */
