#ifndef __SOURCE4_LIB_REGISTRY_TOOLS_COMMON_H__
#define __SOURCE4_LIB_REGISTRY_TOOLS_COMMON_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../source4/lib/registry/tools/common.c  */

struct registry_context *reg_common_open_remote(const char *remote,
						struct tevent_context *ev_ctx,
						struct loadparm_context *lp_ctx,
						struct cli_credentials *creds);
struct registry_key *reg_common_open_file(const char *path,
					  struct tevent_context *ev_ctx,
					  struct loadparm_context *lp_ctx,
					  struct cli_credentials *creds);
struct registry_context *reg_common_open_local(struct cli_credentials *creds, 
					       struct tevent_context *ev_ctx, 
					       struct loadparm_context *lp_ctx);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_LIB_REGISTRY_TOOLS_COMMON_H__ */

