/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/trkwks.h"

#ifndef _HEADER_NDR_trkwks
#define _HEADER_NDR_trkwks

#define NDR_TRKWKS_UUID "300f3532-38cc-11d0-a3f0-0020af6b0add"
#define NDR_TRKWKS_VERSION 131073
#define NDR_TRKWKS_NAME "trkwks"
#define NDR_TRKWKS_HELPSTRING "Distributed Key Tracking Service"
extern const struct ndr_interface_table ndr_table_trkwks;
#define NDR_TRKWKS_UNKNOWN0 (0x00)

#define NDR_TRKWKS_CALL_COUNT (1)
void ndr_print_trkwks_Unknown0(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct trkwks_Unknown0 *r);
#endif /* _HEADER_NDR_trkwks */
