/* parser auto-generated by pidl */

#include "replace.h"
#include "bin/default/librpc/gen_ndr/ndr_dbgidl.h"

static enum ndr_err_code ndr_push_dummy_dbgidl(struct ndr_push *ndr, ndr_flags_type flags, const struct dummy_dbgidl *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_dummy_dbgidl(struct ndr_pull *ndr, ndr_flags_type flags, struct dummy_dbgidl *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_dummy_dbgidl(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct dummy_dbgidl *r)
{
	ndr_print_struct(ndr, name, "dummy_dbgidl");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "dummy_dbgidl");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "dummy_dbgidl");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

#ifndef SKIP_NDR_TABLE_dbgidl
static const struct ndr_interface_public_struct dbgidl_public_structs[] = {
	{ .name = NULL }
};

static const struct ndr_interface_call dbgidl_calls[] = {
	{
		"dummy_dbgidl",
		sizeof(struct dummy_dbgidl),
		(ndr_push_flags_fn_t) ndr_push_dummy_dbgidl,
		(ndr_pull_flags_fn_t) ndr_pull_dummy_dbgidl,
		(ndr_print_function_t) ndr_print_dummy_dbgidl,
		{ 0, NULL },
		{ 0, NULL },
	},
	{ .name = NULL }
};

static const char * const dbgidl_endpoint_strings[] = {
	"ncacn_np:[\\pipe\\dbgidl]", 
};

static const struct ndr_interface_string_array dbgidl_endpoints = {
	.count	= 1,
	.names	= dbgidl_endpoint_strings
};

static const char * const dbgidl_authservice_strings[] = {
	"host", 
};

static const struct ndr_interface_string_array dbgidl_authservices = {
	.count	= 1,
	.names	= dbgidl_authservice_strings
};


const struct ndr_interface_table ndr_table_dbgidl = {
	.name		= "dbgidl",
	.syntax_id	= {
		{0x1d55b526,0xc137,0x46c5,{0xab,0x79},{0x63,0x8f,0x2a,0x68,0xe8,0x69}},
		NDR_DBGIDL_VERSION
	},
	.helpstring	= NDR_DBGIDL_HELPSTRING,
	.num_calls	= 1,
	.calls		= dbgidl_calls,
	.num_public_structs	= 0,
	.public_structs		= dbgidl_public_structs,
	.endpoints	= &dbgidl_endpoints,
	.authservices	= &dbgidl_authservices
};

#endif /* SKIP_NDR_TABLE_dbgidl */
