/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_messaging
#define _PIDL_HEADER_messaging

#include <stdint.h>


#include "librpc/gen_ndr/server_id.h"
#ifndef _HEADER_messaging
#define _HEADER_messaging

#define MSG_TYPE_MASK	( 0xFFFF )
#define AUTH_EVENT_NAME	( "auth_event" )
#define DSDB_EVENT_NAME	( "dsdb_event" )
#define DSDB_PWD_EVENT_NAME	( "dsdb_password_event" )
#define DSDB_GROUP_EVENT_NAME	( "dsdb_group_event" )
enum messaging_type
#ifndef USE_UINT_ENUMS
 {
	MSG_DEBUG=(int)(0x0001),
	MSG_PING=(int)(0x0002),
	MSG_PONG=(int)(0x0003),
	MSG_PROFILE=(int)(0x0004),
	MSG_REQ_DEBUGLEVEL=(int)(0x0005),
	MSG_DEBUGLEVEL=(int)(0x0006),
	MSG_REQ_PROFILELEVEL=(int)(0x0007),
	MSG_PROFILELEVEL=(int)(0x0008),
	MSG_REQ_POOL_USAGE=(int)(0x0009),
	MSG_POOL_USAGE=(int)(0x000A),
	MSG_REQ_DMALLOC_MARK=(int)(0x000B),
	MSG_REQ_DMALLOC_LOG_CHANGED=(int)(0x000C),
	MSG_SHUTDOWN=(int)(0x000D),
	ID_CACHE_DELETE=(int)(0x000F),
	ID_CACHE_KILL=(int)(0x0010),
	MSG_SMB_CONF_UPDATED=(int)(0x0021),
	MSG_RELOAD_TLS_CERTIFICATES=(int)(0x0022),
	MSG_PREFORK_CHILD_EVENT=(int)(0x0031),
	MSG_PREFORK_PARENT_EVENT=(int)(0x0032),
	MSG_REQ_RINGBUF_LOG=(int)(0x0033),
	MSG_RINGBUF_LOG=(int)(0x0034),
	MSG_DAEMON_READY_FD=(int)(0x0035),
	MSG_FORCE_ELECTION=(int)(0x0101),
	MSG_WINS_NEW_ENTRY=(int)(0x0102),
	MSG_SEND_PACKET=(int)(0x0103),
	MSG_PRINTER_NOTIFY2=(int)(0x0202),
	MSG_PRINTER_DRVUPGRADE=(int)(0x0203),
	MSG_PRINTERDATA_INIT_RESET=(int)(0x0204),
	MSG_PRINTER_UPDATE=(int)(0x0205),
	MSG_PRINTER_MOD=(int)(0x0206),
	MSG_PRINTER_PCAP=(int)(0x0207),
	MSG_SMB_FORCE_TDIS=(int)(0x0302),
	MSG_SMB_BREAK_REQUEST=(int)(0x0306),
	MSG_SMB_KERNEL_BREAK=(int)(0x030A),
	MSG_SMB_FILE_RENAME=(int)(0x030B),
	MSG_SMB_INJECT_FAULT=(int)(0x030C),
	MSG_SMB_BLOCKING_LOCK_CANCEL=(int)(0x030D),
	MSG_SMB_NOTIFY=(int)(0x030E),
	MSG_PVFS_NOTIFY=(int)(0x0310),
	MSG_SMB_CLOSE_FILE=(int)(0x0313),
	MSG_SMB_NOTIFY_CLEANUP=(int)(0x0314),
	MSG_SMB_SCAVENGER=(int)(0x0315),
	MSG_SMB_KILL_CLIENT_IP=(int)(0x0316),
	MSG_SMB_TELL_NUM_CHILDREN=(int)(0x0317),
	MSG_SMB_NUM_CHILDREN=(int)(0x0318),
	MSG_SMB_NOTIFY_CANCEL_DELETED=(int)(0x0319),
	MSG_SMB_NOTIFY_REC_CHANGE=(int)(0x031A),
	MSG_SMB_NOTIFY_TRIGGER=(int)(0x031B),
	MSG_SMB_NOTIFY_GET_DB=(int)(0x031C),
	MSG_SMB_NOTIFY_DB=(int)(0x031D),
	MSG_SMB_NOTIFY_REC_CHANGES=(int)(0x031E),
	MSG_SMB_NOTIFY_STARTED=(int)(0x031F),
	MSG_SMB_SLEEP=(int)(0x0320),
	MSG_SMB_FORCE_TDIS_DENIED=(int)(0x0321),
	MSG_SMB_IP_DROPPED=(int)(0x0322),
	MSG_WINBIND_FINISHED=(int)(0x0401),
	MSG_WINBIND_FORGET_STATE=(int)(0x0402),
	MSG_WINBIND_ONLINE=(int)(0x0403),
	MSG_WINBIND_OFFLINE=(int)(0x0404),
	MSG_WINBIND_ONLINESTATUS=(int)(0x0405),
	MSG_WINBIND_VALIDATE_CACHE=(int)(0x0408),
	MSG_WINBIND_DUMP_DOMAIN_LIST=(int)(0x0409),
	MSG_WINBIND_IP_DROPPED=(int)(0x040A),
	MSG_WINBIND_DOMAIN_ONLINE=(int)(0x040B),
	MSG_WINBIND_DOMAIN_OFFLINE=(int)(0x040C),
	MSG_WINBIND_RELOAD_TRUSTED_DOMAINS=(int)(0x040D),
	MSG_WINBIND_DISCONNECT_DC=(int)(0x040E),
	MSG_SMBXSRV_SESSION_CLOSE=(int)(0x0600),
	MSG_SMBXSRV_CONNECTION_PASS=(int)(0x0601),
	MSG_SMBXSRV_CONNECTION_PASSED=(int)(0x0602),
	MSG_SMBXSRV_CONNECTION_DROP=(int)(0x0603),
	MSG_RPCD_WITNESS_REGISTRATION_UPDATE=(int)(0x0680),
	MSG_BRL_RETRY=(int)(0x0700),
	MSG_PVFS_RETRY_OPEN=(int)(0x0701),
	MSG_IRPC=(int)(0x0702),
	MSG_NTVFS_OPLOCK_BREAK=(int)(0x0703),
	MSG_DREPL_ALLOCATE_RID=(int)(0x0704),
	MSG_AUTH_LOG=(int)(0x0800),
	MSG_DSDB_LOG=(int)(0x0801),
	MSG_DSDB_PWD_LOG=(int)(0x0802),
	MSG_GROUP_LOG=(int)(0x0803),
	MSG_DBWRAP_MODIFIED=(int)(4003),
	MSG_RPC_HOST_NEW_CLIENT=(int)(4004),
	MSG_RPC_WORKER_STATUS=(int)(4005),
	MSG_RPC_DUMP_STATUS=(int)(4006),
	MSG_TMP_BASE=(int)(0xF000)
}
#else
 { __do_not_use_enum_messaging_type=INT_MAX}
#define MSG_DEBUG ( 0x0001 )
#define MSG_PING ( 0x0002 )
#define MSG_PONG ( 0x0003 )
#define MSG_PROFILE ( 0x0004 )
#define MSG_REQ_DEBUGLEVEL ( 0x0005 )
#define MSG_DEBUGLEVEL ( 0x0006 )
#define MSG_REQ_PROFILELEVEL ( 0x0007 )
#define MSG_PROFILELEVEL ( 0x0008 )
#define MSG_REQ_POOL_USAGE ( 0x0009 )
#define MSG_POOL_USAGE ( 0x000A )
#define MSG_REQ_DMALLOC_MARK ( 0x000B )
#define MSG_REQ_DMALLOC_LOG_CHANGED ( 0x000C )
#define MSG_SHUTDOWN ( 0x000D )
#define ID_CACHE_DELETE ( 0x000F )
#define ID_CACHE_KILL ( 0x0010 )
#define MSG_SMB_CONF_UPDATED ( 0x0021 )
#define MSG_RELOAD_TLS_CERTIFICATES ( 0x0022 )
#define MSG_PREFORK_CHILD_EVENT ( 0x0031 )
#define MSG_PREFORK_PARENT_EVENT ( 0x0032 )
#define MSG_REQ_RINGBUF_LOG ( 0x0033 )
#define MSG_RINGBUF_LOG ( 0x0034 )
#define MSG_DAEMON_READY_FD ( 0x0035 )
#define MSG_FORCE_ELECTION ( 0x0101 )
#define MSG_WINS_NEW_ENTRY ( 0x0102 )
#define MSG_SEND_PACKET ( 0x0103 )
#define MSG_PRINTER_NOTIFY2 ( 0x0202 )
#define MSG_PRINTER_DRVUPGRADE ( 0x0203 )
#define MSG_PRINTERDATA_INIT_RESET ( 0x0204 )
#define MSG_PRINTER_UPDATE ( 0x0205 )
#define MSG_PRINTER_MOD ( 0x0206 )
#define MSG_PRINTER_PCAP ( 0x0207 )
#define MSG_SMB_FORCE_TDIS ( 0x0302 )
#define MSG_SMB_BREAK_REQUEST ( 0x0306 )
#define MSG_SMB_KERNEL_BREAK ( 0x030A )
#define MSG_SMB_FILE_RENAME ( 0x030B )
#define MSG_SMB_INJECT_FAULT ( 0x030C )
#define MSG_SMB_BLOCKING_LOCK_CANCEL ( 0x030D )
#define MSG_SMB_NOTIFY ( 0x030E )
#define MSG_PVFS_NOTIFY ( 0x0310 )
#define MSG_SMB_CLOSE_FILE ( 0x0313 )
#define MSG_SMB_NOTIFY_CLEANUP ( 0x0314 )
#define MSG_SMB_SCAVENGER ( 0x0315 )
#define MSG_SMB_KILL_CLIENT_IP ( 0x0316 )
#define MSG_SMB_TELL_NUM_CHILDREN ( 0x0317 )
#define MSG_SMB_NUM_CHILDREN ( 0x0318 )
#define MSG_SMB_NOTIFY_CANCEL_DELETED ( 0x0319 )
#define MSG_SMB_NOTIFY_REC_CHANGE ( 0x031A )
#define MSG_SMB_NOTIFY_TRIGGER ( 0x031B )
#define MSG_SMB_NOTIFY_GET_DB ( 0x031C )
#define MSG_SMB_NOTIFY_DB ( 0x031D )
#define MSG_SMB_NOTIFY_REC_CHANGES ( 0x031E )
#define MSG_SMB_NOTIFY_STARTED ( 0x031F )
#define MSG_SMB_SLEEP ( 0x0320 )
#define MSG_SMB_FORCE_TDIS_DENIED ( 0x0321 )
#define MSG_SMB_IP_DROPPED ( 0x0322 )
#define MSG_WINBIND_FINISHED ( 0x0401 )
#define MSG_WINBIND_FORGET_STATE ( 0x0402 )
#define MSG_WINBIND_ONLINE ( 0x0403 )
#define MSG_WINBIND_OFFLINE ( 0x0404 )
#define MSG_WINBIND_ONLINESTATUS ( 0x0405 )
#define MSG_WINBIND_VALIDATE_CACHE ( 0x0408 )
#define MSG_WINBIND_DUMP_DOMAIN_LIST ( 0x0409 )
#define MSG_WINBIND_IP_DROPPED ( 0x040A )
#define MSG_WINBIND_DOMAIN_ONLINE ( 0x040B )
#define MSG_WINBIND_DOMAIN_OFFLINE ( 0x040C )
#define MSG_WINBIND_RELOAD_TRUSTED_DOMAINS ( 0x040D )
#define MSG_WINBIND_DISCONNECT_DC ( 0x040E )
#define MSG_SMBXSRV_SESSION_CLOSE ( 0x0600 )
#define MSG_SMBXSRV_CONNECTION_PASS ( 0x0601 )
#define MSG_SMBXSRV_CONNECTION_PASSED ( 0x0602 )
#define MSG_SMBXSRV_CONNECTION_DROP ( 0x0603 )
#define MSG_RPCD_WITNESS_REGISTRATION_UPDATE ( 0x0680 )
#define MSG_BRL_RETRY ( 0x0700 )
#define MSG_PVFS_RETRY_OPEN ( 0x0701 )
#define MSG_IRPC ( 0x0702 )
#define MSG_NTVFS_OPLOCK_BREAK ( 0x0703 )
#define MSG_DREPL_ALLOCATE_RID ( 0x0704 )
#define MSG_AUTH_LOG ( 0x0800 )
#define MSG_DSDB_LOG ( 0x0801 )
#define MSG_DSDB_PWD_LOG ( 0x0802 )
#define MSG_GROUP_LOG ( 0x0803 )
#define MSG_DBWRAP_MODIFIED ( 4003 )
#define MSG_RPC_HOST_NEW_CLIENT ( 4004 )
#define MSG_RPC_WORKER_STATUS ( 4005 )
#define MSG_RPC_DUMP_STATUS ( 4006 )
#define MSG_TMP_BASE ( 0xF000 )
#endif
;

struct messaging_rec {
	struct messaging_rec *prev;/* [skip,unique] */
	struct messaging_rec *next;/* [skip,unique] */
	uint32_t msg_version;
	enum messaging_type msg_type;
	struct server_id dest;
	struct server_id src;
	DATA_BLOB buf;
	uint8_t num_fds;
	int64_t *fds;
}/* [public] */;

struct messaging_reclog {
	uint64_t rec_index;
	uint32_t num_recs;
	struct messaging_rec **recs;/* [unique] */
}/* [public] */;

#endif /* _HEADER_messaging */
#endif /* _PIDL_HEADER_messaging */
