#ifndef __LIB_UTIL_UNIX_PRIVS_H__
#define __LIB_UTIL_UNIX_PRIVS_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../lib/util/unix_privs.c  */


/**
 * Obtain root privileges for the current process.
 *
 * The privileges can be dropped by talloc_free()-ing the 
 * token returned by this function
 */
void *root_privileges(void);
uid_t root_privileges_original_uid(void *s);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __LIB_UTIL_UNIX_PRIVS_H__ */

