/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/source3/librpc/gen_ndr/libnetapi.h"

#ifndef _HEADER_NDR_libnetapi
#define _HEADER_NDR_libnetapi

extern const struct ndr_interface_table ndr_table_libnetapi;
#define NDR_NETJOINDOMAIN (0x00)

#define NDR_NETUNJOINDOMAIN (0x01)

#define NDR_NETGETJOININFORMATION (0x02)

#define NDR_NETGETJOINABLEOUS (0x03)

#define NDR_NETRENAMEMACHINEINDOMAIN (0x04)

#define NDR_NETPROVISIONCOMPUTERACCOUNT (0x05)

#define NDR_NETREQUESTOFFLINEDOMAINJOIN (0x06)

#define NDR_NETCOMPOSEOFFLINEDOMAINJOIN (0x07)

#define NDR_NETSERVERGETINFO (0x08)

#define NDR_NETSERVERSETINFO (0x09)

#define NDR_NETWKSTAGETINFO (0x0a)

#define NDR_NETGETDCNAME (0x0b)

#define NDR_NETGETANYDCNAME (0x0c)

#define NDR_DSGETDCNAME (0x0d)

#define NDR_NETUSERADD (0x0e)

#define NDR_NETUSERDEL (0x0f)

#define NDR_NETUSERENUM (0x10)

#define NDR_NETUSERCHANGEPASSWORD (0x11)

#define NDR_NETUSERGETINFO (0x12)

#define NDR_NETUSERSETINFO (0x13)

#define NDR_NETUSERGETGROUPS (0x14)

#define NDR_NETUSERSETGROUPS (0x15)

#define NDR_NETUSERGETLOCALGROUPS (0x16)

#define NDR_NETUSERMODALSGET (0x17)

#define NDR_NETUSERMODALSSET (0x18)

#define NDR_NETQUERYDISPLAYINFORMATION (0x19)

#define NDR_NETGROUPADD (0x1a)

#define NDR_NETGROUPDEL (0x1b)

#define NDR_NETGROUPENUM (0x1c)

#define NDR_NETGROUPSETINFO (0x1d)

#define NDR_NETGROUPGETINFO (0x1e)

#define NDR_NETGROUPADDUSER (0x1f)

#define NDR_NETGROUPDELUSER (0x20)

#define NDR_NETGROUPGETUSERS (0x21)

#define NDR_NETGROUPSETUSERS (0x22)

#define NDR_NETLOCALGROUPADD (0x23)

#define NDR_NETLOCALGROUPDEL (0x24)

#define NDR_NETLOCALGROUPGETINFO (0x25)

#define NDR_NETLOCALGROUPSETINFO (0x26)

#define NDR_NETLOCALGROUPENUM (0x27)

#define NDR_NETLOCALGROUPADDMEMBERS (0x28)

#define NDR_NETLOCALGROUPDELMEMBERS (0x29)

#define NDR_NETLOCALGROUPGETMEMBERS (0x2a)

#define NDR_NETLOCALGROUPSETMEMBERS (0x2b)

#define NDR_NETREMOTETOD (0x2c)

#define NDR_NETSHAREADD (0x2d)

#define NDR_NETSHAREDEL (0x2e)

#define NDR_NETSHAREENUM (0x2f)

#define NDR_NETSHAREGETINFO (0x30)

#define NDR_NETSHARESETINFO (0x31)

#define NDR_NETFILECLOSE (0x32)

#define NDR_NETFILEGETINFO (0x33)

#define NDR_NETFILEENUM (0x34)

#define NDR_NETSHUTDOWNINIT (0x35)

#define NDR_NETSHUTDOWNABORT (0x36)

#define NDR_I_NETLOGONCONTROL (0x37)

#define NDR_I_NETLOGONCONTROL2 (0x38)

#define NDR_LIBNETAPI_CALL_COUNT (57)
enum ndr_err_code ndr_push_NET_API_STATUS(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum NET_API_STATUS r);
enum ndr_err_code ndr_pull_NET_API_STATUS(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum NET_API_STATUS *r);
void ndr_print_NET_API_STATUS(struct ndr_print *ndr, const char *name, enum NET_API_STATUS r);
enum ndr_err_code ndr_push_domsid(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct domsid *r);
enum ndr_err_code ndr_pull_domsid(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct domsid *r);
void ndr_print_domsid(struct ndr_print *ndr, const char *name, const struct domsid *r);
enum ndr_err_code ndr_push_NetJoinFlags(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint32_t r);
enum ndr_err_code ndr_pull_NetJoinFlags(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint32_t *r);
void ndr_print_NetJoinFlags(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_NetProvisionFlags(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint32_t r);
enum ndr_err_code ndr_pull_NetProvisionFlags(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint32_t *r);
void ndr_print_NetProvisionFlags(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_NetProvisionJoinFlags(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint32_t r);
enum ndr_err_code ndr_pull_NetProvisionJoinFlags(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint32_t *r);
void ndr_print_NetProvisionJoinFlags(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_SERVER_INFO_100(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_100 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_100(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_100 *r);
void ndr_print_SERVER_INFO_100(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_100 *r);
enum ndr_err_code ndr_push_SERVER_INFO_101(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_101 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_101(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_101 *r);
void ndr_print_SERVER_INFO_101(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_101 *r);
enum ndr_err_code ndr_push_SERVER_INFO_102(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_102 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_102(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_102 *r);
void ndr_print_SERVER_INFO_102(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_102 *r);
enum ndr_err_code ndr_push_SERVER_INFO_402(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_402 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_402(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_402 *r);
void ndr_print_SERVER_INFO_402(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_402 *r);
enum ndr_err_code ndr_push_SERVER_INFO_403(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_403 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_403(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_403 *r);
void ndr_print_SERVER_INFO_403(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_403 *r);
enum ndr_err_code ndr_push_SERVER_INFO_502(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_502 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_502(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_502 *r);
void ndr_print_SERVER_INFO_502(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_502 *r);
enum ndr_err_code ndr_push_SERVER_INFO_503(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_503 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_503(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_503 *r);
void ndr_print_SERVER_INFO_503(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_503 *r);
enum ndr_err_code ndr_push_SERVER_INFO_599(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_599 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_599(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_599 *r);
void ndr_print_SERVER_INFO_599(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_599 *r);
enum ndr_err_code ndr_push_SERVER_INFO_598(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_598 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_598(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_598 *r);
void ndr_print_SERVER_INFO_598(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_598 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1005(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1005 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1005(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1005 *r);
void ndr_print_SERVER_INFO_1005(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1005 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1107(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1107 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1107(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1107 *r);
void ndr_print_SERVER_INFO_1107(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1107 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1010(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1010 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1010(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1010 *r);
void ndr_print_SERVER_INFO_1010(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1010 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1016(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1016 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1016(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1016 *r);
void ndr_print_SERVER_INFO_1016(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1016 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1017(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1017 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1017(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1017 *r);
void ndr_print_SERVER_INFO_1017(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1017 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1018(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1018 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1018(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1018 *r);
void ndr_print_SERVER_INFO_1018(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1018 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1501(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1501 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1501(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1501 *r);
void ndr_print_SERVER_INFO_1501(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1501 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1502(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1502 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1502(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1502 *r);
void ndr_print_SERVER_INFO_1502(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1502 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1503(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1503 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1503(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1503 *r);
void ndr_print_SERVER_INFO_1503(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1503 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1506(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1506 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1506(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1506 *r);
void ndr_print_SERVER_INFO_1506(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1506 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1509(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1509 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1509(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1509 *r);
void ndr_print_SERVER_INFO_1509(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1509 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1510(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1510 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1510(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1510 *r);
void ndr_print_SERVER_INFO_1510(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1510 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1511(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1511 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1511(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1511 *r);
void ndr_print_SERVER_INFO_1511(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1511 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1512(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1512 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1512(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1512 *r);
void ndr_print_SERVER_INFO_1512(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1512 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1513(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1513 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1513(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1513 *r);
void ndr_print_SERVER_INFO_1513(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1513 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1514(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1514 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1514(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1514 *r);
void ndr_print_SERVER_INFO_1514(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1514 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1515(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1515 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1515(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1515 *r);
void ndr_print_SERVER_INFO_1515(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1515 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1516(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1516 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1516(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1516 *r);
void ndr_print_SERVER_INFO_1516(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1516 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1518(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1518 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1518(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1518 *r);
void ndr_print_SERVER_INFO_1518(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1518 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1520(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1520 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1520(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1520 *r);
void ndr_print_SERVER_INFO_1520(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1520 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1521(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1521 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1521(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1521 *r);
void ndr_print_SERVER_INFO_1521(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1521 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1522(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1522 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1522(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1522 *r);
void ndr_print_SERVER_INFO_1522(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1522 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1523(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1523 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1523(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1523 *r);
void ndr_print_SERVER_INFO_1523(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1523 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1524(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1524 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1524(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1524 *r);
void ndr_print_SERVER_INFO_1524(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1524 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1525(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1525 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1525(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1525 *r);
void ndr_print_SERVER_INFO_1525(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1525 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1528(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1528 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1528(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1528 *r);
void ndr_print_SERVER_INFO_1528(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1528 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1529(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1529 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1529(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1529 *r);
void ndr_print_SERVER_INFO_1529(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1529 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1530(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1530 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1530(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1530 *r);
void ndr_print_SERVER_INFO_1530(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1530 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1533(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1533 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1533(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1533 *r);
void ndr_print_SERVER_INFO_1533(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1533 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1534(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1534 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1534(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1534 *r);
void ndr_print_SERVER_INFO_1534(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1534 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1535(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1535 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1535(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1535 *r);
void ndr_print_SERVER_INFO_1535(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1535 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1536(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1536 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1536(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1536 *r);
void ndr_print_SERVER_INFO_1536(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1536 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1537(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1537 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1537(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1537 *r);
void ndr_print_SERVER_INFO_1537(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1537 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1538(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1538 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1538(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1538 *r);
void ndr_print_SERVER_INFO_1538(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1538 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1539(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1539 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1539(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1539 *r);
void ndr_print_SERVER_INFO_1539(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1539 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1540(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1540 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1540(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1540 *r);
void ndr_print_SERVER_INFO_1540(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1540 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1541(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1541 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1541(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1541 *r);
void ndr_print_SERVER_INFO_1541(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1541 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1542(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1542 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1542(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1542 *r);
void ndr_print_SERVER_INFO_1542(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1542 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1543(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1543 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1543(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1543 *r);
void ndr_print_SERVER_INFO_1543(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1543 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1544(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1544 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1544(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1544 *r);
void ndr_print_SERVER_INFO_1544(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1544 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1545(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1545 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1545(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1545 *r);
void ndr_print_SERVER_INFO_1545(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1545 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1546(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1546 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1546(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1546 *r);
void ndr_print_SERVER_INFO_1546(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1546 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1547(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1547 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1547(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1547 *r);
void ndr_print_SERVER_INFO_1547(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1547 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1548(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1548 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1548(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1548 *r);
void ndr_print_SERVER_INFO_1548(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1548 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1549(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1549 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1549(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1549 *r);
void ndr_print_SERVER_INFO_1549(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1549 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1550(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1550 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1550(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1550 *r);
void ndr_print_SERVER_INFO_1550(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1550 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1552(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1552 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1552(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1552 *r);
void ndr_print_SERVER_INFO_1552(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1552 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1553(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1553 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1553(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1553 *r);
void ndr_print_SERVER_INFO_1553(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1553 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1554(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1554 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1554(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1554 *r);
void ndr_print_SERVER_INFO_1554(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1554 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1555(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1555 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1555(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1555 *r);
void ndr_print_SERVER_INFO_1555(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1555 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1556(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1556 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1556(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1556 *r);
void ndr_print_SERVER_INFO_1556(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1556 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1557(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1557 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1557(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1557 *r);
void ndr_print_SERVER_INFO_1557(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1557 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1560(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1560 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1560(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1560 *r);
void ndr_print_SERVER_INFO_1560(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1560 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1561(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1561 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1561(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1561 *r);
void ndr_print_SERVER_INFO_1561(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1561 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1562(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1562 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1562(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1562 *r);
void ndr_print_SERVER_INFO_1562(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1562 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1563(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1563 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1563(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1563 *r);
void ndr_print_SERVER_INFO_1563(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1563 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1564(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1564 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1564(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1564 *r);
void ndr_print_SERVER_INFO_1564(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1564 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1565(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1565 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1565(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1565 *r);
void ndr_print_SERVER_INFO_1565(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1565 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1566(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1566 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1566(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1566 *r);
void ndr_print_SERVER_INFO_1566(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1566 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1567(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1567 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1567(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1567 *r);
void ndr_print_SERVER_INFO_1567(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1567 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1568(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1568 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1568(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1568 *r);
void ndr_print_SERVER_INFO_1568(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1568 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1569(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1569 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1569(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1569 *r);
void ndr_print_SERVER_INFO_1569(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1569 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1570(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1570 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1570(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1570 *r);
void ndr_print_SERVER_INFO_1570(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1570 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1571(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1571 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1571(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1571 *r);
void ndr_print_SERVER_INFO_1571(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1571 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1572(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1572 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1572(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1572 *r);
void ndr_print_SERVER_INFO_1572(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1572 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1573(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1573 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1573(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1573 *r);
void ndr_print_SERVER_INFO_1573(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1573 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1574(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1574 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1574(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1574 *r);
void ndr_print_SERVER_INFO_1574(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1574 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1575(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1575 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1575(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1575 *r);
void ndr_print_SERVER_INFO_1575(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1575 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1576(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1576 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1576(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1576 *r);
void ndr_print_SERVER_INFO_1576(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1576 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1577(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1577 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1577(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1577 *r);
void ndr_print_SERVER_INFO_1577(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1577 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1578(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1578 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1578(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1578 *r);
void ndr_print_SERVER_INFO_1578(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1578 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1579(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1579 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1579(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1579 *r);
void ndr_print_SERVER_INFO_1579(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1579 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1580(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1580 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1580(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1580 *r);
void ndr_print_SERVER_INFO_1580(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1580 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1581(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1581 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1581(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1581 *r);
void ndr_print_SERVER_INFO_1581(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1581 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1582(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1582 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1582(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1582 *r);
void ndr_print_SERVER_INFO_1582(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1582 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1583(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1583 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1583(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1583 *r);
void ndr_print_SERVER_INFO_1583(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1583 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1584(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1584 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1584(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1584 *r);
void ndr_print_SERVER_INFO_1584(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1584 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1585(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1585 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1585(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1585 *r);
void ndr_print_SERVER_INFO_1585(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1585 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1586(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1586 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1586(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1586 *r);
void ndr_print_SERVER_INFO_1586(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1586 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1587(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1587 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1587(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1587 *r);
void ndr_print_SERVER_INFO_1587(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1587 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1588(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1588 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1588(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1588 *r);
void ndr_print_SERVER_INFO_1588(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1588 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1590(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1590 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1590(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1590 *r);
void ndr_print_SERVER_INFO_1590(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1590 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1591(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1591 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1591(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1591 *r);
void ndr_print_SERVER_INFO_1591(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1591 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1592(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1592 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1592(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1592 *r);
void ndr_print_SERVER_INFO_1592(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1592 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1593(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1593 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1593(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1593 *r);
void ndr_print_SERVER_INFO_1593(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1593 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1594(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1594 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1594(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1594 *r);
void ndr_print_SERVER_INFO_1594(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1594 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1595(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1595 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1595(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1595 *r);
void ndr_print_SERVER_INFO_1595(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1595 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1596(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1596 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1596(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1596 *r);
void ndr_print_SERVER_INFO_1596(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1596 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1597(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1597 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1597(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1597 *r);
void ndr_print_SERVER_INFO_1597(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1597 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1598(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1598 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1598(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1598 *r);
void ndr_print_SERVER_INFO_1598(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1598 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1599(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1599 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1599(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1599 *r);
void ndr_print_SERVER_INFO_1599(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1599 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1600(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1600 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1600(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1600 *r);
void ndr_print_SERVER_INFO_1600(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1600 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1601(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1601 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1601(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1601 *r);
void ndr_print_SERVER_INFO_1601(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1601 *r);
enum ndr_err_code ndr_push_SERVER_INFO_1602(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct SERVER_INFO_1602 *r);
enum ndr_err_code ndr_pull_SERVER_INFO_1602(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct SERVER_INFO_1602 *r);
void ndr_print_SERVER_INFO_1602(struct ndr_print *ndr, const char *name, const struct SERVER_INFO_1602 *r);
enum ndr_err_code ndr_push_WKSTA_INFO_100(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct WKSTA_INFO_100 *r);
enum ndr_err_code ndr_pull_WKSTA_INFO_100(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct WKSTA_INFO_100 *r);
void ndr_print_WKSTA_INFO_100(struct ndr_print *ndr, const char *name, const struct WKSTA_INFO_100 *r);
enum ndr_err_code ndr_push_WKSTA_INFO_101(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct WKSTA_INFO_101 *r);
enum ndr_err_code ndr_pull_WKSTA_INFO_101(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct WKSTA_INFO_101 *r);
void ndr_print_WKSTA_INFO_101(struct ndr_print *ndr, const char *name, const struct WKSTA_INFO_101 *r);
enum ndr_err_code ndr_push_WKSTA_INFO_102(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct WKSTA_INFO_102 *r);
enum ndr_err_code ndr_pull_WKSTA_INFO_102(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct WKSTA_INFO_102 *r);
void ndr_print_WKSTA_INFO_102(struct ndr_print *ndr, const char *name, const struct WKSTA_INFO_102 *r);
enum ndr_err_code ndr_push_DOMAIN_CONTROLLER_INFO_FLAGS(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint32_t r);
enum ndr_err_code ndr_pull_DOMAIN_CONTROLLER_INFO_FLAGS(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint32_t *r);
void ndr_print_DOMAIN_CONTROLLER_INFO_FLAGS(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_DOMAIN_CONTROLLER_INFO(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct DOMAIN_CONTROLLER_INFO *r);
enum ndr_err_code ndr_pull_DOMAIN_CONTROLLER_INFO(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct DOMAIN_CONTROLLER_INFO *r);
void ndr_print_DOMAIN_CONTROLLER_INFO(struct ndr_print *ndr, const char *name, const struct DOMAIN_CONTROLLER_INFO *r);
enum ndr_err_code ndr_push_USER_INFO_0(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_0 *r);
enum ndr_err_code ndr_pull_USER_INFO_0(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_0 *r);
void ndr_print_USER_INFO_0(struct ndr_print *ndr, const char *name, const struct USER_INFO_0 *r);
enum ndr_err_code ndr_push_USER_INFO_1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1 *r);
enum ndr_err_code ndr_pull_USER_INFO_1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1 *r);
void ndr_print_USER_INFO_1(struct ndr_print *ndr, const char *name, const struct USER_INFO_1 *r);
enum ndr_err_code ndr_push_USER_INFO_2(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_2 *r);
enum ndr_err_code ndr_pull_USER_INFO_2(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_2 *r);
void ndr_print_USER_INFO_2(struct ndr_print *ndr, const char *name, const struct USER_INFO_2 *r);
enum ndr_err_code ndr_push_USER_INFO_3(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_3 *r);
enum ndr_err_code ndr_pull_USER_INFO_3(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_3 *r);
void ndr_print_USER_INFO_3(struct ndr_print *ndr, const char *name, const struct USER_INFO_3 *r);
enum ndr_err_code ndr_push_USER_INFO_4(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_4 *r);
enum ndr_err_code ndr_pull_USER_INFO_4(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_4 *r);
void ndr_print_USER_INFO_4(struct ndr_print *ndr, const char *name, const struct USER_INFO_4 *r);
enum ndr_err_code ndr_push_USER_INFO_10(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_10 *r);
enum ndr_err_code ndr_pull_USER_INFO_10(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_10 *r);
void ndr_print_USER_INFO_10(struct ndr_print *ndr, const char *name, const struct USER_INFO_10 *r);
enum ndr_err_code ndr_push_USER_INFO_11(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_11 *r);
enum ndr_err_code ndr_pull_USER_INFO_11(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_11 *r);
void ndr_print_USER_INFO_11(struct ndr_print *ndr, const char *name, const struct USER_INFO_11 *r);
enum ndr_err_code ndr_push_USER_INFO_20(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_20 *r);
enum ndr_err_code ndr_pull_USER_INFO_20(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_20 *r);
void ndr_print_USER_INFO_20(struct ndr_print *ndr, const char *name, const struct USER_INFO_20 *r);
enum ndr_err_code ndr_push_USER_INFO_21(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_21 *r);
enum ndr_err_code ndr_pull_USER_INFO_21(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_21 *r);
void ndr_print_USER_INFO_21(struct ndr_print *ndr, const char *name, const struct USER_INFO_21 *r);
enum ndr_err_code ndr_push_USER_INFO_22(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_22 *r);
enum ndr_err_code ndr_pull_USER_INFO_22(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_22 *r);
void ndr_print_USER_INFO_22(struct ndr_print *ndr, const char *name, const struct USER_INFO_22 *r);
enum ndr_err_code ndr_push_USER_INFO_23(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_23 *r);
enum ndr_err_code ndr_pull_USER_INFO_23(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_23 *r);
void ndr_print_USER_INFO_23(struct ndr_print *ndr, const char *name, const struct USER_INFO_23 *r);
enum ndr_err_code ndr_push_USER_INFO_1003(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1003 *r);
enum ndr_err_code ndr_pull_USER_INFO_1003(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1003 *r);
void ndr_print_USER_INFO_1003(struct ndr_print *ndr, const char *name, const struct USER_INFO_1003 *r);
enum ndr_err_code ndr_push_USER_INFO_1005(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1005 *r);
enum ndr_err_code ndr_pull_USER_INFO_1005(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1005 *r);
void ndr_print_USER_INFO_1005(struct ndr_print *ndr, const char *name, const struct USER_INFO_1005 *r);
enum ndr_err_code ndr_push_USER_INFO_1006(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1006 *r);
enum ndr_err_code ndr_pull_USER_INFO_1006(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1006 *r);
void ndr_print_USER_INFO_1006(struct ndr_print *ndr, const char *name, const struct USER_INFO_1006 *r);
enum ndr_err_code ndr_push_USER_INFO_1007(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1007 *r);
enum ndr_err_code ndr_pull_USER_INFO_1007(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1007 *r);
void ndr_print_USER_INFO_1007(struct ndr_print *ndr, const char *name, const struct USER_INFO_1007 *r);
enum ndr_err_code ndr_push_USER_INFO_1008(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1008 *r);
enum ndr_err_code ndr_pull_USER_INFO_1008(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1008 *r);
void ndr_print_USER_INFO_1008(struct ndr_print *ndr, const char *name, const struct USER_INFO_1008 *r);
enum ndr_err_code ndr_push_USER_INFO_1009(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1009 *r);
enum ndr_err_code ndr_pull_USER_INFO_1009(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1009 *r);
void ndr_print_USER_INFO_1009(struct ndr_print *ndr, const char *name, const struct USER_INFO_1009 *r);
enum ndr_err_code ndr_push_USER_INFO_1010(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1010 *r);
enum ndr_err_code ndr_pull_USER_INFO_1010(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1010 *r);
void ndr_print_USER_INFO_1010(struct ndr_print *ndr, const char *name, const struct USER_INFO_1010 *r);
enum ndr_err_code ndr_push_USER_INFO_1011(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1011 *r);
enum ndr_err_code ndr_pull_USER_INFO_1011(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1011 *r);
void ndr_print_USER_INFO_1011(struct ndr_print *ndr, const char *name, const struct USER_INFO_1011 *r);
enum ndr_err_code ndr_push_USER_INFO_1012(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1012 *r);
enum ndr_err_code ndr_pull_USER_INFO_1012(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1012 *r);
void ndr_print_USER_INFO_1012(struct ndr_print *ndr, const char *name, const struct USER_INFO_1012 *r);
enum ndr_err_code ndr_push_USER_INFO_1013(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1013 *r);
enum ndr_err_code ndr_pull_USER_INFO_1013(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1013 *r);
void ndr_print_USER_INFO_1013(struct ndr_print *ndr, const char *name, const struct USER_INFO_1013 *r);
enum ndr_err_code ndr_push_USER_INFO_1014(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1014 *r);
enum ndr_err_code ndr_pull_USER_INFO_1014(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1014 *r);
void ndr_print_USER_INFO_1014(struct ndr_print *ndr, const char *name, const struct USER_INFO_1014 *r);
enum ndr_err_code ndr_push_USER_INFO_1017(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1017 *r);
enum ndr_err_code ndr_pull_USER_INFO_1017(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1017 *r);
void ndr_print_USER_INFO_1017(struct ndr_print *ndr, const char *name, const struct USER_INFO_1017 *r);
enum ndr_err_code ndr_push_USER_INFO_1018(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1018 *r);
enum ndr_err_code ndr_pull_USER_INFO_1018(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1018 *r);
void ndr_print_USER_INFO_1018(struct ndr_print *ndr, const char *name, const struct USER_INFO_1018 *r);
enum ndr_err_code ndr_push_USER_INFO_1020(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1020 *r);
enum ndr_err_code ndr_pull_USER_INFO_1020(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1020 *r);
void ndr_print_USER_INFO_1020(struct ndr_print *ndr, const char *name, const struct USER_INFO_1020 *r);
enum ndr_err_code ndr_push_USER_INFO_1023(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1023 *r);
enum ndr_err_code ndr_pull_USER_INFO_1023(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1023 *r);
void ndr_print_USER_INFO_1023(struct ndr_print *ndr, const char *name, const struct USER_INFO_1023 *r);
enum ndr_err_code ndr_push_USER_INFO_1024(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1024 *r);
enum ndr_err_code ndr_pull_USER_INFO_1024(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1024 *r);
void ndr_print_USER_INFO_1024(struct ndr_print *ndr, const char *name, const struct USER_INFO_1024 *r);
enum ndr_err_code ndr_push_USER_INFO_1025(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1025 *r);
enum ndr_err_code ndr_pull_USER_INFO_1025(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1025 *r);
void ndr_print_USER_INFO_1025(struct ndr_print *ndr, const char *name, const struct USER_INFO_1025 *r);
enum ndr_err_code ndr_push_USER_INFO_1051(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1051 *r);
enum ndr_err_code ndr_pull_USER_INFO_1051(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1051 *r);
void ndr_print_USER_INFO_1051(struct ndr_print *ndr, const char *name, const struct USER_INFO_1051 *r);
enum ndr_err_code ndr_push_USER_INFO_1052(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1052 *r);
enum ndr_err_code ndr_pull_USER_INFO_1052(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1052 *r);
void ndr_print_USER_INFO_1052(struct ndr_print *ndr, const char *name, const struct USER_INFO_1052 *r);
enum ndr_err_code ndr_push_USER_INFO_1053(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_1053 *r);
enum ndr_err_code ndr_pull_USER_INFO_1053(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_1053 *r);
void ndr_print_USER_INFO_1053(struct ndr_print *ndr, const char *name, const struct USER_INFO_1053 *r);
enum ndr_err_code ndr_push_USER_INFO_X(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct USER_INFO_X *r);
enum ndr_err_code ndr_pull_USER_INFO_X(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct USER_INFO_X *r);
void ndr_print_USER_INFO_X(struct ndr_print *ndr, const char *name, const struct USER_INFO_X *r);
enum ndr_err_code ndr_push_GROUP_USERS_INFO_0(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct GROUP_USERS_INFO_0 *r);
enum ndr_err_code ndr_pull_GROUP_USERS_INFO_0(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct GROUP_USERS_INFO_0 *r);
void ndr_print_GROUP_USERS_INFO_0(struct ndr_print *ndr, const char *name, const struct GROUP_USERS_INFO_0 *r);
enum ndr_err_code ndr_push_GROUP_USERS_INFO_1(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct GROUP_USERS_INFO_1 *r);
enum ndr_err_code ndr_pull_GROUP_USERS_INFO_1(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct GROUP_USERS_INFO_1 *r);
void ndr_print_GROUP_USERS_INFO_1(struct ndr_print *ndr, const char *name, const struct GROUP_USERS_INFO_1 *r);
enum ndr_err_code ndr_push_NET_DISPLAY_USER(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct NET_DISPLAY_USER *r);
enum ndr_err_code ndr_pull_NET_DISPLAY_USER(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct NET_DISPLAY_USER *r);
void ndr_print_NET_DISPLAY_USER(struct ndr_print *ndr, const char *name, const struct NET_DISPLAY_USER *r);
enum ndr_err_code ndr_push_NET_DISPLAY_MACHINE(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct NET_DISPLAY_MACHINE *r);
enum ndr_err_code ndr_pull_NET_DISPLAY_MACHINE(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct NET_DISPLAY_MACHINE *r);
void ndr_print_NET_DISPLAY_MACHINE(struct ndr_print *ndr, const char *name, const struct NET_DISPLAY_MACHINE *r);
enum ndr_err_code ndr_push_NET_DISPLAY_GROUP(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct NET_DISPLAY_GROUP *r);
enum ndr_err_code ndr_pull_NET_DISPLAY_GROUP(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct NET_DISPLAY_GROUP *r);
void ndr_print_NET_DISPLAY_GROUP(struct ndr_print *ndr, const char *name, const struct NET_DISPLAY_GROUP *r);
enum ndr_err_code ndr_push_SHARE_INFO_1005_FLAGS(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint32_t r);
enum ndr_err_code ndr_pull_SHARE_INFO_1005_FLAGS(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint32_t *r);
void ndr_print_SHARE_INFO_1005_FLAGS(struct ndr_print *ndr, const char *name, uint32_t r);
enum ndr_err_code ndr_push_NetJoinDomain(struct ndr_push *ndr, ndr_flags_type flags, const struct NetJoinDomain *r);
enum ndr_err_code ndr_pull_NetJoinDomain(struct ndr_pull *ndr, ndr_flags_type flags, struct NetJoinDomain *r);
void ndr_print_NetJoinDomain(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetJoinDomain *r);
enum ndr_err_code ndr_push_NetUnjoinDomain(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUnjoinDomain *r);
enum ndr_err_code ndr_pull_NetUnjoinDomain(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUnjoinDomain *r);
void ndr_print_NetUnjoinDomain(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUnjoinDomain *r);
enum ndr_err_code ndr_push_NetGetJoinInformation(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGetJoinInformation *r);
enum ndr_err_code ndr_pull_NetGetJoinInformation(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGetJoinInformation *r);
void ndr_print_NetGetJoinInformation(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGetJoinInformation *r);
enum ndr_err_code ndr_push_NetGetJoinableOUs(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGetJoinableOUs *r);
enum ndr_err_code ndr_pull_NetGetJoinableOUs(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGetJoinableOUs *r);
void ndr_print_NetGetJoinableOUs(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGetJoinableOUs *r);
enum ndr_err_code ndr_push_NetRenameMachineInDomain(struct ndr_push *ndr, ndr_flags_type flags, const struct NetRenameMachineInDomain *r);
enum ndr_err_code ndr_pull_NetRenameMachineInDomain(struct ndr_pull *ndr, ndr_flags_type flags, struct NetRenameMachineInDomain *r);
void ndr_print_NetRenameMachineInDomain(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetRenameMachineInDomain *r);
enum ndr_err_code ndr_push_NetProvisionComputerAccount(struct ndr_push *ndr, ndr_flags_type flags, const struct NetProvisionComputerAccount *r);
enum ndr_err_code ndr_pull_NetProvisionComputerAccount(struct ndr_pull *ndr, ndr_flags_type flags, struct NetProvisionComputerAccount *r);
void ndr_print_NetProvisionComputerAccount(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetProvisionComputerAccount *r);
enum ndr_err_code ndr_push_NetRequestOfflineDomainJoin(struct ndr_push *ndr, ndr_flags_type flags, const struct NetRequestOfflineDomainJoin *r);
enum ndr_err_code ndr_pull_NetRequestOfflineDomainJoin(struct ndr_pull *ndr, ndr_flags_type flags, struct NetRequestOfflineDomainJoin *r);
void ndr_print_NetRequestOfflineDomainJoin(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetRequestOfflineDomainJoin *r);
enum ndr_err_code ndr_push_NetComposeOfflineDomainJoin(struct ndr_push *ndr, ndr_flags_type flags, const struct NetComposeOfflineDomainJoin *r);
enum ndr_err_code ndr_pull_NetComposeOfflineDomainJoin(struct ndr_pull *ndr, ndr_flags_type flags, struct NetComposeOfflineDomainJoin *r);
void ndr_print_NetComposeOfflineDomainJoin(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetComposeOfflineDomainJoin *r);
enum ndr_err_code ndr_push_NetServerGetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetServerGetInfo *r);
enum ndr_err_code ndr_pull_NetServerGetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetServerGetInfo *r);
void ndr_print_NetServerGetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetServerGetInfo *r);
enum ndr_err_code ndr_push_NetServerSetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetServerSetInfo *r);
enum ndr_err_code ndr_pull_NetServerSetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetServerSetInfo *r);
void ndr_print_NetServerSetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetServerSetInfo *r);
enum ndr_err_code ndr_push_NetWkstaGetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetWkstaGetInfo *r);
enum ndr_err_code ndr_pull_NetWkstaGetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetWkstaGetInfo *r);
void ndr_print_NetWkstaGetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetWkstaGetInfo *r);
enum ndr_err_code ndr_push_NetGetDCName(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGetDCName *r);
enum ndr_err_code ndr_pull_NetGetDCName(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGetDCName *r);
void ndr_print_NetGetDCName(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGetDCName *r);
enum ndr_err_code ndr_push_NetGetAnyDCName(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGetAnyDCName *r);
enum ndr_err_code ndr_pull_NetGetAnyDCName(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGetAnyDCName *r);
void ndr_print_NetGetAnyDCName(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGetAnyDCName *r);
enum ndr_err_code ndr_push_DsGetDcName(struct ndr_push *ndr, ndr_flags_type flags, const struct DsGetDcName *r);
enum ndr_err_code ndr_pull_DsGetDcName(struct ndr_pull *ndr, ndr_flags_type flags, struct DsGetDcName *r);
void ndr_print_DsGetDcName(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct DsGetDcName *r);
enum ndr_err_code ndr_push_NetUserAdd(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUserAdd *r);
enum ndr_err_code ndr_pull_NetUserAdd(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUserAdd *r);
void ndr_print_NetUserAdd(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUserAdd *r);
enum ndr_err_code ndr_push_NetUserDel(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUserDel *r);
enum ndr_err_code ndr_pull_NetUserDel(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUserDel *r);
void ndr_print_NetUserDel(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUserDel *r);
enum ndr_err_code ndr_push_NetUserEnum(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUserEnum *r);
enum ndr_err_code ndr_pull_NetUserEnum(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUserEnum *r);
void ndr_print_NetUserEnum(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUserEnum *r);
enum ndr_err_code ndr_push_NetUserChangePassword(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUserChangePassword *r);
enum ndr_err_code ndr_pull_NetUserChangePassword(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUserChangePassword *r);
void ndr_print_NetUserChangePassword(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUserChangePassword *r);
enum ndr_err_code ndr_push_NetUserGetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUserGetInfo *r);
enum ndr_err_code ndr_pull_NetUserGetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUserGetInfo *r);
void ndr_print_NetUserGetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUserGetInfo *r);
enum ndr_err_code ndr_push_NetUserSetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUserSetInfo *r);
enum ndr_err_code ndr_pull_NetUserSetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUserSetInfo *r);
void ndr_print_NetUserSetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUserSetInfo *r);
enum ndr_err_code ndr_push_NetUserGetGroups(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUserGetGroups *r);
enum ndr_err_code ndr_pull_NetUserGetGroups(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUserGetGroups *r);
void ndr_print_NetUserGetGroups(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUserGetGroups *r);
enum ndr_err_code ndr_push_NetUserSetGroups(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUserSetGroups *r);
enum ndr_err_code ndr_pull_NetUserSetGroups(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUserSetGroups *r);
void ndr_print_NetUserSetGroups(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUserSetGroups *r);
enum ndr_err_code ndr_push_NetUserGetLocalGroups(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUserGetLocalGroups *r);
enum ndr_err_code ndr_pull_NetUserGetLocalGroups(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUserGetLocalGroups *r);
void ndr_print_NetUserGetLocalGroups(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUserGetLocalGroups *r);
enum ndr_err_code ndr_push_NetUserModalsGet(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUserModalsGet *r);
enum ndr_err_code ndr_pull_NetUserModalsGet(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUserModalsGet *r);
void ndr_print_NetUserModalsGet(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUserModalsGet *r);
enum ndr_err_code ndr_push_NetUserModalsSet(struct ndr_push *ndr, ndr_flags_type flags, const struct NetUserModalsSet *r);
enum ndr_err_code ndr_pull_NetUserModalsSet(struct ndr_pull *ndr, ndr_flags_type flags, struct NetUserModalsSet *r);
void ndr_print_NetUserModalsSet(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetUserModalsSet *r);
enum ndr_err_code ndr_push_NetQueryDisplayInformation(struct ndr_push *ndr, ndr_flags_type flags, const struct NetQueryDisplayInformation *r);
enum ndr_err_code ndr_pull_NetQueryDisplayInformation(struct ndr_pull *ndr, ndr_flags_type flags, struct NetQueryDisplayInformation *r);
void ndr_print_NetQueryDisplayInformation(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetQueryDisplayInformation *r);
enum ndr_err_code ndr_push_NetGroupAdd(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGroupAdd *r);
enum ndr_err_code ndr_pull_NetGroupAdd(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGroupAdd *r);
void ndr_print_NetGroupAdd(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGroupAdd *r);
enum ndr_err_code ndr_push_NetGroupDel(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGroupDel *r);
enum ndr_err_code ndr_pull_NetGroupDel(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGroupDel *r);
void ndr_print_NetGroupDel(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGroupDel *r);
enum ndr_err_code ndr_push_NetGroupEnum(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGroupEnum *r);
enum ndr_err_code ndr_pull_NetGroupEnum(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGroupEnum *r);
void ndr_print_NetGroupEnum(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGroupEnum *r);
enum ndr_err_code ndr_push_NetGroupSetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGroupSetInfo *r);
enum ndr_err_code ndr_pull_NetGroupSetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGroupSetInfo *r);
void ndr_print_NetGroupSetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGroupSetInfo *r);
enum ndr_err_code ndr_push_NetGroupGetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGroupGetInfo *r);
enum ndr_err_code ndr_pull_NetGroupGetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGroupGetInfo *r);
void ndr_print_NetGroupGetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGroupGetInfo *r);
enum ndr_err_code ndr_push_NetGroupAddUser(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGroupAddUser *r);
enum ndr_err_code ndr_pull_NetGroupAddUser(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGroupAddUser *r);
void ndr_print_NetGroupAddUser(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGroupAddUser *r);
enum ndr_err_code ndr_push_NetGroupDelUser(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGroupDelUser *r);
enum ndr_err_code ndr_pull_NetGroupDelUser(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGroupDelUser *r);
void ndr_print_NetGroupDelUser(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGroupDelUser *r);
enum ndr_err_code ndr_push_NetGroupGetUsers(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGroupGetUsers *r);
enum ndr_err_code ndr_pull_NetGroupGetUsers(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGroupGetUsers *r);
void ndr_print_NetGroupGetUsers(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGroupGetUsers *r);
enum ndr_err_code ndr_push_NetGroupSetUsers(struct ndr_push *ndr, ndr_flags_type flags, const struct NetGroupSetUsers *r);
enum ndr_err_code ndr_pull_NetGroupSetUsers(struct ndr_pull *ndr, ndr_flags_type flags, struct NetGroupSetUsers *r);
void ndr_print_NetGroupSetUsers(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetGroupSetUsers *r);
enum ndr_err_code ndr_push_NetLocalGroupAdd(struct ndr_push *ndr, ndr_flags_type flags, const struct NetLocalGroupAdd *r);
enum ndr_err_code ndr_pull_NetLocalGroupAdd(struct ndr_pull *ndr, ndr_flags_type flags, struct NetLocalGroupAdd *r);
void ndr_print_NetLocalGroupAdd(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetLocalGroupAdd *r);
enum ndr_err_code ndr_push_NetLocalGroupDel(struct ndr_push *ndr, ndr_flags_type flags, const struct NetLocalGroupDel *r);
enum ndr_err_code ndr_pull_NetLocalGroupDel(struct ndr_pull *ndr, ndr_flags_type flags, struct NetLocalGroupDel *r);
void ndr_print_NetLocalGroupDel(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetLocalGroupDel *r);
enum ndr_err_code ndr_push_NetLocalGroupGetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetLocalGroupGetInfo *r);
enum ndr_err_code ndr_pull_NetLocalGroupGetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetLocalGroupGetInfo *r);
void ndr_print_NetLocalGroupGetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetLocalGroupGetInfo *r);
enum ndr_err_code ndr_push_NetLocalGroupSetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetLocalGroupSetInfo *r);
enum ndr_err_code ndr_pull_NetLocalGroupSetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetLocalGroupSetInfo *r);
void ndr_print_NetLocalGroupSetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetLocalGroupSetInfo *r);
enum ndr_err_code ndr_push_NetLocalGroupEnum(struct ndr_push *ndr, ndr_flags_type flags, const struct NetLocalGroupEnum *r);
enum ndr_err_code ndr_pull_NetLocalGroupEnum(struct ndr_pull *ndr, ndr_flags_type flags, struct NetLocalGroupEnum *r);
void ndr_print_NetLocalGroupEnum(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetLocalGroupEnum *r);
enum ndr_err_code ndr_push_NetLocalGroupAddMembers(struct ndr_push *ndr, ndr_flags_type flags, const struct NetLocalGroupAddMembers *r);
enum ndr_err_code ndr_pull_NetLocalGroupAddMembers(struct ndr_pull *ndr, ndr_flags_type flags, struct NetLocalGroupAddMembers *r);
void ndr_print_NetLocalGroupAddMembers(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetLocalGroupAddMembers *r);
enum ndr_err_code ndr_push_NetLocalGroupDelMembers(struct ndr_push *ndr, ndr_flags_type flags, const struct NetLocalGroupDelMembers *r);
enum ndr_err_code ndr_pull_NetLocalGroupDelMembers(struct ndr_pull *ndr, ndr_flags_type flags, struct NetLocalGroupDelMembers *r);
void ndr_print_NetLocalGroupDelMembers(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetLocalGroupDelMembers *r);
enum ndr_err_code ndr_push_NetLocalGroupGetMembers(struct ndr_push *ndr, ndr_flags_type flags, const struct NetLocalGroupGetMembers *r);
enum ndr_err_code ndr_pull_NetLocalGroupGetMembers(struct ndr_pull *ndr, ndr_flags_type flags, struct NetLocalGroupGetMembers *r);
void ndr_print_NetLocalGroupGetMembers(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetLocalGroupGetMembers *r);
enum ndr_err_code ndr_push_NetLocalGroupSetMembers(struct ndr_push *ndr, ndr_flags_type flags, const struct NetLocalGroupSetMembers *r);
enum ndr_err_code ndr_pull_NetLocalGroupSetMembers(struct ndr_pull *ndr, ndr_flags_type flags, struct NetLocalGroupSetMembers *r);
void ndr_print_NetLocalGroupSetMembers(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetLocalGroupSetMembers *r);
enum ndr_err_code ndr_push_NetRemoteTOD(struct ndr_push *ndr, ndr_flags_type flags, const struct NetRemoteTOD *r);
enum ndr_err_code ndr_pull_NetRemoteTOD(struct ndr_pull *ndr, ndr_flags_type flags, struct NetRemoteTOD *r);
void ndr_print_NetRemoteTOD(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetRemoteTOD *r);
enum ndr_err_code ndr_push_NetShareAdd(struct ndr_push *ndr, ndr_flags_type flags, const struct NetShareAdd *r);
enum ndr_err_code ndr_pull_NetShareAdd(struct ndr_pull *ndr, ndr_flags_type flags, struct NetShareAdd *r);
void ndr_print_NetShareAdd(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetShareAdd *r);
enum ndr_err_code ndr_push_NetShareDel(struct ndr_push *ndr, ndr_flags_type flags, const struct NetShareDel *r);
enum ndr_err_code ndr_pull_NetShareDel(struct ndr_pull *ndr, ndr_flags_type flags, struct NetShareDel *r);
void ndr_print_NetShareDel(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetShareDel *r);
enum ndr_err_code ndr_push_NetShareEnum(struct ndr_push *ndr, ndr_flags_type flags, const struct NetShareEnum *r);
enum ndr_err_code ndr_pull_NetShareEnum(struct ndr_pull *ndr, ndr_flags_type flags, struct NetShareEnum *r);
void ndr_print_NetShareEnum(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetShareEnum *r);
enum ndr_err_code ndr_push_NetShareGetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetShareGetInfo *r);
enum ndr_err_code ndr_pull_NetShareGetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetShareGetInfo *r);
void ndr_print_NetShareGetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetShareGetInfo *r);
enum ndr_err_code ndr_push_NetShareSetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetShareSetInfo *r);
enum ndr_err_code ndr_pull_NetShareSetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetShareSetInfo *r);
void ndr_print_NetShareSetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetShareSetInfo *r);
enum ndr_err_code ndr_push_NetFileClose(struct ndr_push *ndr, ndr_flags_type flags, const struct NetFileClose *r);
enum ndr_err_code ndr_pull_NetFileClose(struct ndr_pull *ndr, ndr_flags_type flags, struct NetFileClose *r);
void ndr_print_NetFileClose(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetFileClose *r);
enum ndr_err_code ndr_push_NetFileGetInfo(struct ndr_push *ndr, ndr_flags_type flags, const struct NetFileGetInfo *r);
enum ndr_err_code ndr_pull_NetFileGetInfo(struct ndr_pull *ndr, ndr_flags_type flags, struct NetFileGetInfo *r);
void ndr_print_NetFileGetInfo(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetFileGetInfo *r);
enum ndr_err_code ndr_push_NetFileEnum(struct ndr_push *ndr, ndr_flags_type flags, const struct NetFileEnum *r);
enum ndr_err_code ndr_pull_NetFileEnum(struct ndr_pull *ndr, ndr_flags_type flags, struct NetFileEnum *r);
void ndr_print_NetFileEnum(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetFileEnum *r);
enum ndr_err_code ndr_push_NetShutdownInit(struct ndr_push *ndr, ndr_flags_type flags, const struct NetShutdownInit *r);
enum ndr_err_code ndr_pull_NetShutdownInit(struct ndr_pull *ndr, ndr_flags_type flags, struct NetShutdownInit *r);
void ndr_print_NetShutdownInit(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetShutdownInit *r);
enum ndr_err_code ndr_push_NetShutdownAbort(struct ndr_push *ndr, ndr_flags_type flags, const struct NetShutdownAbort *r);
enum ndr_err_code ndr_pull_NetShutdownAbort(struct ndr_pull *ndr, ndr_flags_type flags, struct NetShutdownAbort *r);
void ndr_print_NetShutdownAbort(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct NetShutdownAbort *r);
enum ndr_err_code ndr_push_I_NetLogonControl(struct ndr_push *ndr, ndr_flags_type flags, const struct I_NetLogonControl *r);
enum ndr_err_code ndr_pull_I_NetLogonControl(struct ndr_pull *ndr, ndr_flags_type flags, struct I_NetLogonControl *r);
void ndr_print_I_NetLogonControl(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct I_NetLogonControl *r);
enum ndr_err_code ndr_push_I_NetLogonControl2(struct ndr_push *ndr, ndr_flags_type flags, const struct I_NetLogonControl2 *r);
enum ndr_err_code ndr_pull_I_NetLogonControl2(struct ndr_pull *ndr, ndr_flags_type flags, struct I_NetLogonControl2 *r);
void ndr_print_I_NetLogonControl2(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct I_NetLogonControl2 *r);
#endif /* _HEADER_NDR_libnetapi */
