/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/auth.h"

#ifndef _HEADER_NDR_auth
#define _HEADER_NDR_auth

#include "../librpc/ndr/ndr_auth.h"
extern const struct ndr_interface_table ndr_table_auth;
#define NDR_AUTH_CALL_COUNT (0)
enum ndr_err_code ndr_push_auth_method(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum auth_method r);
enum ndr_err_code ndr_pull_auth_method(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum auth_method *r);
void ndr_print_auth_method(struct ndr_print *ndr, const char *name, enum auth_method r);
enum ndr_err_code ndr_push_auth_user_info(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct auth_user_info *r);
enum ndr_err_code ndr_pull_auth_user_info(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct auth_user_info *r);
void ndr_print_auth_user_info(struct ndr_print *ndr, const char *name, const struct auth_user_info *r);
enum ndr_err_code ndr_push_auth_user_info_torture(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct auth_user_info_torture *r);
enum ndr_err_code ndr_pull_auth_user_info_torture(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct auth_user_info_torture *r);
void ndr_print_auth_user_info_torture(struct ndr_print *ndr, const char *name, const struct auth_user_info_torture *r);
enum ndr_err_code ndr_push_auth_user_info_unix(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct auth_user_info_unix *r);
enum ndr_err_code ndr_pull_auth_user_info_unix(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct auth_user_info_unix *r);
void ndr_print_auth_user_info_unix(struct ndr_print *ndr, const char *name, const struct auth_user_info_unix *r);
void ndr_print_ticket_type(struct ndr_print *ndr, const char *name, enum ticket_type r);
enum ndr_err_code ndr_push_auth_sid_origin(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum auth_sid_origin r);
enum ndr_err_code ndr_pull_auth_sid_origin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum auth_sid_origin *r);
void ndr_print_auth_sid_origin(struct ndr_print *ndr, const char *name, enum auth_sid_origin r);
enum ndr_err_code ndr_push_auth_SidAttr(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct auth_SidAttr *r);
enum ndr_err_code ndr_pull_auth_SidAttr(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct auth_SidAttr *r);
void ndr_print_auth_SidAttr(struct ndr_print *ndr, const char *name, const struct auth_SidAttr *r);
enum ndr_err_code ndr_push_auth_user_info_dc(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct auth_user_info_dc *r);
enum ndr_err_code ndr_pull_auth_user_info_dc(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct auth_user_info_dc *r);
void ndr_print_auth_user_info_dc(struct ndr_print *ndr, const char *name, const struct auth_user_info_dc *r);
enum ndr_err_code ndr_push_auth_session_info(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct auth_session_info *r);
enum ndr_err_code ndr_pull_auth_session_info(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct auth_session_info *r);
void ndr_print_auth_session_info(struct ndr_print *ndr, const char *name, const struct auth_session_info *r);
enum ndr_err_code ndr_push_auth_session_info_transport(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct auth_session_info_transport *r);
enum ndr_err_code ndr_pull_auth_session_info_transport(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct auth_session_info_transport *r);
void ndr_print_auth_session_info_transport(struct ndr_print *ndr, const char *name, const struct auth_session_info_transport *r);
#endif /* _HEADER_NDR_auth */
