/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/conditional_ace.h"

#ifndef _HEADER_NDR_conditional_ace
#define _HEADER_NDR_conditional_ace

extern const struct ndr_interface_table ndr_table_conditional_ace;
#define NDR_CONDITIONAL_ACE_CALL_COUNT (0)
void ndr_print_token_type(struct ndr_print *ndr, const char *name, enum token_type r);
void ndr_print_ace_condition_result(struct ndr_print *ndr, const char *name, const struct ace_condition_result *r);
enum ndr_err_code ndr_push_ace_condition_int(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ace_condition_int *r);
enum ndr_err_code ndr_pull_ace_condition_int(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ace_condition_int *r);
void ndr_print_ace_condition_int(struct ndr_print *ndr, const char *name, const struct ace_condition_int *r);
enum ndr_err_code ndr_push_ace_condition_unicode(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ace_condition_unicode *r);
enum ndr_err_code ndr_pull_ace_condition_unicode(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ace_condition_unicode *r);
void ndr_print_ace_condition_unicode(struct ndr_print *ndr, const char *name, const struct ace_condition_unicode *r);
enum ndr_err_code ndr_push_ace_condition_sid(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ace_condition_sid *r);
enum ndr_err_code ndr_pull_ace_condition_sid(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ace_condition_sid *r);
void ndr_print_ace_condition_sid(struct ndr_print *ndr, const char *name, const struct ace_condition_sid *r);
void ndr_print_ace_condition_composite(struct ndr_print *ndr, const char *name, const struct ace_condition_composite *r);
void ndr_print_ace_condition_op(struct ndr_print *ndr, const char *name, const struct ace_condition_op *r);
void ndr_print_ace_condition_sddl_op(struct ndr_print *ndr, const char *name, const struct ace_condition_sddl_op *r);
void ndr_print_ace_condition_token_data(struct ndr_print *ndr, const char *name, const union ace_condition_token_data *r);
enum ndr_err_code ndr_push_ace_condition_token(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ace_condition_token *r);
enum ndr_err_code ndr_pull_ace_condition_token(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ace_condition_token *r);
void ndr_print_ace_condition_token(struct ndr_print *ndr, const char *name, const struct ace_condition_token *r);
enum ndr_err_code ndr_push_ace_condition_script(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct ace_condition_script *r);
enum ndr_err_code ndr_pull_ace_condition_script(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct ace_condition_script *r);
void ndr_print_ace_condition_script(struct ndr_print *ndr, const char *name, const struct ace_condition_script *r);
#endif /* _HEADER_NDR_conditional_ace */
