#ifndef __SOURCE4_TORTURE_UNIX_PROTO_H__
#define __SOURCE4_TORTURE_UNIX_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../source4/torture/unix/unix.c  */

NTSTATUS torture_unix_init(TALLOC_CTX *ctx);

/* The following definitions come from ../../source4/torture/unix/whoami.c  */

bool torture_unix_whoami(struct torture_context *torture);

/* The following definitions come from ../../source4/torture/unix/unix_info2.c  */

bool unix_torture_unix_info2(struct torture_context *torture);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_TORTURE_UNIX_PROTO_H__ */

