/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/frsapi.h"

#ifndef _HEADER_NDR_frsapi
#define _HEADER_NDR_frsapi

#define NDR_FRSAPI_UUID "d049b186-814f-11d1-9a3c-00c04fc9b232"
#define NDR_FRSAPI_VERSION 65537
#define NDR_FRSAPI_NAME "frsapi"
#define NDR_FRSAPI_HELPSTRING "File Replication API"
extern const struct ndr_interface_table ndr_table_frsapi;
#define NDR_FRSAPI_VERIFY_PROMOTION (0x00)

#define NDR_FRSAPI_PROMOTION_STATUS (0x01)

#define NDR_FRSAPI_START_DEMOTION (0x02)

#define NDR_FRSAPI_COMMIT_DEMOTION (0x03)

#define NDR_FRSAPI_SETDSPOLLINGINTERVALW (0x04)

#define NDR_FRSAPI_GETDSPOLLINGINTERVALW (0x05)

#define NDR_FRSAPI_VERIFY_PROMOTION_W (0x06)

#define NDR_FRSAPI_INFOW (0x07)

#define NDR_FRSAPI_ISPATHREPLICATED (0x08)

#define NDR_FRSAPI_WRITERCOMMAND (0x09)

#define NDR_FRSAPI_FORCEREPLICATION (0x0a)

#define NDR_FRSAPI_CALL_COUNT (11)
void ndr_print_frsapi_InfoEnum(struct ndr_print *ndr, const char *name, enum frsapi_InfoEnum r);
void ndr_print_frsapi_Info(struct ndr_print *ndr, const char *name, const struct frsapi_Info *r);
void ndr_print_frsapi_ReplicaSetType(struct ndr_print *ndr, const char *name, enum frsapi_ReplicaSetType r);
void ndr_print_frsapi_WriterCommandsValues(struct ndr_print *ndr, const char *name, enum frsapi_WriterCommandsValues r);
void ndr_print_FRSAPI_VERIFY_PROMOTION(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct FRSAPI_VERIFY_PROMOTION *r);
void ndr_print_FRSAPI_PROMOTION_STATUS(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct FRSAPI_PROMOTION_STATUS *r);
void ndr_print_FRSAPI_START_DEMOTION(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct FRSAPI_START_DEMOTION *r);
void ndr_print_FRSAPI_COMMIT_DEMOTION(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct FRSAPI_COMMIT_DEMOTION *r);
void ndr_print_frsapi_SetDsPollingIntervalW(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct frsapi_SetDsPollingIntervalW *r);
void ndr_print_frsapi_GetDsPollingIntervalW(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct frsapi_GetDsPollingIntervalW *r);
void ndr_print_FRSAPI_VERIFY_PROMOTION_W(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct FRSAPI_VERIFY_PROMOTION_W *r);
void ndr_print_frsapi_InfoW(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct frsapi_InfoW *r);
void ndr_print_frsapi_IsPathReplicated(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct frsapi_IsPathReplicated *r);
void ndr_print_frsapi_WriterCommand(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct frsapi_WriterCommand *r);
void ndr_print_frsapi_ForceReplication(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct frsapi_ForceReplication *r);
#endif /* _HEADER_NDR_frsapi */
