/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/mdssvc.h"

#ifndef _HEADER_NDR_mdssvc
#define _HEADER_NDR_mdssvc

#define NDR_MDSSVC_UUID "885d85fb-c754-4062-a0e7-6872ce0064f4"
#define NDR_MDSSVC_VERSION 2.0
#define NDR_MDSSVC_NAME "mdssvc"
#define NDR_MDSSVC_HELPSTRING "Spotlight metadata search service"
extern const struct ndr_interface_table ndr_table_mdssvc;
#define NDR_MDSSVC_OPEN (0x00)

#define NDR_MDSSVC_UNKNOWN1 (0x01)

#define NDR_MDSSVC_CMD (0x02)

#define NDR_MDSSVC_CLOSE (0x03)

#define NDR_MDSSVC_CALL_COUNT (4)
enum ndr_err_code ndr_push_mdssvc_blob(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct mdssvc_blob *r);
enum ndr_err_code ndr_pull_mdssvc_blob(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct mdssvc_blob *r);
void ndr_print_mdssvc_blob(struct ndr_print *ndr, const char *name, const struct mdssvc_blob *r);
void ndr_print_mdssvc_open(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct mdssvc_open *r);
void ndr_print_mdssvc_unknown1(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct mdssvc_unknown1 *r);
void ndr_print_mdssvc_cmd(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct mdssvc_cmd *r);
void ndr_print_mdssvc_close(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct mdssvc_close *r);
#endif /* _HEADER_NDR_mdssvc */
