/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/nfs4acl.h"

#ifndef _HEADER_NDR_nfs4acl_interface
#define _HEADER_NDR_nfs4acl_interface

extern const struct ndr_interface_table ndr_table_nfs4acl_interface;
#define NDR_NFS4ACL_INTERFACE_CALL_COUNT (0)
void ndr_print_nfs4ace(struct ndr_print *ndr, const char *name, const struct nfs4ace *r);
enum ndr_err_code ndr_push_nfs4acl(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct nfs4acl *r);
enum ndr_err_code ndr_pull_nfs4acl(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct nfs4acl *r);
void ndr_print_nfs4acl(struct ndr_print *ndr, const char *name, const struct nfs4acl *r);
#endif /* _HEADER_NDR_nfs4acl_interface */
