/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/claims.h"

#ifndef _HEADER_NDR_claims
#define _HEADER_NDR_claims

#include "../librpc/ndr/ndr_claims.h"
#define NDR_CLAIMS_UUID "bba9cb76-eb0c-462c-aa1b-5d8c34415701"
#define NDR_CLAIMS_VERSION 1.0
#define NDR_CLAIMS_NAME "claims"
#define NDR_CLAIMS_HELPSTRING "Active Directory Claims"
extern const struct ndr_interface_table ndr_table_claims;
#define NDR_CLAIMS_CALL_COUNT (0)
void ndr_print_CLAIM_TYPE(struct ndr_print *ndr, const char *name, enum CLAIM_TYPE r);
void ndr_print_CLAIMS_SOURCE_TYPE(struct ndr_print *ndr, const char *name, enum CLAIMS_SOURCE_TYPE r);
void ndr_print_CLAIMS_COMPRESSION_FORMAT(struct ndr_print *ndr, const char *name, enum CLAIMS_COMPRESSION_FORMAT r);
void ndr_print_CLAIM_INT64(struct ndr_print *ndr, const char *name, const struct CLAIM_INT64 *r);
void ndr_print_CLAIM_UINT64(struct ndr_print *ndr, const char *name, const struct CLAIM_UINT64 *r);
void ndr_print_CLAIM_STRING(struct ndr_print *ndr, const char *name, const struct CLAIM_STRING *r);
void ndr_print_CLAIM_ENTRY_VALUES(struct ndr_print *ndr, const char *name, const union CLAIM_ENTRY_VALUES *r);
void ndr_print_CLAIM_ENTRY(struct ndr_print *ndr, const char *name, const struct CLAIM_ENTRY *r);
void ndr_print_CLAIMS_ARRAY(struct ndr_print *ndr, const char *name, const struct CLAIMS_ARRAY *r);
void ndr_print_CLAIMS_SET_METADATA_CTR(struct ndr_print *ndr, const char *name, const struct CLAIMS_SET_METADATA_CTR *r);
void ndr_print_CLAIMS_SET_CTR(struct ndr_print *ndr, const char *name, const struct CLAIMS_SET_CTR *r);
enum ndr_err_code ndr_push_CLAIMS_SET(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct CLAIMS_SET *r);
enum ndr_err_code ndr_pull_CLAIMS_SET(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct CLAIMS_SET *r);
void ndr_print_CLAIMS_SET(struct ndr_print *ndr, const char *name, const struct CLAIMS_SET *r);
enum ndr_err_code ndr_push_CLAIMS_SET_NDR(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct CLAIMS_SET_NDR *r);
enum ndr_err_code ndr_pull_CLAIMS_SET_NDR(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct CLAIMS_SET_NDR *r);
void ndr_print_CLAIMS_SET_NDR(struct ndr_print *ndr, const char *name, const struct CLAIMS_SET_NDR *r);
size_t ndr_size_CLAIMS_SET_NDR(const struct CLAIMS_SET_NDR *r, libndr_flags flags);
enum ndr_err_code ndr_push_CLAIMS_SET_METADATA_NDR(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct CLAIMS_SET_METADATA_NDR *r);
enum ndr_err_code ndr_pull_CLAIMS_SET_METADATA_NDR(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct CLAIMS_SET_METADATA_NDR *r);
void ndr_print_CLAIMS_SET_METADATA_NDR(struct ndr_print *ndr, const char *name, const struct CLAIMS_SET_METADATA_NDR *r);
enum ndr_err_code ndr_push_CLAIMS_SET_METADATA(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct CLAIMS_SET_METADATA *r);
enum ndr_err_code ndr_pull_CLAIMS_SET_METADATA(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct CLAIMS_SET_METADATA *r);
void ndr_print_CLAIMS_SET_METADATA(struct ndr_print *ndr, const char *name, const struct CLAIMS_SET_METADATA *r);
enum ndr_err_code ndr_push_claims_tf_condition_operator(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum claims_tf_condition_operator r);
enum ndr_err_code ndr_pull_claims_tf_condition_operator(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum claims_tf_condition_operator *r);
void ndr_print_claims_tf_condition_operator(struct ndr_print *ndr, const char *name, enum claims_tf_condition_operator r);
enum ndr_err_code ndr_push_claims_tf_property_enum(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum claims_tf_property_enum r);
enum ndr_err_code ndr_pull_claims_tf_property_enum(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum claims_tf_property_enum *r);
void ndr_print_claims_tf_property_enum(struct ndr_print *ndr, const char *name, enum claims_tf_property_enum r);
enum ndr_err_code ndr_push_claims_tf_condition(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct claims_tf_condition *r);
enum ndr_err_code ndr_pull_claims_tf_condition(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct claims_tf_condition *r);
void ndr_print_claims_tf_condition(struct ndr_print *ndr, const char *name, const struct claims_tf_condition *r);
enum ndr_err_code ndr_push_claims_tf_condition_set(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct claims_tf_condition_set *r);
enum ndr_err_code ndr_pull_claims_tf_condition_set(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct claims_tf_condition_set *r);
void ndr_print_claims_tf_condition_set(struct ndr_print *ndr, const char *name, const struct claims_tf_condition_set *r);
enum ndr_err_code ndr_push_claims_tf_property_ref(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct claims_tf_property_ref *r);
enum ndr_err_code ndr_pull_claims_tf_property_ref(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct claims_tf_property_ref *r);
void ndr_print_claims_tf_property_ref(struct ndr_print *ndr, const char *name, const struct claims_tf_property_ref *r);
enum ndr_err_code ndr_push_claims_tf_property(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct claims_tf_property *r);
enum ndr_err_code ndr_pull_claims_tf_property(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct claims_tf_property *r);
void ndr_print_claims_tf_property(struct ndr_print *ndr, const char *name, const struct claims_tf_property *r);
enum ndr_err_code ndr_push_claims_tf_rule_action(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct claims_tf_rule_action *r);
enum ndr_err_code ndr_pull_claims_tf_rule_action(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct claims_tf_rule_action *r);
void ndr_print_claims_tf_rule_action(struct ndr_print *ndr, const char *name, const struct claims_tf_rule_action *r);
enum ndr_err_code ndr_push_claims_tf_rule(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct claims_tf_rule *r);
enum ndr_err_code ndr_pull_claims_tf_rule(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct claims_tf_rule *r);
void ndr_print_claims_tf_rule(struct ndr_print *ndr, const char *name, const struct claims_tf_rule *r);
enum ndr_err_code ndr_push_claims_tf_rule_set(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct claims_tf_rule_set *r);
enum ndr_err_code ndr_pull_claims_tf_rule_set(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct claims_tf_rule_set *r);
void ndr_print_claims_tf_rule_set(struct ndr_print *ndr, const char *name, const struct claims_tf_rule_set *r);
#endif /* _HEADER_NDR_claims */
