#ifndef __NDR_NETLOGON_SCOMPAT_H__
#define __NDR_NETLOGON_SCOMPAT_H__

struct pipes_struct;
struct dcesrv_endpoint_server;
struct dcesrv_call_state;

const struct dcesrv_endpoint_server *netlogon_get_ep_server(void);

NTSTATUS netlogon__op_ndr_pull(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_pull *pull, void **r);
struct netr_LogonUasLogon;
WERROR _netr_LogonUasLogon(struct pipes_struct *p, struct netr_LogonUasLogon *r);
struct netr_LogonUasLogoff;
WERROR _netr_LogonUasLogoff(struct pipes_struct *p, struct netr_LogonUasLogoff *r);
struct netr_LogonSamLogon;
NTSTATUS _netr_LogonSamLogon(struct pipes_struct *p, struct netr_LogonSamLogon *r);
struct netr_LogonSamLogoff;
NTSTATUS _netr_LogonSamLogoff(struct pipes_struct *p, struct netr_LogonSamLogoff *r);
struct netr_ServerReqChallenge;
NTSTATUS _netr_ServerReqChallenge(struct pipes_struct *p, struct netr_ServerReqChallenge *r);
struct netr_ServerAuthenticate;
NTSTATUS _netr_ServerAuthenticate(struct pipes_struct *p, struct netr_ServerAuthenticate *r);
struct netr_ServerPasswordSet;
NTSTATUS _netr_ServerPasswordSet(struct pipes_struct *p, struct netr_ServerPasswordSet *r);
struct netr_DatabaseDeltas;
NTSTATUS _netr_DatabaseDeltas(struct pipes_struct *p, struct netr_DatabaseDeltas *r);
struct netr_DatabaseSync;
NTSTATUS _netr_DatabaseSync(struct pipes_struct *p, struct netr_DatabaseSync *r);
struct netr_AccountDeltas;
NTSTATUS _netr_AccountDeltas(struct pipes_struct *p, struct netr_AccountDeltas *r);
struct netr_AccountSync;
NTSTATUS _netr_AccountSync(struct pipes_struct *p, struct netr_AccountSync *r);
struct netr_GetDcName;
WERROR _netr_GetDcName(struct pipes_struct *p, struct netr_GetDcName *r);
struct netr_LogonControl;
WERROR _netr_LogonControl(struct pipes_struct *p, struct netr_LogonControl *r);
struct netr_GetAnyDCName;
WERROR _netr_GetAnyDCName(struct pipes_struct *p, struct netr_GetAnyDCName *r);
struct netr_LogonControl2;
WERROR _netr_LogonControl2(struct pipes_struct *p, struct netr_LogonControl2 *r);
struct netr_ServerAuthenticate2;
NTSTATUS _netr_ServerAuthenticate2(struct pipes_struct *p, struct netr_ServerAuthenticate2 *r);
struct netr_DatabaseSync2;
NTSTATUS _netr_DatabaseSync2(struct pipes_struct *p, struct netr_DatabaseSync2 *r);
struct netr_DatabaseRedo;
NTSTATUS _netr_DatabaseRedo(struct pipes_struct *p, struct netr_DatabaseRedo *r);
struct netr_LogonControl2Ex;
WERROR _netr_LogonControl2Ex(struct pipes_struct *p, struct netr_LogonControl2Ex *r);
struct netr_NetrEnumerateTrustedDomains;
NTSTATUS _netr_NetrEnumerateTrustedDomains(struct pipes_struct *p, struct netr_NetrEnumerateTrustedDomains *r);
struct netr_DsRGetDCName;
WERROR _netr_DsRGetDCName(struct pipes_struct *p, struct netr_DsRGetDCName *r);
struct netr_LogonGetCapabilities;
NTSTATUS _netr_LogonGetCapabilities(struct pipes_struct *p, struct netr_LogonGetCapabilities *r);
struct netr_NETRLOGONSETSERVICEBITS;
WERROR _netr_NETRLOGONSETSERVICEBITS(struct pipes_struct *p, struct netr_NETRLOGONSETSERVICEBITS *r);
struct netr_LogonGetTrustRid;
WERROR _netr_LogonGetTrustRid(struct pipes_struct *p, struct netr_LogonGetTrustRid *r);
struct netr_NETRLOGONCOMPUTESERVERDIGEST;
WERROR _netr_NETRLOGONCOMPUTESERVERDIGEST(struct pipes_struct *p, struct netr_NETRLOGONCOMPUTESERVERDIGEST *r);
struct netr_NETRLOGONCOMPUTECLIENTDIGEST;
WERROR _netr_NETRLOGONCOMPUTECLIENTDIGEST(struct pipes_struct *p, struct netr_NETRLOGONCOMPUTECLIENTDIGEST *r);
struct netr_ServerAuthenticate3;
NTSTATUS _netr_ServerAuthenticate3(struct pipes_struct *p, struct netr_ServerAuthenticate3 *r);
struct netr_DsRGetDCNameEx;
WERROR _netr_DsRGetDCNameEx(struct pipes_struct *p, struct netr_DsRGetDCNameEx *r);
struct netr_DsRGetSiteName;
WERROR _netr_DsRGetSiteName(struct pipes_struct *p, struct netr_DsRGetSiteName *r);
struct netr_LogonGetDomainInfo;
NTSTATUS _netr_LogonGetDomainInfo(struct pipes_struct *p, struct netr_LogonGetDomainInfo *r);
struct netr_ServerPasswordSet2;
NTSTATUS _netr_ServerPasswordSet2(struct pipes_struct *p, struct netr_ServerPasswordSet2 *r);
struct netr_ServerPasswordGet;
NTSTATUS _netr_ServerPasswordGet(struct pipes_struct *p, struct netr_ServerPasswordGet *r);
struct netr_NetrLogonSendToSam;
NTSTATUS _netr_NetrLogonSendToSam(struct pipes_struct *p, struct netr_NetrLogonSendToSam *r);
struct netr_DsRAddressToSitenamesW;
WERROR _netr_DsRAddressToSitenamesW(struct pipes_struct *p, struct netr_DsRAddressToSitenamesW *r);
struct netr_DsRGetDCNameEx2;
WERROR _netr_DsRGetDCNameEx2(struct pipes_struct *p, struct netr_DsRGetDCNameEx2 *r);
struct netr_NETRLOGONGETTIMESERVICEPARENTDOMAIN;
WERROR _netr_NETRLOGONGETTIMESERVICEPARENTDOMAIN(struct pipes_struct *p, struct netr_NETRLOGONGETTIMESERVICEPARENTDOMAIN *r);
struct netr_NetrEnumerateTrustedDomainsEx;
WERROR _netr_NetrEnumerateTrustedDomainsEx(struct pipes_struct *p, struct netr_NetrEnumerateTrustedDomainsEx *r);
struct netr_DsRAddressToSitenamesExW;
WERROR _netr_DsRAddressToSitenamesExW(struct pipes_struct *p, struct netr_DsRAddressToSitenamesExW *r);
struct netr_DsrGetDcSiteCoverageW;
WERROR _netr_DsrGetDcSiteCoverageW(struct pipes_struct *p, struct netr_DsrGetDcSiteCoverageW *r);
struct netr_LogonSamLogonEx;
NTSTATUS _netr_LogonSamLogonEx(struct pipes_struct *p, struct netr_LogonSamLogonEx *r);
struct netr_DsrEnumerateDomainTrusts;
WERROR _netr_DsrEnumerateDomainTrusts(struct pipes_struct *p, struct netr_DsrEnumerateDomainTrusts *r);
struct netr_DsrDeregisterDNSHostRecords;
WERROR _netr_DsrDeregisterDNSHostRecords(struct pipes_struct *p, struct netr_DsrDeregisterDNSHostRecords *r);
struct netr_ServerTrustPasswordsGet;
NTSTATUS _netr_ServerTrustPasswordsGet(struct pipes_struct *p, struct netr_ServerTrustPasswordsGet *r);
struct netr_DsRGetForestTrustInformation;
WERROR _netr_DsRGetForestTrustInformation(struct pipes_struct *p, struct netr_DsRGetForestTrustInformation *r);
struct netr_GetForestTrustInformation;
NTSTATUS _netr_GetForestTrustInformation(struct pipes_struct *p, struct netr_GetForestTrustInformation *r);
struct netr_LogonSamLogonWithFlags;
NTSTATUS _netr_LogonSamLogonWithFlags(struct pipes_struct *p, struct netr_LogonSamLogonWithFlags *r);
struct netr_ServerGetTrustInfo;
NTSTATUS _netr_ServerGetTrustInfo(struct pipes_struct *p, struct netr_ServerGetTrustInfo *r);
struct netr_Unused47;
NTSTATUS _netr_Unused47(struct pipes_struct *p, struct netr_Unused47 *r);
struct netr_DsrUpdateReadOnlyServerDnsRecords;
NTSTATUS _netr_DsrUpdateReadOnlyServerDnsRecords(struct pipes_struct *p, struct netr_DsrUpdateReadOnlyServerDnsRecords *r);
struct netr_Opnum49NotUsedOnWire;
void _netr_Opnum49NotUsedOnWire(struct pipes_struct *p, struct netr_Opnum49NotUsedOnWire *r);
struct netr_Opnum50NotUsedOnWire;
void _netr_Opnum50NotUsedOnWire(struct pipes_struct *p, struct netr_Opnum50NotUsedOnWire *r);
struct netr_Opnum51NotUsedOnWire;
void _netr_Opnum51NotUsedOnWire(struct pipes_struct *p, struct netr_Opnum51NotUsedOnWire *r);
struct netr_Opnum52NotUsedOnWire;
void _netr_Opnum52NotUsedOnWire(struct pipes_struct *p, struct netr_Opnum52NotUsedOnWire *r);
struct netr_Opnum53NotUsedOnWire;
void _netr_Opnum53NotUsedOnWire(struct pipes_struct *p, struct netr_Opnum53NotUsedOnWire *r);
struct netr_ChainSetClientAttributes;
NTSTATUS _netr_ChainSetClientAttributes(struct pipes_struct *p, struct netr_ChainSetClientAttributes *r);
struct netr_Opnum55NotUsedOnWire;
void _netr_Opnum55NotUsedOnWire(struct pipes_struct *p, struct netr_Opnum55NotUsedOnWire *r);
struct netr_Opnum56NotUsedOnWire;
void _netr_Opnum56NotUsedOnWire(struct pipes_struct *p, struct netr_Opnum56NotUsedOnWire *r);
struct netr_Opnum57NotUsedOnWire;
void _netr_Opnum57NotUsedOnWire(struct pipes_struct *p, struct netr_Opnum57NotUsedOnWire *r);
struct netr_Opnum58NotUsedOnWire;
void _netr_Opnum58NotUsedOnWire(struct pipes_struct *p, struct netr_Opnum58NotUsedOnWire *r);
struct netr_ServerAuthenticateKerberos;
NTSTATUS _netr_ServerAuthenticateKerberos(struct pipes_struct *p, struct netr_ServerAuthenticateKerberos *r);
NTSTATUS netlogon__op_dispatch(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS netlogon__op_reply(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
NTSTATUS netlogon__op_ndr_push(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, struct ndr_push *push, const void *r);
NTSTATUS netlogon__op_local(struct dcesrv_call_state *dce_call, TALLOC_CTX *mem_ctx, void *r);
#endif /* __NDR_NETLOGON_SCOMPAT_H__ */
