#ifndef __LIB_LDB_SAMBA_LDIF_HANDLERS_PROTO_H__
#define __LIB_LDB_SAMBA_LDIF_HANDLERS_PROTO_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../lib/ldb-samba/ldif_handlers.c  */

int ldif_write_objectSid(struct ldb_context *ldb, void *mem_ctx,
				const struct ldb_val *in, struct ldb_val *out);
const struct ldb_schema_syntax *ldb_samba_syntax_by_name(struct ldb_context *ldb, const char *name);
const struct ldb_schema_syntax *ldb_samba_syntax_by_lDAPDisplayName(struct ldb_context *ldb, const char *name);
int ldb_register_samba_handlers(struct ldb_context *ldb);

/* The following definitions come from ../../lib/ldb-samba/ldb_matching_rules.c  */

int ldb_register_samba_matching_rules(struct ldb_context *ldb);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __LIB_LDB_SAMBA_LDIF_HANDLERS_PROTO_H__ */

