/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_rot
#define _PIDL_HEADER_rot

#include <stdint.h>


#include "librpc/gen_ndr/orpc.h"
#ifndef _HEADER_rot
#define _HEADER_rot


struct rot_add {
	struct {
		uint32_t flags;
		struct MInterfacePointer *unk;/* [ref] */
		struct MInterfacePointer *moniker;/* [ref] */
	} in;

	struct {
		uint32_t *rotid;/* [ref] */
		WERROR result;
	} out;

};


struct rot_remove {
	struct {
		uint32_t rotid;
	} in;

	struct {
		WERROR result;
	} out;

};


struct rot_is_listed {
	struct {
		struct MInterfacePointer *moniker;/* [ref] */
	} in;

	struct {
		WERROR result;
	} out;

};


struct rot_get_interface_pointer {
	struct {
		struct MInterfacePointer *moniker;/* [ref] */
	} in;

	struct {
		struct MInterfacePointer *ip;/* [ref] */
		WERROR result;
	} out;

};


struct rot_set_modification_time {
	struct {
		uint32_t rotid;
		NTTIME *t;/* [ref] */
	} in;

	struct {
		WERROR result;
	} out;

};


struct rot_get_modification_time {
	struct {
		struct MInterfacePointer *moniker;/* [ref] */
	} in;

	struct {
		NTTIME *t;/* [ref] */
		WERROR result;
	} out;

};


struct rot_enum {
	struct {
		struct MInterfacePointer *EnumMoniker;/* [ref] */
		WERROR result;
	} out;

};

#endif /* _HEADER_rot */
#endif /* _PIDL_HEADER_rot */
