/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_winstation
#define _PIDL_HEADER_winstation

#include <stdint.h>


#ifndef _HEADER_winstation
#define _HEADER_winstation


struct winstation_foo {
	int _dummy_element;
};

#endif /* _HEADER_winstation */
#endif /* _PIDL_HEADER_winstation */
