#ifndef __SOURCE4_KDC_SDB_HDB_H__
#define __SOURCE4_KDC_SDB_HDB_H__

#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2) PRINTF_ATTRIBUTE(a1, a2)
/* This file was automatically generated by mkproto.pl. DO NOT EDIT */

/* this file contains prototypes for functions that are private 
 * to this subsystem or library. These functions should not be 
 * used outside this particular subsystem! */


/* The following definitions come from ../../source4/kdc/sdb_to_hdb.c  */


/**
* @brief Convert the sdb certificate mappings to a HDB_Ext_CertificateMapping
*
* @param s[in]  The certificate mappings to be used
* @param h[out] The converted mappings
*
* @return      0 if  there are no errors
*         ENOMEM unable to allocate memory
*
* @note The HDB_Ext_CertificateMappings may not contain any mappings, this
*       can happen when:
*         - Enforcement mode is none
*         - Or the user is not permitted to use PKINIT when the enforcement
*           mode is compatibility or strong.
*
* @note Memory is allocated and will need to be freed with a call
*       to free_HDB_Ext_CertificateMappings
*
* @note h must be initialised to zero and have no associated allocated
*       memory
*
*/
int sdb_entry_to_hdb_entry(krb5_context context,
			   const struct sdb_entry *s,
			   hdb_entry *h);
#undef _PRINTF_ATTRIBUTE
#define _PRINTF_ATTRIBUTE(a1, a2)

#endif /* __SOURCE4_KDC_SDB_HDB_H__ */

