/* header auto-generated by pidl */

#ifndef _PIDL_HEADER_ad_backup
#define _PIDL_HEADER_ad_backup

#include <stdint.h>


#ifndef _HEADER_ad_backup
#define _HEADER_ad_backup


struct HrRBackupPrepare {
	int _dummy_element;
};


struct HrRBackupEnd {
	int _dummy_element;
};


struct HrRBackupGetAttachmentInformation {
	int _dummy_element;
};


struct HrRBackupOpenFile {
	int _dummy_element;
};


struct HrRBackupRead {
	int _dummy_element;
};


struct HrRBackupClose {
	int _dummy_element;
};


struct HrRBackupGetBackupLogs {
	int _dummy_element;
};


struct HrRBackupTruncateLogs {
	int _dummy_element;
};


struct HrRBackupPing {
	int _dummy_element;
};

#endif /* _HEADER_ad_backup */
#ifndef _HEADER_ad_restore
#define _HEADER_ad_restore


struct HrRIsNTDSOnline {
	int _dummy_element;
};


struct HrRRestorePrepare {
	int _dummy_element;
};


struct HrRRestoreRegister {
	int _dummy_element;
};


struct HrRRestoreRegisterComplete {
	int _dummy_element;
};


struct HrRRestoreGetDatabaseLocations {
	int _dummy_element;
};


struct HrRRestoreEnd {
	int _dummy_element;
};


struct HrRRestoreSetCurrentLogNumber {
	int _dummy_element;
};


struct HrRRestoreCheckLogsForBackup {
	int _dummy_element;
};

#endif /* _HEADER_ad_restore */
#endif /* _PIDL_HEADER_ad_backup */
