/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/keycredlink.h"

#ifndef _HEADER_NDR_keycredlink
#define _HEADER_NDR_keycredlink

#include "../librpc/ndr/ndr_keycredlink.h"
extern const struct ndr_interface_table ndr_table_keycredlink;
#define NDR_KEYCREDLINK_CALL_COUNT (0)
enum ndr_err_code ndr_push_KEYCREDENTIALLINK_ENTRY_Identifier(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum KEYCREDENTIALLINK_ENTRY_Identifier r);
enum ndr_err_code ndr_pull_KEYCREDENTIALLINK_ENTRY_Identifier(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum KEYCREDENTIALLINK_ENTRY_Identifier *r);
void ndr_print_KEYCREDENTIALLINK_ENTRY_Identifier(struct ndr_print *ndr, const char *name, enum KEYCREDENTIALLINK_ENTRY_Identifier r);
enum ndr_err_code ndr_push_KEYCREDENTIALLINK_ENTRY_KeyUsage(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum KEYCREDENTIALLINK_ENTRY_KeyUsage r);
enum ndr_err_code ndr_pull_KEYCREDENTIALLINK_ENTRY_KeyUsage(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum KEYCREDENTIALLINK_ENTRY_KeyUsage *r);
void ndr_print_KEYCREDENTIALLINK_ENTRY_KeyUsage(struct ndr_print *ndr, const char *name, enum KEYCREDENTIALLINK_ENTRY_KeyUsage r);
enum ndr_err_code ndr_push_KEYCREDENTIALLINK_ENTRY_KeySource(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum KEYCREDENTIALLINK_ENTRY_KeySource r);
enum ndr_err_code ndr_pull_KEYCREDENTIALLINK_ENTRY_KeySource(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum KEYCREDENTIALLINK_ENTRY_KeySource *r);
void ndr_print_KEYCREDENTIALLINK_ENTRY_KeySource(struct ndr_print *ndr, const char *name, enum KEYCREDENTIALLINK_ENTRY_KeySource r);
enum ndr_err_code ndr_push_CUSTOM_KEY_INFO_Flags(struct ndr_push *ndr, ndr_flags_type ndr_flags, uint8_t r);
enum ndr_err_code ndr_pull_CUSTOM_KEY_INFO_Flags(struct ndr_pull *ndr, ndr_flags_type ndr_flags, uint8_t *r);
void ndr_print_CUSTOM_KEY_INFO_Flags(struct ndr_print *ndr, const char *name, uint8_t r);
enum ndr_err_code ndr_push_CUSTOM_KEY_INFO_VolType(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum CUSTOM_KEY_INFO_VolType r);
enum ndr_err_code ndr_pull_CUSTOM_KEY_INFO_VolType(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum CUSTOM_KEY_INFO_VolType *r);
void ndr_print_CUSTOM_KEY_INFO_VolType(struct ndr_print *ndr, const char *name, enum CUSTOM_KEY_INFO_VolType r);
enum ndr_err_code ndr_push_CUSTOM_KEY_INFO_SupportsNotification(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum CUSTOM_KEY_INFO_SupportsNotification r);
enum ndr_err_code ndr_pull_CUSTOM_KEY_INFO_SupportsNotification(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum CUSTOM_KEY_INFO_SupportsNotification *r);
void ndr_print_CUSTOM_KEY_INFO_SupportsNotification(struct ndr_print *ndr, const char *name, enum CUSTOM_KEY_INFO_SupportsNotification r);
enum ndr_err_code ndr_push_CUSTOM_KEY_INFO_KeyStrength(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum CUSTOM_KEY_INFO_KeyStrength r);
enum ndr_err_code ndr_pull_CUSTOM_KEY_INFO_KeyStrength(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum CUSTOM_KEY_INFO_KeyStrength *r);
void ndr_print_CUSTOM_KEY_INFO_KeyStrength(struct ndr_print *ndr, const char *name, enum CUSTOM_KEY_INFO_KeyStrength r);
enum ndr_err_code ndr_push_EncodedExtendedCKI(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct EncodedExtendedCKI *r);
enum ndr_err_code ndr_pull_EncodedExtendedCKI(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct EncodedExtendedCKI *r);
void ndr_print_EncodedExtendedCKI(struct ndr_print *ndr, const char *name, const struct EncodedExtendedCKI *r);
enum ndr_err_code ndr_push_CUSTOM_KEY_INFORMATION(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct CUSTOM_KEY_INFORMATION *r);
enum ndr_err_code ndr_pull_CUSTOM_KEY_INFORMATION(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct CUSTOM_KEY_INFORMATION *r);
void ndr_print_CUSTOM_KEY_INFORMATION(struct ndr_print *ndr, const char *name, const struct CUSTOM_KEY_INFORMATION *r);
enum ndr_err_code ndr_push_KEYCREDENTIALLINK_ENTRY_Value(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union KEYCREDENTIALLINK_ENTRY_Value *r);
enum ndr_err_code ndr_pull_KEYCREDENTIALLINK_ENTRY_Value(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union KEYCREDENTIALLINK_ENTRY_Value *r);
void ndr_print_KEYCREDENTIALLINK_ENTRY_Value(struct ndr_print *ndr, const char *name, const union KEYCREDENTIALLINK_ENTRY_Value *r);
size_t ndr_size_KEYCREDENTIALLINK_ENTRY_Value(const union KEYCREDENTIALLINK_ENTRY_Value *r, uint32_t level, libndr_flags flags);
enum ndr_err_code ndr_push_KEYCREDENTIALLINK_ENTRY(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct KEYCREDENTIALLINK_ENTRY *r);
enum ndr_err_code ndr_pull_KEYCREDENTIALLINK_ENTRY(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct KEYCREDENTIALLINK_ENTRY *r);
void ndr_print_KEYCREDENTIALLINK_ENTRY(struct ndr_print *ndr, const char *name, const struct KEYCREDENTIALLINK_ENTRY *r);
enum ndr_err_code ndr_push_KEYCREDENTIALLINK_BLOB(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct KEYCREDENTIALLINK_BLOB *r);
enum ndr_err_code ndr_pull_KEYCREDENTIALLINK_BLOB(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct KEYCREDENTIALLINK_BLOB *r);
void ndr_print_KEYCREDENTIALLINK_BLOB(struct ndr_print *ndr, const char *name, const struct KEYCREDENTIALLINK_BLOB *r);
enum ndr_err_code ndr_push_KeyMaterialInternal(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct KeyMaterialInternal *r);
enum ndr_err_code ndr_pull_KeyMaterialInternal(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct KeyMaterialInternal *r);
void ndr_print_KeyMaterialInternal(struct ndr_print *ndr, const char *name, const struct KeyMaterialInternal *r);
#endif /* _HEADER_NDR_keycredlink */
