/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/source4/librpc/gen_ndr/sasl_helpers.h"

#ifndef _HEADER_NDR_sasl_helpers
#define _HEADER_NDR_sasl_helpers

#define NDR_SASL_HELPERS_UUID "7512b2f4-5f4f-11e4-bbe6-3c970e8d8226"
#define NDR_SASL_HELPERS_VERSION 1.0
#define NDR_SASL_HELPERS_NAME "sasl_helpers"
#define NDR_SASL_HELPERS_HELPSTRING "SASL helpers"
extern const struct ndr_interface_table ndr_table_sasl_helpers;
#define NDR_SASL_HELPERS_CALL_COUNT (0)
enum ndr_err_code ndr_push_saslauthdRequest(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct saslauthdRequest *r);
enum ndr_err_code ndr_pull_saslauthdRequest(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct saslauthdRequest *r);
void ndr_print_saslauthdRequest(struct ndr_print *ndr, const char *name, const struct saslauthdRequest *r);
#endif /* _HEADER_NDR_sasl_helpers */
