/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/w32time.h"

#ifndef _HEADER_NDR_w32time
#define _HEADER_NDR_w32time

#define NDR_W32TIME_UUID "8fb6d884-2388-11d0-8c35-00c04fda2795"
#define NDR_W32TIME_VERSION 65540
#define NDR_W32TIME_NAME "w32time"
#define NDR_W32TIME_HELPSTRING "Win32 Time Server"
extern const struct ndr_interface_table ndr_table_w32time;
#define NDR_W32TIME_SYNCTIME (0x00)

#define NDR_W32TIME_GETNETLOGONSERVICEBITS (0x01)

#define NDR_W32TIME_QUERYPROVIDERSTATUS (0x02)

#define NDR_W32TIME_CALL_COUNT (3)
void ndr_print_w32time_SyncTime(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct w32time_SyncTime *r);
void ndr_print_w32time_GetNetLogonServiceBits(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct w32time_GetNetLogonServiceBits *r);
void ndr_print_w32time_QueryProviderStatus(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct w32time_QueryProviderStatus *r);
#endif /* _HEADER_NDR_w32time */
