/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/wsp.h"

#ifndef _HEADER_NDR_msftewds
#define _HEADER_NDR_msftewds

#include "../librpc/wsp/wsp_helper.h"
extern const struct ndr_interface_table ndr_table_msftewds;
#define NDR_MSFTEWDS_CALL_COUNT (0)
enum ndr_err_code ndr_push_vt_variant_wrap(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_variant_wrap *r);
enum ndr_err_code ndr_pull_vt_variant_wrap(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_variant_wrap *r);
void ndr_print_vt_variant_wrap(struct ndr_print *ndr, const char *name, const struct vt_variant_wrap *r);
enum ndr_err_code ndr_push_vt_decimal(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_decimal *r);
enum ndr_err_code ndr_pull_vt_decimal(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_decimal *r);
void ndr_print_vt_decimal(struct ndr_print *ndr, const char *name, const struct vt_decimal *r);
enum ndr_err_code ndr_push_vt_decimal_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_decimal_vec *r);
enum ndr_err_code ndr_pull_vt_decimal_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_decimal_vec *r);
void ndr_print_vt_decimal_vec(struct ndr_print *ndr, const char *name, const struct vt_decimal_vec *r);
enum ndr_err_code ndr_push_vt_bstr(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_bstr *r);
enum ndr_err_code ndr_pull_vt_bstr(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_bstr *r);
void ndr_print_vt_bstr(struct ndr_print *ndr, const char *name, const struct vt_bstr *r);
enum ndr_err_code ndr_push_vt_bstr_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_bstr_vec *r);
enum ndr_err_code ndr_pull_vt_bstr_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_bstr_vec *r);
void ndr_print_vt_bstr_vec(struct ndr_print *ndr, const char *name, const struct vt_bstr_vec *r);
enum ndr_err_code ndr_push_vt_lpwstr(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_lpwstr *r);
enum ndr_err_code ndr_pull_vt_lpwstr(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_lpwstr *r);
void ndr_print_vt_lpwstr(struct ndr_print *ndr, const char *name, const struct vt_lpwstr *r);
enum ndr_err_code ndr_push_vt_lpwstr_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_lpwstr_vec *r);
enum ndr_err_code ndr_pull_vt_lpwstr_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_lpwstr_vec *r);
void ndr_print_vt_lpwstr_vec(struct ndr_print *ndr, const char *name, const struct vt_lpwstr_vec *r);
enum ndr_err_code ndr_push_vt_compressed_lpwstr(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_compressed_lpwstr *r);
enum ndr_err_code ndr_pull_vt_compressed_lpwstr(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_compressed_lpwstr *r);
void ndr_print_vt_compressed_lpwstr(struct ndr_print *ndr, const char *name, const struct vt_compressed_lpwstr *r);
enum ndr_err_code ndr_push_vt_compressed_lpwstr_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_compressed_lpwstr_vec *r);
enum ndr_err_code ndr_pull_vt_compressed_lpwstr_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_compressed_lpwstr_vec *r);
void ndr_print_vt_compressed_lpwstr_vec(struct ndr_print *ndr, const char *name, const struct vt_compressed_lpwstr_vec *r);
enum ndr_err_code ndr_push_vt_i1_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_i1_vec *r);
enum ndr_err_code ndr_pull_vt_i1_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_i1_vec *r);
void ndr_print_vt_i1_vec(struct ndr_print *ndr, const char *name, const struct vt_i1_vec *r);
enum ndr_err_code ndr_push_vt_ui1_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_ui1_vec *r);
enum ndr_err_code ndr_pull_vt_ui1_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_ui1_vec *r);
void ndr_print_vt_ui1_vec(struct ndr_print *ndr, const char *name, const struct vt_ui1_vec *r);
enum ndr_err_code ndr_push_vt_i2_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_i2_vec *r);
enum ndr_err_code ndr_pull_vt_i2_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_i2_vec *r);
void ndr_print_vt_i2_vec(struct ndr_print *ndr, const char *name, const struct vt_i2_vec *r);
enum ndr_err_code ndr_push_vt_ui2_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_ui2_vec *r);
enum ndr_err_code ndr_pull_vt_ui2_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_ui2_vec *r);
void ndr_print_vt_ui2_vec(struct ndr_print *ndr, const char *name, const struct vt_ui2_vec *r);
enum ndr_err_code ndr_push_vt_i4_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_i4_vec *r);
enum ndr_err_code ndr_pull_vt_i4_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_i4_vec *r);
void ndr_print_vt_i4_vec(struct ndr_print *ndr, const char *name, const struct vt_i4_vec *r);
enum ndr_err_code ndr_push_vt_ui4_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_ui4_vec *r);
enum ndr_err_code ndr_pull_vt_ui4_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_ui4_vec *r);
void ndr_print_vt_ui4_vec(struct ndr_print *ndr, const char *name, const struct vt_ui4_vec *r);
enum ndr_err_code ndr_push_vt_dlong_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_dlong_vec *r);
enum ndr_err_code ndr_pull_vt_dlong_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_dlong_vec *r);
void ndr_print_vt_dlong_vec(struct ndr_print *ndr, const char *name, const struct vt_dlong_vec *r);
enum ndr_err_code ndr_push_vt_udlong_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_udlong_vec *r);
enum ndr_err_code ndr_pull_vt_udlong_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_udlong_vec *r);
void ndr_print_vt_udlong_vec(struct ndr_print *ndr, const char *name, const struct vt_udlong_vec *r);
enum ndr_err_code ndr_push_vt_clsid_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_clsid_vec *r);
enum ndr_err_code ndr_pull_vt_clsid_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_clsid_vec *r);
void ndr_print_vt_clsid_vec(struct ndr_print *ndr, const char *name, const struct vt_clsid_vec *r);
enum ndr_err_code ndr_push_vt_variant_wrap_vec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_variant_wrap_vec *r);
enum ndr_err_code ndr_pull_vt_variant_wrap_vec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_variant_wrap_vec *r);
void ndr_print_vt_variant_wrap_vec(struct ndr_print *ndr, const char *name, const struct vt_variant_wrap_vec *r);
enum ndr_err_code ndr_push_safearraybound(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct safearraybound *r);
enum ndr_err_code ndr_pull_safearraybound(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct safearraybound *r);
void ndr_print_safearraybound(struct ndr_print *ndr, const char *name, const struct safearraybound *r);
enum ndr_err_code ndr_push_vt_i4_safe_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_i4_safe_array *r);
enum ndr_err_code ndr_pull_vt_i4_safe_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_i4_safe_array *r);
void ndr_print_vt_i4_safe_array(struct ndr_print *ndr, const char *name, const struct vt_i4_safe_array *r);
enum ndr_err_code ndr_push_vt_ui4_safe_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_ui4_safe_array *r);
enum ndr_err_code ndr_pull_vt_ui4_safe_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_ui4_safe_array *r);
void ndr_print_vt_ui4_safe_array(struct ndr_print *ndr, const char *name, const struct vt_ui4_safe_array *r);
enum ndr_err_code ndr_push_vt_bstr_safe_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_bstr_safe_array *r);
enum ndr_err_code ndr_pull_vt_bstr_safe_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_bstr_safe_array *r);
void ndr_print_vt_bstr_safe_array(struct ndr_print *ndr, const char *name, const struct vt_bstr_safe_array *r);
enum ndr_err_code ndr_push_vt_i1_safe_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_i1_safe_array *r);
enum ndr_err_code ndr_pull_vt_i1_safe_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_i1_safe_array *r);
void ndr_print_vt_i1_safe_array(struct ndr_print *ndr, const char *name, const struct vt_i1_safe_array *r);
enum ndr_err_code ndr_push_vt_ui1_safe_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_ui1_safe_array *r);
enum ndr_err_code ndr_pull_vt_ui1_safe_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_ui1_safe_array *r);
void ndr_print_vt_ui1_safe_array(struct ndr_print *ndr, const char *name, const struct vt_ui1_safe_array *r);
enum ndr_err_code ndr_push_vt_i2_safe_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_i2_safe_array *r);
enum ndr_err_code ndr_pull_vt_i2_safe_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_i2_safe_array *r);
void ndr_print_vt_i2_safe_array(struct ndr_print *ndr, const char *name, const struct vt_i2_safe_array *r);
enum ndr_err_code ndr_push_vt_ui2_safe_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_ui2_safe_array *r);
enum ndr_err_code ndr_pull_vt_ui2_safe_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_ui2_safe_array *r);
void ndr_print_vt_ui2_safe_array(struct ndr_print *ndr, const char *name, const struct vt_ui2_safe_array *r);
enum ndr_err_code ndr_push_vt_dlong_safe_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_dlong_safe_array *r);
enum ndr_err_code ndr_pull_vt_dlong_safe_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_dlong_safe_array *r);
void ndr_print_vt_dlong_safe_array(struct ndr_print *ndr, const char *name, const struct vt_dlong_safe_array *r);
enum ndr_err_code ndr_push_vt_variant_wrap_safearray(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_variant_wrap_safearray *r);
enum ndr_err_code ndr_pull_vt_variant_wrap_safearray(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_variant_wrap_safearray *r);
void ndr_print_vt_variant_wrap_safearray(struct ndr_print *ndr, const char *name, const struct vt_variant_wrap_safearray *r);
enum ndr_err_code ndr_push_variant_types(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union variant_types *r);
enum ndr_err_code ndr_pull_variant_types(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union variant_types *r);
void ndr_print_variant_types(struct ndr_print *ndr, const char *name, const union variant_types *r);
enum ndr_err_code ndr_push_wsp_cbasestoragevariant(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cbasestoragevariant *r);
enum ndr_err_code ndr_pull_wsp_cbasestoragevariant(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cbasestoragevariant *r);
void ndr_print_wsp_cbasestoragevariant(struct ndr_print *ndr, const char *name, const struct wsp_cbasestoragevariant *r);
enum ndr_err_code ndr_push_wsp_cdbcolid_opt_name(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union wsp_cdbcolid_opt_name *r);
enum ndr_err_code ndr_pull_wsp_cdbcolid_opt_name(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union wsp_cdbcolid_opt_name *r);
void ndr_print_wsp_cdbcolid_opt_name(struct ndr_print *ndr, const char *name, const union wsp_cdbcolid_opt_name *r);
enum ndr_err_code ndr_push_wsp_cdbcolid(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cdbcolid *r);
enum ndr_err_code ndr_pull_wsp_cdbcolid(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cdbcolid *r);
void ndr_print_wsp_cdbcolid(struct ndr_print *ndr, const char *name, const struct wsp_cdbcolid *r);
enum ndr_err_code ndr_push_wsp_header(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_header *r);
enum ndr_err_code ndr_pull_wsp_header(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_header *r);
void ndr_print_wsp_header(struct ndr_print *ndr, const char *name, const struct wsp_header *r);
enum ndr_err_code ndr_push_wsp_cdbprop(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cdbprop *r);
enum ndr_err_code ndr_pull_wsp_cdbprop(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cdbprop *r);
void ndr_print_wsp_cdbprop(struct ndr_print *ndr, const char *name, const struct wsp_cdbprop *r);
enum ndr_err_code ndr_push_wsp_cdbpropset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cdbpropset *r);
enum ndr_err_code ndr_pull_wsp_cdbpropset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cdbpropset *r);
void ndr_print_wsp_cdbpropset(struct ndr_print *ndr, const char *name, const struct wsp_cdbpropset *r);
enum ndr_err_code ndr_push_wsp_csort(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_csort *r);
enum ndr_err_code ndr_pull_wsp_csort(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_csort *r);
void ndr_print_wsp_csort(struct ndr_print *ndr, const char *name, const struct wsp_csort *r);
enum ndr_err_code ndr_push_wsp_csortset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_csortset *r);
enum ndr_err_code ndr_pull_wsp_csortset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_csortset *r);
void ndr_print_wsp_csortset(struct ndr_print *ndr, const char *name, const struct wsp_csortset *r);
enum ndr_err_code ndr_push_connectin_propsets(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct connectin_propsets *r);
enum ndr_err_code ndr_pull_connectin_propsets(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct connectin_propsets *r);
void ndr_print_connectin_propsets(struct ndr_print *ndr, const char *name, const struct connectin_propsets *r);
enum ndr_err_code ndr_push_connectin_extpropsets(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct connectin_extpropsets *r);
enum ndr_err_code ndr_pull_connectin_extpropsets(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct connectin_extpropsets *r);
void ndr_print_connectin_extpropsets(struct ndr_print *ndr, const char *name, const struct connectin_extpropsets *r);
enum ndr_err_code ndr_push_wsp_cpmconnectin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmconnectin *r);
enum ndr_err_code ndr_pull_wsp_cpmconnectin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmconnectin *r);
void ndr_print_wsp_cpmconnectin(struct ndr_print *ndr, const char *name, const struct wsp_cpmconnectin *r);
enum ndr_err_code ndr_push_version_info(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct version_info *r);
enum ndr_err_code ndr_pull_version_info(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct version_info *r);
void ndr_print_version_info(struct ndr_print *ndr, const char *name, const struct version_info *r);
enum ndr_err_code ndr_push_version_dependant(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union version_dependant *r);
enum ndr_err_code ndr_pull_version_dependant(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union version_dependant *r);
void ndr_print_version_dependant(struct ndr_print *ndr, const char *name, const union version_dependant *r);
enum ndr_err_code ndr_push_wsp_cpmconnectout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmconnectout *r);
enum ndr_err_code ndr_pull_wsp_cpmconnectout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmconnectout *r);
void ndr_print_wsp_cpmconnectout(struct ndr_print *ndr, const char *name, const struct wsp_cpmconnectout *r);
enum ndr_err_code ndr_push_wsp_ccolumnset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_ccolumnset *r);
enum ndr_err_code ndr_pull_wsp_ccolumnset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_ccolumnset *r);
void ndr_print_wsp_ccolumnset(struct ndr_print *ndr, const char *name, const struct wsp_ccolumnset *r);
enum ndr_err_code ndr_push_wsp_cnoderestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cnoderestriction *r);
enum ndr_err_code ndr_pull_wsp_cnoderestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cnoderestriction *r);
void ndr_print_wsp_cnoderestriction(struct ndr_print *ndr, const char *name, const struct wsp_cnoderestriction *r);
enum ndr_err_code ndr_push_wsp_len_string_pair(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_len_string_pair *r);
enum ndr_err_code ndr_pull_wsp_len_string_pair(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_len_string_pair *r);
void ndr_print_wsp_len_string_pair(struct ndr_print *ndr, const char *name, const struct wsp_len_string_pair *r);
enum ndr_err_code ndr_push_wsp_propname_or_propid(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union wsp_propname_or_propid *r);
enum ndr_err_code ndr_pull_wsp_propname_or_propid(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union wsp_propname_or_propid *r);
void ndr_print_wsp_propname_or_propid(struct ndr_print *ndr, const char *name, const union wsp_propname_or_propid *r);
enum ndr_err_code ndr_push_wsp_labeldata(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_labeldata *r);
enum ndr_err_code ndr_pull_wsp_labeldata(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_labeldata *r);
void ndr_print_wsp_labeldata(struct ndr_print *ndr, const char *name, const struct wsp_labeldata *r);
enum ndr_err_code ndr_push_opt_labeldata(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union opt_labeldata *r);
enum ndr_err_code ndr_pull_opt_labeldata(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union opt_labeldata *r);
void ndr_print_opt_labeldata(struct ndr_print *ndr, const char *name, const union opt_labeldata *r);
enum ndr_err_code ndr_push_wsp_rangeboundary(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_rangeboundary *r);
enum ndr_err_code ndr_pull_wsp_rangeboundary(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_rangeboundary *r);
void ndr_print_wsp_rangeboundary(struct ndr_print *ndr, const char *name, const struct wsp_rangeboundary *r);
enum ndr_err_code ndr_push_wsp_crangecategspec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_crangecategspec *r);
enum ndr_err_code ndr_pull_wsp_crangecategspec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_crangecategspec *r);
void ndr_print_wsp_crangecategspec(struct ndr_print *ndr, const char *name, const struct wsp_crangecategspec *r);
enum ndr_err_code ndr_push_opt_crangecatespec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union opt_crangecatespec *r);
enum ndr_err_code ndr_pull_opt_crangecatespec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union opt_crangecatespec *r);
void ndr_print_opt_crangecatespec(struct ndr_print *ndr, const char *name, const union opt_crangecatespec *r);
enum ndr_err_code ndr_push_wsp_ccategspec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_ccategspec *r);
enum ndr_err_code ndr_pull_wsp_ccategspec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_ccategspec *r);
void ndr_print_wsp_ccategspec(struct ndr_print *ndr, const char *name, const struct wsp_ccategspec *r);
enum ndr_err_code ndr_push_wsp_repofdata(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_repofdata *r);
enum ndr_err_code ndr_pull_wsp_repofdata(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_repofdata *r);
void ndr_print_wsp_repofdata(struct ndr_print *ndr, const char *name, const struct wsp_repofdata *r);
enum ndr_err_code ndr_push_opt_type_data(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union opt_type_data *r);
enum ndr_err_code ndr_pull_opt_type_data(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union opt_type_data *r);
void ndr_print_opt_type_data(struct ndr_print *ndr, const char *name, const union opt_type_data *r);
enum ndr_err_code ndr_push_wsp_caggregspec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_caggregspec *r);
enum ndr_err_code ndr_pull_wsp_caggregspec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_caggregspec *r);
void ndr_print_wsp_caggregspec(struct ndr_print *ndr, const char *name, const struct wsp_caggregspec *r);
enum ndr_err_code ndr_push_wsp_caggregset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_caggregset *r);
enum ndr_err_code ndr_pull_wsp_caggregset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_caggregset *r);
void ndr_print_wsp_caggregset(struct ndr_print *ndr, const char *name, const struct wsp_caggregset *r);
enum ndr_err_code ndr_push_wsp_caggregsortkey(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_caggregsortkey *r);
enum ndr_err_code ndr_pull_wsp_caggregsortkey(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_caggregsortkey *r);
void ndr_print_wsp_caggregsortkey(struct ndr_print *ndr, const char *name, const struct wsp_caggregsortkey *r);
enum ndr_err_code ndr_push_wsp_csortaggregset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_csortaggregset *r);
enum ndr_err_code ndr_pull_wsp_csortaggregset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_csortaggregset *r);
void ndr_print_wsp_csortaggregset(struct ndr_print *ndr, const char *name, const struct wsp_csortaggregset *r);
enum ndr_err_code ndr_push_wsp_opt_ingroupid(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union wsp_opt_ingroupid *r);
enum ndr_err_code ndr_pull_wsp_opt_ingroupid(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union wsp_opt_ingroupid *r);
void ndr_print_wsp_opt_ingroupid(struct ndr_print *ndr, const char *name, const union wsp_opt_ingroupid *r);
enum ndr_err_code ndr_push_wsp_cingroupsortaggregset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cingroupsortaggregset *r);
enum ndr_err_code ndr_pull_wsp_cingroupsortaggregset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cingroupsortaggregset *r);
void ndr_print_wsp_cingroupsortaggregset(struct ndr_print *ndr, const char *name, const struct wsp_cingroupsortaggregset *r);
enum ndr_err_code ndr_push_wsp_cingroupsortaggregsets(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cingroupsortaggregsets *r);
enum ndr_err_code ndr_pull_wsp_cingroupsortaggregsets(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cingroupsortaggregsets *r);
void ndr_print_wsp_cingroupsortaggregsets(struct ndr_print *ndr, const char *name, const struct wsp_cingroupsortaggregsets *r);
enum ndr_err_code ndr_push_wsp_ccategorizationspec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_ccategorizationspec *r);
enum ndr_err_code ndr_pull_wsp_ccategorizationspec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_ccategorizationspec *r);
void ndr_print_wsp_ccategorizationspec(struct ndr_print *ndr, const char *name, const struct wsp_ccategorizationspec *r);
enum ndr_err_code ndr_push_wsp_ccategorizationset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_ccategorizationset *r);
enum ndr_err_code ndr_pull_wsp_ccategorizationset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_ccategorizationset *r);
void ndr_print_wsp_ccategorizationset(struct ndr_print *ndr, const char *name, const struct wsp_ccategorizationset *r);
enum ndr_err_code ndr_push_wsp_cfullpropspec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cfullpropspec *r);
enum ndr_err_code ndr_pull_wsp_cfullpropspec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cfullpropspec *r);
void ndr_print_wsp_cfullpropspec(struct ndr_print *ndr, const char *name, const struct wsp_cfullpropspec *r);
enum ndr_err_code ndr_push_wsp_ccontentrestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_ccontentrestriction *r);
enum ndr_err_code ndr_pull_wsp_ccontentrestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_ccontentrestriction *r);
void ndr_print_wsp_ccontentrestriction(struct ndr_print *ndr, const char *name, const struct wsp_ccontentrestriction *r);
enum ndr_err_code ndr_push_wsp_cpropertyrestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpropertyrestriction *r);
enum ndr_err_code ndr_pull_wsp_cpropertyrestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpropertyrestriction *r);
void ndr_print_wsp_cpropertyrestriction(struct ndr_print *ndr, const char *name, const struct wsp_cpropertyrestriction *r);
enum ndr_err_code ndr_push_wsp_cnatlanguagerestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cnatlanguagerestriction *r);
enum ndr_err_code ndr_pull_wsp_cnatlanguagerestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cnatlanguagerestriction *r);
void ndr_print_wsp_cnatlanguagerestriction(struct ndr_print *ndr, const char *name, const struct wsp_cnatlanguagerestriction *r);
enum ndr_err_code ndr_push_wsp_wrap_crestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_wrap_crestriction *r);
enum ndr_err_code ndr_pull_wsp_wrap_crestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_wrap_crestriction *r);
void ndr_print_wsp_wrap_crestriction(struct ndr_print *ndr, const char *name, const struct wsp_wrap_crestriction *r);
enum ndr_err_code ndr_push_wsp_ccoercionrestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_ccoercionrestriction *r);
enum ndr_err_code ndr_pull_wsp_ccoercionrestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_ccoercionrestriction *r);
void ndr_print_wsp_ccoercionrestriction(struct ndr_print *ndr, const char *name, const struct wsp_ccoercionrestriction *r);
enum ndr_err_code ndr_push_wsp_cvectorrestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cvectorrestriction *r);
enum ndr_err_code ndr_pull_wsp_cvectorrestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cvectorrestriction *r);
void ndr_print_wsp_cvectorrestriction(struct ndr_print *ndr, const char *name, const struct wsp_cvectorrestriction *r);
enum ndr_err_code ndr_push_wsp_cscoperestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cscoperestriction *r);
enum ndr_err_code ndr_pull_wsp_cscoperestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cscoperestriction *r);
void ndr_print_wsp_cscoperestriction(struct ndr_print *ndr, const char *name, const struct wsp_cscoperestriction *r);
enum ndr_err_code ndr_push_wsp_creusewhere(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_creusewhere *r);
enum ndr_err_code ndr_pull_wsp_creusewhere(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_creusewhere *r);
void ndr_print_wsp_creusewhere(struct ndr_print *ndr, const char *name, const struct wsp_creusewhere *r);
enum ndr_err_code ndr_push_wsp_cinternalpropertyrestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cinternalpropertyrestriction *r);
enum ndr_err_code ndr_pull_wsp_cinternalpropertyrestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cinternalpropertyrestriction *r);
void ndr_print_wsp_cinternalpropertyrestriction(struct ndr_print *ndr, const char *name, const struct wsp_cinternalpropertyrestriction *r);
enum ndr_err_code ndr_push_wsp_cprobrestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cprobrestriction *r);
enum ndr_err_code ndr_pull_wsp_cprobrestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cprobrestriction *r);
void ndr_print_wsp_cprobrestriction(struct ndr_print *ndr, const char *name, const struct wsp_cprobrestriction *r);
enum ndr_err_code ndr_push_wsp_cfeedbackrestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cfeedbackrestriction *r);
enum ndr_err_code ndr_pull_wsp_cfeedbackrestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cfeedbackrestriction *r);
void ndr_print_wsp_cfeedbackrestriction(struct ndr_print *ndr, const char *name, const struct wsp_cfeedbackrestriction *r);
enum ndr_err_code ndr_push_wsp_creldocrestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_creldocrestriction *r);
enum ndr_err_code ndr_pull_wsp_creldocrestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_creldocrestriction *r);
void ndr_print_wsp_creldocrestriction(struct ndr_print *ndr, const char *name, const struct wsp_creldocrestriction *r);
enum ndr_err_code ndr_push_wsp_crestrictions(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union wsp_crestrictions *r);
enum ndr_err_code ndr_pull_wsp_crestrictions(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union wsp_crestrictions *r);
void ndr_print_wsp_crestrictions(struct ndr_print *ndr, const char *name, const union wsp_crestrictions *r);
enum ndr_err_code ndr_push_wsp_crestriction(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_crestriction *r);
enum ndr_err_code ndr_pull_wsp_crestriction(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_crestriction *r);
void ndr_print_wsp_crestriction(struct ndr_print *ndr, const char *name, const struct wsp_crestriction *r);
enum ndr_err_code ndr_push_wsp_crestrictionarray(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_crestrictionarray *r);
enum ndr_err_code ndr_pull_wsp_crestrictionarray(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_crestrictionarray *r);
void ndr_print_wsp_crestrictionarray(struct ndr_print *ndr, const char *name, const struct wsp_crestrictionarray *r);
enum ndr_err_code ndr_push_wsp_crowsetproperties(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_crowsetproperties *r);
enum ndr_err_code ndr_pull_wsp_crowsetproperties(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_crowsetproperties *r);
void ndr_print_wsp_crowsetproperties(struct ndr_print *ndr, const char *name, const struct wsp_crowsetproperties *r);
enum ndr_err_code ndr_push_wsp_cpidmapper(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpidmapper *r);
enum ndr_err_code ndr_pull_wsp_cpidmapper(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpidmapper *r);
void ndr_print_wsp_cpidmapper(struct ndr_print *ndr, const char *name, const struct wsp_cpidmapper *r);
enum ndr_err_code ndr_push_wsp_sproperty(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_sproperty *r);
enum ndr_err_code ndr_pull_wsp_sproperty(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_sproperty *r);
void ndr_print_wsp_sproperty(struct ndr_print *ndr, const char *name, const struct wsp_sproperty *r);
enum ndr_err_code ndr_push_wsp_ccolumngroup(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_ccolumngroup *r);
enum ndr_err_code ndr_pull_wsp_ccolumngroup(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_ccolumngroup *r);
void ndr_print_wsp_ccolumngroup(struct ndr_print *ndr, const char *name, const struct wsp_ccolumngroup *r);
enum ndr_err_code ndr_push_wsp_ccolumngrouparray(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_ccolumngrouparray *r);
enum ndr_err_code ndr_pull_wsp_ccolumngrouparray(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_ccolumngrouparray *r);
void ndr_print_wsp_ccolumngrouparray(struct ndr_print *ndr, const char *name, const struct wsp_ccolumngrouparray *r);
enum ndr_err_code ndr_push_opt_wsp_cingroupsortaggregsets(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union opt_wsp_cingroupsortaggregsets *r);
enum ndr_err_code ndr_pull_opt_wsp_cingroupsortaggregsets(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union opt_wsp_cingroupsortaggregsets *r);
void ndr_print_opt_wsp_cingroupsortaggregsets(struct ndr_print *ndr, const char *name, const union opt_wsp_cingroupsortaggregsets *r);
enum ndr_err_code ndr_push_opt_wsp_crestrictionarray(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union opt_wsp_crestrictionarray *r);
enum ndr_err_code ndr_pull_opt_wsp_crestrictionarray(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union opt_wsp_crestrictionarray *r);
void ndr_print_opt_wsp_crestrictionarray(struct ndr_print *ndr, const char *name, const union opt_wsp_crestrictionarray *r);
enum ndr_err_code ndr_push_opt_wsp_ccolumnset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union opt_wsp_ccolumnset *r);
enum ndr_err_code ndr_pull_opt_wsp_ccolumnset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union opt_wsp_ccolumnset *r);
void ndr_print_opt_wsp_ccolumnset(struct ndr_print *ndr, const char *name, const union opt_wsp_ccolumnset *r);
enum ndr_err_code ndr_push_opt_wsp_ccategorizationset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union opt_wsp_ccategorizationset *r);
enum ndr_err_code ndr_pull_opt_wsp_ccategorizationset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union opt_wsp_ccategorizationset *r);
void ndr_print_opt_wsp_ccategorizationset(struct ndr_print *ndr, const char *name, const union opt_wsp_ccategorizationset *r);
enum ndr_err_code ndr_push_wsp_cpmcreatequeryin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmcreatequeryin *r);
enum ndr_err_code ndr_pull_wsp_cpmcreatequeryin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmcreatequeryin *r);
void ndr_print_wsp_cpmcreatequeryin(struct ndr_print *ndr, const char *name, const struct wsp_cpmcreatequeryin *r);
enum ndr_err_code ndr_push_wsp_cpmcreatequeryout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmcreatequeryout *r);
enum ndr_err_code ndr_pull_wsp_cpmcreatequeryout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmcreatequeryout *r);
void ndr_print_wsp_cpmcreatequeryout(struct ndr_print *ndr, const char *name, const struct wsp_cpmcreatequeryout *r);
enum ndr_err_code ndr_push_aggregatetype(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union aggregatetype *r);
enum ndr_err_code ndr_pull_aggregatetype(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union aggregatetype *r);
void ndr_print_aggregatetype(struct ndr_print *ndr, const char *name, const union aggregatetype *r);
enum ndr_err_code ndr_push_valueoffset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union valueoffset *r);
enum ndr_err_code ndr_pull_valueoffset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union valueoffset *r);
void ndr_print_valueoffset(struct ndr_print *ndr, const char *name, const union valueoffset *r);
enum ndr_err_code ndr_push_valuesize(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union valuesize *r);
enum ndr_err_code ndr_pull_valuesize(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union valuesize *r);
void ndr_print_valuesize(struct ndr_print *ndr, const char *name, const union valuesize *r);
enum ndr_err_code ndr_push_lengthoffset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union lengthoffset *r);
enum ndr_err_code ndr_pull_lengthoffset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union lengthoffset *r);
void ndr_print_lengthoffset(struct ndr_print *ndr, const char *name, const union lengthoffset *r);
enum ndr_err_code ndr_push_statusoffset(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union statusoffset *r);
enum ndr_err_code ndr_pull_statusoffset(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union statusoffset *r);
void ndr_print_statusoffset(struct ndr_print *ndr, const char *name, const union statusoffset *r);
enum ndr_err_code ndr_push_wsp_ctablecolumn(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_ctablecolumn *r);
enum ndr_err_code ndr_pull_wsp_ctablecolumn(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_ctablecolumn *r);
void ndr_print_wsp_ctablecolumn(struct ndr_print *ndr, const char *name, const struct wsp_ctablecolumn *r);
enum ndr_err_code ndr_push_wsp_ccompletioncategspec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_ccompletioncategspec *r);
enum ndr_err_code ndr_pull_wsp_ccompletioncategspec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_ccompletioncategspec *r);
void ndr_print_wsp_ccompletioncategspec(struct ndr_print *ndr, const char *name, const struct wsp_ccompletioncategspec *r);
enum ndr_err_code ndr_push_wsp_cpmsetbindingsin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmsetbindingsin *r);
enum ndr_err_code ndr_pull_wsp_cpmsetbindingsin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmsetbindingsin *r);
void ndr_print_wsp_cpmsetbindingsin(struct ndr_print *ndr, const char *name, const struct wsp_cpmsetbindingsin *r);
enum ndr_err_code ndr_push_wsp_crowseeknext(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_crowseeknext *r);
enum ndr_err_code ndr_pull_wsp_crowseeknext(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_crowseeknext *r);
void ndr_print_wsp_crowseeknext(struct ndr_print *ndr, const char *name, const struct wsp_crowseeknext *r);
enum ndr_err_code ndr_push_wsp_crowseekat(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_crowseekat *r);
enum ndr_err_code ndr_pull_wsp_crowseekat(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_crowseekat *r);
void ndr_print_wsp_crowseekat(struct ndr_print *ndr, const char *name, const struct wsp_crowseekat *r);
enum ndr_err_code ndr_push_wsp_crowseekatratio(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_crowseekatratio *r);
enum ndr_err_code ndr_pull_wsp_crowseekatratio(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_crowseekatratio *r);
void ndr_print_wsp_crowseekatratio(struct ndr_print *ndr, const char *name, const struct wsp_crowseekatratio *r);
enum ndr_err_code ndr_push_wsp_crowseekbybookmark(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_crowseekbybookmark *r);
enum ndr_err_code ndr_pull_wsp_crowseekbybookmark(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_crowseekbybookmark *r);
void ndr_print_wsp_crowseekbybookmark(struct ndr_print *ndr, const char *name, const struct wsp_crowseekbybookmark *r);
enum ndr_err_code ndr_push_wsp_seekdescription(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union wsp_seekdescription *r);
enum ndr_err_code ndr_pull_wsp_seekdescription(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union wsp_seekdescription *r);
void ndr_print_wsp_seekdescription(struct ndr_print *ndr, const char *name, const union wsp_seekdescription *r);
enum ndr_err_code ndr_push_wsp_cpmgetrowsin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmgetrowsin *r);
enum ndr_err_code ndr_pull_wsp_cpmgetrowsin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmgetrowsin *r);
void ndr_print_wsp_cpmgetrowsin(struct ndr_print *ndr, const char *name, const struct wsp_cpmgetrowsin *r);
enum ndr_err_code ndr_push_wsp_ctablevariant(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_ctablevariant *r);
enum ndr_err_code ndr_pull_wsp_ctablevariant(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_ctablevariant *r);
void ndr_print_wsp_ctablevariant(struct ndr_print *ndr, const char *name, const struct wsp_ctablevariant *r);
enum ndr_err_code ndr_push_wsp_cpmgetrowsout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmgetrowsout *r);
enum ndr_err_code ndr_pull_wsp_cpmgetrowsout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmgetrowsout *r);
void ndr_print_wsp_cpmgetrowsout(struct ndr_print *ndr, const char *name, const struct wsp_cpmgetrowsout *r);
enum ndr_err_code ndr_push_wsp_cpmfreecursorin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmfreecursorin *r);
enum ndr_err_code ndr_pull_wsp_cpmfreecursorin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmfreecursorin *r);
void ndr_print_wsp_cpmfreecursorin(struct ndr_print *ndr, const char *name, const struct wsp_cpmfreecursorin *r);
enum ndr_err_code ndr_push_wsp_cpmfreecursorout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmfreecursorout *r);
enum ndr_err_code ndr_pull_wsp_cpmfreecursorout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmfreecursorout *r);
void ndr_print_wsp_cpmfreecursorout(struct ndr_print *ndr, const char *name, const struct wsp_cpmfreecursorout *r);
enum ndr_err_code ndr_push_wsp_cpmgetquerystatusin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmgetquerystatusin *r);
enum ndr_err_code ndr_pull_wsp_cpmgetquerystatusin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmgetquerystatusin *r);
void ndr_print_wsp_cpmgetquerystatusin(struct ndr_print *ndr, const char *name, const struct wsp_cpmgetquerystatusin *r);
enum ndr_err_code ndr_push_wsp_cpmgetquerystatusout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmgetquerystatusout *r);
enum ndr_err_code ndr_pull_wsp_cpmgetquerystatusout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmgetquerystatusout *r);
void ndr_print_wsp_cpmgetquerystatusout(struct ndr_print *ndr, const char *name, const struct wsp_cpmgetquerystatusout *r);
enum ndr_err_code ndr_push_wsp_cpmgetquerystatusexin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmgetquerystatusexin *r);
enum ndr_err_code ndr_pull_wsp_cpmgetquerystatusexin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmgetquerystatusexin *r);
void ndr_print_wsp_cpmgetquerystatusexin(struct ndr_print *ndr, const char *name, const struct wsp_cpmgetquerystatusexin *r);
enum ndr_err_code ndr_push_wsp_cpmgetquerystatusexout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmgetquerystatusexout *r);
enum ndr_err_code ndr_pull_wsp_cpmgetquerystatusexout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmgetquerystatusexout *r);
void ndr_print_wsp_cpmgetquerystatusexout(struct ndr_print *ndr, const char *name, const struct wsp_cpmgetquerystatusexout *r);
enum ndr_err_code ndr_push_wsp_cpmrestartpositionin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmrestartpositionin *r);
enum ndr_err_code ndr_pull_wsp_cpmrestartpositionin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmrestartpositionin *r);
void ndr_print_wsp_cpmrestartpositionin(struct ndr_print *ndr, const char *name, const struct wsp_cpmrestartpositionin *r);
enum ndr_err_code ndr_push_wsp_cpmratiofinishedin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmratiofinishedin *r);
enum ndr_err_code ndr_pull_wsp_cpmratiofinishedin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmratiofinishedin *r);
void ndr_print_wsp_cpmratiofinishedin(struct ndr_print *ndr, const char *name, const struct wsp_cpmratiofinishedin *r);
enum ndr_err_code ndr_push_wsp_cpmratiofinishedout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmratiofinishedout *r);
enum ndr_err_code ndr_pull_wsp_cpmratiofinishedout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmratiofinishedout *r);
void ndr_print_wsp_cpmratiofinishedout(struct ndr_print *ndr, const char *name, const struct wsp_cpmratiofinishedout *r);
enum ndr_err_code ndr_push_wsp_cpmfetchvaluein(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmfetchvaluein *r);
enum ndr_err_code ndr_pull_wsp_cpmfetchvaluein(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmfetchvaluein *r);
void ndr_print_wsp_cpmfetchvaluein(struct ndr_print *ndr, const char *name, const struct wsp_cpmfetchvaluein *r);
enum ndr_err_code ndr_push_vt_i1_safe2_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_i1_safe2_array *r);
enum ndr_err_code ndr_pull_vt_i1_safe2_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_i1_safe2_array *r);
void ndr_print_vt_i1_safe2_array(struct ndr_print *ndr, const char *name, const struct vt_i1_safe2_array *r);
enum ndr_err_code ndr_push_vt_ui1_safe2_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_ui1_safe2_array *r);
enum ndr_err_code ndr_pull_vt_ui1_safe2_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_ui1_safe2_array *r);
void ndr_print_vt_ui1_safe2_array(struct ndr_print *ndr, const char *name, const struct vt_ui1_safe2_array *r);
enum ndr_err_code ndr_push_vt_i2_safe2_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_i2_safe2_array *r);
enum ndr_err_code ndr_pull_vt_i2_safe2_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_i2_safe2_array *r);
void ndr_print_vt_i2_safe2_array(struct ndr_print *ndr, const char *name, const struct vt_i2_safe2_array *r);
enum ndr_err_code ndr_push_vt_ui2_safe2_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_ui2_safe2_array *r);
enum ndr_err_code ndr_pull_vt_ui2_safe2_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_ui2_safe2_array *r);
void ndr_print_vt_ui2_safe2_array(struct ndr_print *ndr, const char *name, const struct vt_ui2_safe2_array *r);
enum ndr_err_code ndr_push_vt_i4_safe2_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_i4_safe2_array *r);
enum ndr_err_code ndr_pull_vt_i4_safe2_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_i4_safe2_array *r);
void ndr_print_vt_i4_safe2_array(struct ndr_print *ndr, const char *name, const struct vt_i4_safe2_array *r);
enum ndr_err_code ndr_push_vt_ui4_safe2_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_ui4_safe2_array *r);
enum ndr_err_code ndr_pull_vt_ui4_safe2_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_ui4_safe2_array *r);
void ndr_print_vt_ui4_safe2_array(struct ndr_print *ndr, const char *name, const struct vt_ui4_safe2_array *r);
enum ndr_err_code ndr_push_vt_dlong_safe2_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_dlong_safe2_array *r);
enum ndr_err_code ndr_pull_vt_dlong_safe2_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_dlong_safe2_array *r);
void ndr_print_vt_dlong_safe2_array(struct ndr_print *ndr, const char *name, const struct vt_dlong_safe2_array *r);
enum ndr_err_code ndr_push_vt_bstr_safe2_array(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_bstr_safe2_array *r);
enum ndr_err_code ndr_pull_vt_bstr_safe2_array(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_bstr_safe2_array *r);
void ndr_print_vt_bstr_safe2_array(struct ndr_print *ndr, const char *name, const struct vt_bstr_safe2_array *r);
enum ndr_err_code ndr_push_vt_variant_wrap_safearray2(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct vt_variant_wrap_safearray2 *r);
enum ndr_err_code ndr_pull_vt_variant_wrap_safearray2(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct vt_variant_wrap_safearray2 *r);
void ndr_print_vt_variant_wrap_safearray2(struct ndr_print *ndr, const char *name, const struct vt_variant_wrap_safearray2 *r);
enum ndr_err_code ndr_push_serialised_types(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union serialised_types *r);
enum ndr_err_code ndr_pull_serialised_types(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union serialised_types *r);
void ndr_print_serialised_types(struct ndr_print *ndr, const char *name, const union serialised_types *r);
enum ndr_err_code ndr_push_wsp_serializedpropertyvalue(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_serializedpropertyvalue *r);
enum ndr_err_code ndr_pull_wsp_serializedpropertyvalue(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_serializedpropertyvalue *r);
void ndr_print_wsp_serializedpropertyvalue(struct ndr_print *ndr, const char *name, const struct wsp_serializedpropertyvalue *r);
enum ndr_err_code ndr_push_wsp_cpmfetchvalueout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmfetchvalueout *r);
enum ndr_err_code ndr_pull_wsp_cpmfetchvalueout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmfetchvalueout *r);
void ndr_print_wsp_cpmfetchvalueout(struct ndr_print *ndr, const char *name, const struct wsp_cpmfetchvalueout *r);
enum ndr_err_code ndr_push_wsp_cpmsetscopeprioritizationin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmsetscopeprioritizationin *r);
enum ndr_err_code ndr_pull_wsp_cpmsetscopeprioritizationin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmsetscopeprioritizationin *r);
void ndr_print_wsp_cpmsetscopeprioritizationin(struct ndr_print *ndr, const char *name, const struct wsp_cpmsetscopeprioritizationin *r);
enum ndr_err_code ndr_push_wsp_cpmsendnotifyout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmsendnotifyout *r);
enum ndr_err_code ndr_pull_wsp_cpmsendnotifyout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmsendnotifyout *r);
void ndr_print_wsp_cpmsendnotifyout(struct ndr_print *ndr, const char *name, const struct wsp_cpmsendnotifyout *r);
enum ndr_err_code ndr_push_wsp_cpmgetrowsetnotifyout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmgetrowsetnotifyout *r);
enum ndr_err_code ndr_pull_wsp_cpmgetrowsetnotifyout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmgetrowsetnotifyout *r);
void ndr_print_wsp_cpmgetrowsetnotifyout(struct ndr_print *ndr, const char *name, const struct wsp_cpmgetrowsetnotifyout *r);
enum ndr_err_code ndr_push_wsp_cpmgetscopestatisticsout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmgetscopestatisticsout *r);
enum ndr_err_code ndr_pull_wsp_cpmgetscopestatisticsout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmgetscopestatisticsout *r);
void ndr_print_wsp_cpmgetscopestatisticsout(struct ndr_print *ndr, const char *name, const struct wsp_cpmgetscopestatisticsout *r);
enum ndr_err_code ndr_push_wsp_cpmgetapproximatepositionin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmgetapproximatepositionin *r);
enum ndr_err_code ndr_pull_wsp_cpmgetapproximatepositionin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmgetapproximatepositionin *r);
void ndr_print_wsp_cpmgetapproximatepositionin(struct ndr_print *ndr, const char *name, const struct wsp_cpmgetapproximatepositionin *r);
enum ndr_err_code ndr_push_wsp_cpmgetapproximatepositionout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmgetapproximatepositionout *r);
enum ndr_err_code ndr_pull_wsp_cpmgetapproximatepositionout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmgetapproximatepositionout *r);
void ndr_print_wsp_cpmgetapproximatepositionout(struct ndr_print *ndr, const char *name, const struct wsp_cpmgetapproximatepositionout *r);
enum ndr_err_code ndr_push_wsp_cpmcomparebmkin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmcomparebmkin *r);
enum ndr_err_code ndr_pull_wsp_cpmcomparebmkin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmcomparebmkin *r);
void ndr_print_wsp_cpmcomparebmkin(struct ndr_print *ndr, const char *name, const struct wsp_cpmcomparebmkin *r);
enum ndr_err_code ndr_push_wsp_cpmcomparebmkout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmcomparebmkout *r);
enum ndr_err_code ndr_pull_wsp_cpmcomparebmkout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmcomparebmkout *r);
void ndr_print_wsp_cpmcomparebmkout(struct ndr_print *ndr, const char *name, const struct wsp_cpmcomparebmkout *r);
enum ndr_err_code ndr_push_wsp_cpmcistateinout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_cpmcistateinout *r);
enum ndr_err_code ndr_pull_wsp_cpmcistateinout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_cpmcistateinout *r);
void ndr_print_wsp_cpmcistateinout(struct ndr_print *ndr, const char *name, const struct wsp_cpmcistateinout *r);
enum ndr_err_code ndr_push_wsp_findindicesin(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_findindicesin *r);
enum ndr_err_code ndr_pull_wsp_findindicesin(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_findindicesin *r);
void ndr_print_wsp_findindicesin(struct ndr_print *ndr, const char *name, const struct wsp_findindicesin *r);
enum ndr_err_code ndr_push_wsp_findindicesout(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_findindicesout *r);
enum ndr_err_code ndr_pull_wsp_findindicesout(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_findindicesout *r);
void ndr_print_wsp_findindicesout(struct ndr_print *ndr, const char *name, const struct wsp_findindicesout *r);
enum ndr_err_code ndr_push_req_message(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union req_message *r);
enum ndr_err_code ndr_pull_req_message(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union req_message *r);
void ndr_print_req_message(struct ndr_print *ndr, const char *name, const union req_message *r);
enum ndr_err_code ndr_push_resp_message(struct ndr_push *ndr, ndr_flags_type ndr_flags, const union resp_message *r);
enum ndr_err_code ndr_pull_resp_message(struct ndr_pull *ndr, ndr_flags_type ndr_flags, union resp_message *r);
void ndr_print_resp_message(struct ndr_print *ndr, const char *name, const union resp_message *r);
enum ndr_err_code ndr_push_wsp_request(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_request *r);
enum ndr_err_code ndr_pull_wsp_request(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_request *r);
void ndr_print_wsp_request(struct ndr_print *ndr, const char *name, const struct wsp_request *r);
enum ndr_err_code ndr_push_wsp_response(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wsp_response *r);
enum ndr_err_code ndr_pull_wsp_response(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wsp_response *r);
void ndr_print_wsp_response(struct ndr_print *ndr, const char *name, const struct wsp_response *r);
#endif /* _HEADER_NDR_msftewds */
