/* client functions auto-generated by pidl */

#include "includes.h"
#include <tevent.h>
#include "lib/util/tevent_ntstatus.h"
#include "bin/default/source3/librpc/gen_ndr/ndr_smbXsrv.h"
#include "bin/default/source3/librpc/gen_ndr/ndr_smbXsrv_c.h"

/* smbXsrv - client functions generated by pidl */

