/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/server_id.h"

#ifndef _HEADER_NDR_server_id
#define _HEADER_NDR_server_id

extern const struct ndr_interface_table ndr_table_server_id;
#define NDR_SERVER_ID_CALL_COUNT (0)
enum ndr_err_code ndr_push_server_id(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct server_id *r);
enum ndr_err_code ndr_pull_server_id(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct server_id *r);
void ndr_print_server_id(struct ndr_print *ndr, const char *name, const struct server_id *r);
#endif /* _HEADER_NDR_server_id */
