/* parser auto-generated by pidl */

#include "replace.h"
#include "bin/default/librpc/gen_ndr/ndr_dsbackup.h"

static enum ndr_err_code ndr_push_HrRBackupPrepare(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRBackupPrepare *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRBackupPrepare(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRBackupPrepare *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRBackupPrepare(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupPrepare *r)
{
	ndr_print_struct(ndr, name, "HrRBackupPrepare");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRBackupPrepare");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRBackupPrepare");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRBackupEnd(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRBackupEnd *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRBackupEnd(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRBackupEnd *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRBackupEnd(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupEnd *r)
{
	ndr_print_struct(ndr, name, "HrRBackupEnd");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRBackupEnd");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRBackupEnd");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRBackupGetAttachmentInformation(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRBackupGetAttachmentInformation *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRBackupGetAttachmentInformation(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRBackupGetAttachmentInformation *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRBackupGetAttachmentInformation(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupGetAttachmentInformation *r)
{
	ndr_print_struct(ndr, name, "HrRBackupGetAttachmentInformation");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRBackupGetAttachmentInformation");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRBackupGetAttachmentInformation");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRBackupOpenFile(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRBackupOpenFile *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRBackupOpenFile(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRBackupOpenFile *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRBackupOpenFile(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupOpenFile *r)
{
	ndr_print_struct(ndr, name, "HrRBackupOpenFile");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRBackupOpenFile");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRBackupOpenFile");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRBackupRead(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRBackupRead *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRBackupRead(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRBackupRead *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRBackupRead(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupRead *r)
{
	ndr_print_struct(ndr, name, "HrRBackupRead");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRBackupRead");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRBackupRead");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRBackupClose(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRBackupClose *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRBackupClose(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRBackupClose *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRBackupClose(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupClose *r)
{
	ndr_print_struct(ndr, name, "HrRBackupClose");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRBackupClose");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRBackupClose");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRBackupGetBackupLogs(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRBackupGetBackupLogs *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRBackupGetBackupLogs(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRBackupGetBackupLogs *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRBackupGetBackupLogs(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupGetBackupLogs *r)
{
	ndr_print_struct(ndr, name, "HrRBackupGetBackupLogs");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRBackupGetBackupLogs");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRBackupGetBackupLogs");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRBackupTruncateLogs(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRBackupTruncateLogs *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRBackupTruncateLogs(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRBackupTruncateLogs *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRBackupTruncateLogs(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupTruncateLogs *r)
{
	ndr_print_struct(ndr, name, "HrRBackupTruncateLogs");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRBackupTruncateLogs");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRBackupTruncateLogs");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRBackupPing(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRBackupPing *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRBackupPing(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRBackupPing *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRBackupPing(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRBackupPing *r)
{
	ndr_print_struct(ndr, name, "HrRBackupPing");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRBackupPing");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRBackupPing");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

#ifndef SKIP_NDR_TABLE_ad_backup
static const struct ndr_interface_public_struct ad_backup_public_structs[] = {
	{ .name = NULL }
};

static const struct ndr_interface_call ad_backup_calls[] = {
	{
		"HrRBackupPrepare",
		sizeof(struct HrRBackupPrepare),
		(ndr_push_flags_fn_t) ndr_push_HrRBackupPrepare,
		(ndr_pull_flags_fn_t) ndr_pull_HrRBackupPrepare,
		(ndr_print_function_t) ndr_print_HrRBackupPrepare,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRBackupEnd",
		sizeof(struct HrRBackupEnd),
		(ndr_push_flags_fn_t) ndr_push_HrRBackupEnd,
		(ndr_pull_flags_fn_t) ndr_pull_HrRBackupEnd,
		(ndr_print_function_t) ndr_print_HrRBackupEnd,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRBackupGetAttachmentInformation",
		sizeof(struct HrRBackupGetAttachmentInformation),
		(ndr_push_flags_fn_t) ndr_push_HrRBackupGetAttachmentInformation,
		(ndr_pull_flags_fn_t) ndr_pull_HrRBackupGetAttachmentInformation,
		(ndr_print_function_t) ndr_print_HrRBackupGetAttachmentInformation,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRBackupOpenFile",
		sizeof(struct HrRBackupOpenFile),
		(ndr_push_flags_fn_t) ndr_push_HrRBackupOpenFile,
		(ndr_pull_flags_fn_t) ndr_pull_HrRBackupOpenFile,
		(ndr_print_function_t) ndr_print_HrRBackupOpenFile,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRBackupRead",
		sizeof(struct HrRBackupRead),
		(ndr_push_flags_fn_t) ndr_push_HrRBackupRead,
		(ndr_pull_flags_fn_t) ndr_pull_HrRBackupRead,
		(ndr_print_function_t) ndr_print_HrRBackupRead,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRBackupClose",
		sizeof(struct HrRBackupClose),
		(ndr_push_flags_fn_t) ndr_push_HrRBackupClose,
		(ndr_pull_flags_fn_t) ndr_pull_HrRBackupClose,
		(ndr_print_function_t) ndr_print_HrRBackupClose,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRBackupGetBackupLogs",
		sizeof(struct HrRBackupGetBackupLogs),
		(ndr_push_flags_fn_t) ndr_push_HrRBackupGetBackupLogs,
		(ndr_pull_flags_fn_t) ndr_pull_HrRBackupGetBackupLogs,
		(ndr_print_function_t) ndr_print_HrRBackupGetBackupLogs,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRBackupTruncateLogs",
		sizeof(struct HrRBackupTruncateLogs),
		(ndr_push_flags_fn_t) ndr_push_HrRBackupTruncateLogs,
		(ndr_pull_flags_fn_t) ndr_pull_HrRBackupTruncateLogs,
		(ndr_print_function_t) ndr_print_HrRBackupTruncateLogs,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRBackupPing",
		sizeof(struct HrRBackupPing),
		(ndr_push_flags_fn_t) ndr_push_HrRBackupPing,
		(ndr_pull_flags_fn_t) ndr_pull_HrRBackupPing,
		(ndr_print_function_t) ndr_print_HrRBackupPing,
		{ 0, NULL },
		{ 0, NULL },
	},
	{ .name = NULL }
};

static const char * const ad_backup_endpoint_strings[] = {
	"ncacn_np:[\\pipe\\ad_backup]", 
};

static const struct ndr_interface_string_array ad_backup_endpoints = {
	.count	= 1,
	.names	= ad_backup_endpoint_strings
};

static const char * const ad_backup_authservice_strings[] = {
	"host", 
};

static const struct ndr_interface_string_array ad_backup_authservices = {
	.count	= 1,
	.names	= ad_backup_authservice_strings
};


const struct ndr_interface_table ndr_table_ad_backup = {
	.name		= "ad_backup",
	.syntax_id	= {
		{0xecec0d70,0xa603,0x11d0,{0x96,0xb1},{0x00,0xa0,0xc9,0x1e,0xce,0x30}},
		NDR_AD_BACKUP_VERSION
	},
	.helpstring	= NDR_AD_BACKUP_HELPSTRING,
	.num_calls	= 9,
	.calls		= ad_backup_calls,
	.num_public_structs	= 0,
	.public_structs		= ad_backup_public_structs,
	.endpoints	= &ad_backup_endpoints,
	.authservices	= &ad_backup_authservices
};

#endif /* SKIP_NDR_TABLE_ad_backup */
static enum ndr_err_code ndr_push_HrRIsNTDSOnline(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRIsNTDSOnline *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRIsNTDSOnline(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRIsNTDSOnline *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRIsNTDSOnline(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRIsNTDSOnline *r)
{
	ndr_print_struct(ndr, name, "HrRIsNTDSOnline");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRIsNTDSOnline");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRIsNTDSOnline");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRRestorePrepare(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRRestorePrepare *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRRestorePrepare(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRRestorePrepare *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRRestorePrepare(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestorePrepare *r)
{
	ndr_print_struct(ndr, name, "HrRRestorePrepare");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRRestorePrepare");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRRestorePrepare");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRRestoreRegister(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRRestoreRegister *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRRestoreRegister(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRRestoreRegister *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRRestoreRegister(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreRegister *r)
{
	ndr_print_struct(ndr, name, "HrRRestoreRegister");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRRestoreRegister");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRRestoreRegister");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRRestoreRegisterComplete(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRRestoreRegisterComplete *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRRestoreRegisterComplete(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRRestoreRegisterComplete *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRRestoreRegisterComplete(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreRegisterComplete *r)
{
	ndr_print_struct(ndr, name, "HrRRestoreRegisterComplete");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRRestoreRegisterComplete");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRRestoreRegisterComplete");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRRestoreGetDatabaseLocations(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRRestoreGetDatabaseLocations *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRRestoreGetDatabaseLocations(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRRestoreGetDatabaseLocations *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRRestoreGetDatabaseLocations(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreGetDatabaseLocations *r)
{
	ndr_print_struct(ndr, name, "HrRRestoreGetDatabaseLocations");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRRestoreGetDatabaseLocations");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRRestoreGetDatabaseLocations");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRRestoreEnd(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRRestoreEnd *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRRestoreEnd(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRRestoreEnd *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRRestoreEnd(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreEnd *r)
{
	ndr_print_struct(ndr, name, "HrRRestoreEnd");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRRestoreEnd");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRRestoreEnd");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRRestoreSetCurrentLogNumber(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRRestoreSetCurrentLogNumber *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRRestoreSetCurrentLogNumber(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRRestoreSetCurrentLogNumber *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRRestoreSetCurrentLogNumber(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreSetCurrentLogNumber *r)
{
	ndr_print_struct(ndr, name, "HrRRestoreSetCurrentLogNumber");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRRestoreSetCurrentLogNumber");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRRestoreSetCurrentLogNumber");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

static enum ndr_err_code ndr_push_HrRRestoreCheckLogsForBackup(struct ndr_push *ndr, ndr_flags_type flags, const struct HrRRestoreCheckLogsForBackup *r)
{
	NDR_PUSH_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
	}
	return NDR_ERR_SUCCESS;
}

static enum ndr_err_code ndr_pull_HrRRestoreCheckLogsForBackup(struct ndr_pull *ndr, ndr_flags_type flags, struct HrRRestoreCheckLogsForBackup *r)
{
	NDR_PULL_CHECK_FN_FLAGS(ndr, flags);
	if (flags & NDR_IN) {
	}
	if (flags & NDR_OUT) {
#ifdef FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION
#endif /* FUZZING_BUILD_MODE_UNSAFE_FOR_PRODUCTION */
	}
	return NDR_ERR_SUCCESS;
}

_PUBLIC_ void ndr_print_HrRRestoreCheckLogsForBackup(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct HrRRestoreCheckLogsForBackup *r)
{
	ndr_print_struct(ndr, name, "HrRRestoreCheckLogsForBackup");
	if (r == NULL) { ndr_print_null(ndr); return; }
	ndr->depth++;
	if (flags & NDR_SET_VALUES) {
		ndr->flags |= LIBNDR_PRINT_SET_VALUES;
	}
	if (flags & NDR_IN) {
		ndr_print_struct(ndr, "in", "HrRRestoreCheckLogsForBackup");
		ndr->depth++;
		ndr->depth--;
	}
	if (flags & NDR_OUT) {
		ndr_print_struct(ndr, "out", "HrRRestoreCheckLogsForBackup");
		ndr->depth++;
		ndr->depth--;
	}
	ndr->depth--;
}

#ifndef SKIP_NDR_TABLE_ad_restore
static const struct ndr_interface_public_struct ad_restore_public_structs[] = {
	{ .name = NULL }
};

static const struct ndr_interface_call ad_restore_calls[] = {
	{
		"HrRIsNTDSOnline",
		sizeof(struct HrRIsNTDSOnline),
		(ndr_push_flags_fn_t) ndr_push_HrRIsNTDSOnline,
		(ndr_pull_flags_fn_t) ndr_pull_HrRIsNTDSOnline,
		(ndr_print_function_t) ndr_print_HrRIsNTDSOnline,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRRestorePrepare",
		sizeof(struct HrRRestorePrepare),
		(ndr_push_flags_fn_t) ndr_push_HrRRestorePrepare,
		(ndr_pull_flags_fn_t) ndr_pull_HrRRestorePrepare,
		(ndr_print_function_t) ndr_print_HrRRestorePrepare,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRRestoreRegister",
		sizeof(struct HrRRestoreRegister),
		(ndr_push_flags_fn_t) ndr_push_HrRRestoreRegister,
		(ndr_pull_flags_fn_t) ndr_pull_HrRRestoreRegister,
		(ndr_print_function_t) ndr_print_HrRRestoreRegister,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRRestoreRegisterComplete",
		sizeof(struct HrRRestoreRegisterComplete),
		(ndr_push_flags_fn_t) ndr_push_HrRRestoreRegisterComplete,
		(ndr_pull_flags_fn_t) ndr_pull_HrRRestoreRegisterComplete,
		(ndr_print_function_t) ndr_print_HrRRestoreRegisterComplete,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRRestoreGetDatabaseLocations",
		sizeof(struct HrRRestoreGetDatabaseLocations),
		(ndr_push_flags_fn_t) ndr_push_HrRRestoreGetDatabaseLocations,
		(ndr_pull_flags_fn_t) ndr_pull_HrRRestoreGetDatabaseLocations,
		(ndr_print_function_t) ndr_print_HrRRestoreGetDatabaseLocations,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRRestoreEnd",
		sizeof(struct HrRRestoreEnd),
		(ndr_push_flags_fn_t) ndr_push_HrRRestoreEnd,
		(ndr_pull_flags_fn_t) ndr_pull_HrRRestoreEnd,
		(ndr_print_function_t) ndr_print_HrRRestoreEnd,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRRestoreSetCurrentLogNumber",
		sizeof(struct HrRRestoreSetCurrentLogNumber),
		(ndr_push_flags_fn_t) ndr_push_HrRRestoreSetCurrentLogNumber,
		(ndr_pull_flags_fn_t) ndr_pull_HrRRestoreSetCurrentLogNumber,
		(ndr_print_function_t) ndr_print_HrRRestoreSetCurrentLogNumber,
		{ 0, NULL },
		{ 0, NULL },
	},
	{
		"HrRRestoreCheckLogsForBackup",
		sizeof(struct HrRRestoreCheckLogsForBackup),
		(ndr_push_flags_fn_t) ndr_push_HrRRestoreCheckLogsForBackup,
		(ndr_pull_flags_fn_t) ndr_pull_HrRRestoreCheckLogsForBackup,
		(ndr_print_function_t) ndr_print_HrRRestoreCheckLogsForBackup,
		{ 0, NULL },
		{ 0, NULL },
	},
	{ .name = NULL }
};

static const char * const ad_restore_endpoint_strings[] = {
	"ncacn_np:[\\pipe\\ad_restore]", 
};

static const struct ndr_interface_string_array ad_restore_endpoints = {
	.count	= 1,
	.names	= ad_restore_endpoint_strings
};

static const char * const ad_restore_authservice_strings[] = {
	"host", 
};

static const struct ndr_interface_string_array ad_restore_authservices = {
	.count	= 1,
	.names	= ad_restore_authservice_strings
};


const struct ndr_interface_table ndr_table_ad_restore = {
	.name		= "ad_restore",
	.syntax_id	= {
		{0x16e0cf3a,0xa604,0x11d0,{0x96,0xb1},{0x00,0xa0,0xc9,0x1e,0xce,0x30}},
		NDR_AD_RESTORE_VERSION
	},
	.helpstring	= NDR_AD_RESTORE_HELPSTRING,
	.num_calls	= 8,
	.calls		= ad_restore_calls,
	.num_public_structs	= 0,
	.public_structs		= ad_restore_public_structs,
	.endpoints	= &ad_restore_endpoints,
	.authservices	= &ad_restore_authservices
};

#endif /* SKIP_NDR_TABLE_ad_restore */
