/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/keysvc.h"

#ifndef _HEADER_NDR_keysvc
#define _HEADER_NDR_keysvc

#define NDR_KEYSVC_UUID "8d0ffe72-d252-11d0-bf8f-00c04fd9126b"
#define NDR_KEYSVC_VERSION 1.0
#define NDR_KEYSVC_NAME "keysvc"
#define NDR_KEYSVC_HELPSTRING "Cryptographic Key Services"
extern const struct ndr_interface_table ndr_table_keysvc;
#define NDR_KEYSVC_UNKNOWN0 (0x00)

#define NDR_KEYSVC_CALL_COUNT (1)
void ndr_print_keysvc_Unknown0(struct ndr_print *ndr, const char *name, ndr_flags_type flags, const struct keysvc_Unknown0 *r);
#endif /* _HEADER_NDR_keysvc */
