/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/messaging.h"

#ifndef _HEADER_NDR_messaging
#define _HEADER_NDR_messaging

extern const struct ndr_interface_table ndr_table_messaging;
#define NDR_MESSAGING_CALL_COUNT (0)
enum ndr_err_code ndr_push_messaging_type(struct ndr_push *ndr, ndr_flags_type ndr_flags, enum messaging_type r);
enum ndr_err_code ndr_pull_messaging_type(struct ndr_pull *ndr, ndr_flags_type ndr_flags, enum messaging_type *r);
void ndr_print_messaging_type(struct ndr_print *ndr, const char *name, enum messaging_type r);
enum ndr_err_code ndr_push_messaging_rec(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct messaging_rec *r);
enum ndr_err_code ndr_pull_messaging_rec(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct messaging_rec *r);
void ndr_print_messaging_rec(struct ndr_print *ndr, const char *name, const struct messaging_rec *r);
enum ndr_err_code ndr_push_messaging_reclog(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct messaging_reclog *r);
enum ndr_err_code ndr_pull_messaging_reclog(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct messaging_reclog *r);
void ndr_print_messaging_reclog(struct ndr_print *ndr, const char *name, const struct messaging_reclog *r);
#endif /* _HEADER_NDR_messaging */
