/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/librpc/gen_ndr/gmsa.h"

#ifndef _HEADER_NDR_gmsa
#define _HEADER_NDR_gmsa

#define NDR_GMSA_UUID "e43b8cf6-1ead-11ee-aed1-e3597136ce70"
#define NDR_GMSA_VERSION 0.0
#define NDR_GMSA_NAME "gmsa"
#define NDR_GMSA_HELPSTRING "Active Directory Group Managed Service Accounts"
extern const struct ndr_interface_table ndr_table_gmsa;
#define NDR_GMSA_CALL_COUNT (0)
void ndr_print_MANAGEDPASSWORD_BLOB_PASSWORDS(struct ndr_print *ndr, const char *name, const struct MANAGEDPASSWORD_BLOB_PASSWORDS *r);
enum ndr_err_code ndr_push_MANAGEDPASSWORD_BLOB(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct MANAGEDPASSWORD_BLOB *r);
enum ndr_err_code ndr_pull_MANAGEDPASSWORD_BLOB(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct MANAGEDPASSWORD_BLOB *r);
void ndr_print_MANAGEDPASSWORD_BLOB(struct ndr_print *ndr, const char *name, const struct MANAGEDPASSWORD_BLOB *r);
#endif /* _HEADER_NDR_gmsa */
