/* header auto-generated by pidl */

#include "librpc/ndr/libndr.h"
#include "bin/default/source4/librpc/gen_ndr/winsrepl.h"

#ifndef _HEADER_NDR_winsrepl
#define _HEADER_NDR_winsrepl

#include "../libcli/nbt/libnbt.h"
#define NDR_WINSREPL_UUID "915f5653-bac1-431c-97ee-9ffb34526921"
#define NDR_WINSREPL_VERSION 0.0
#define NDR_WINSREPL_NAME "winsrepl"
#define NDR_WINSREPL_HELPSTRING "WINS Replication PDUs"
extern const struct ndr_interface_table ndr_table_winsrepl;
#define NDR_WINSREPL_CALL_COUNT (0)
void ndr_print_wrepl_ip(struct ndr_print *ndr, const char *name, const struct wrepl_ip *r);
void ndr_print_wrepl_address_list(struct ndr_print *ndr, const char *name, const struct wrepl_address_list *r);
void ndr_print_wrepl_addresses(struct ndr_print *ndr, const char *name, const union wrepl_addresses *r);
void ndr_print_wrepl_flags(struct ndr_print *ndr, const char *name, uint32_t r);
void ndr_print_wrepl_group_flag(struct ndr_print *ndr, const char *name, enum wrepl_group_flag r);
void ndr_print_wrepl_wins_name(struct ndr_print *ndr, const char *name, const struct wrepl_wins_name *r);
void ndr_print_wrepl_send_reply(struct ndr_print *ndr, const char *name, const struct wrepl_send_reply *r);
void ndr_print_wrepl_wins_owner(struct ndr_print *ndr, const char *name, const struct wrepl_wins_owner *r);
void ndr_print_wrepl_table(struct ndr_print *ndr, const char *name, const struct wrepl_table *r);
void ndr_print_wrepl_replication_cmd(struct ndr_print *ndr, const char *name, enum wrepl_replication_cmd r);
void ndr_print_wrepl_replication_info(struct ndr_print *ndr, const char *name, const union wrepl_replication_info *r);
void ndr_print_wrepl_replication(struct ndr_print *ndr, const char *name, const struct wrepl_replication *r);
void ndr_print_wrepl_start(struct ndr_print *ndr, const char *name, const struct wrepl_start *r);
void ndr_print_wrepl_stop(struct ndr_print *ndr, const char *name, const struct wrepl_stop *r);
void ndr_print_wrepl_mess_type(struct ndr_print *ndr, const char *name, enum wrepl_mess_type r);
void ndr_print_wrepl_message(struct ndr_print *ndr, const char *name, const union wrepl_message *r);
enum ndr_err_code ndr_push_wrepl_packet(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wrepl_packet *r);
enum ndr_err_code ndr_pull_wrepl_packet(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wrepl_packet *r);
void ndr_print_wrepl_packet(struct ndr_print *ndr, const char *name, const struct wrepl_packet *r);
size_t ndr_size_wrepl_packet(const struct wrepl_packet *r, libndr_flags flags);
enum ndr_err_code ndr_push_wrepl_wrap(struct ndr_push *ndr, ndr_flags_type ndr_flags, const struct wrepl_wrap *r);
enum ndr_err_code ndr_pull_wrepl_wrap(struct ndr_pull *ndr, ndr_flags_type ndr_flags, struct wrepl_wrap *r);
void ndr_print_wrepl_wrap(struct ndr_print *ndr, const char *name, const struct wrepl_wrap *r);
#endif /* _HEADER_NDR_winsrepl */
