/*
 *  Unix SMB/CIFS implementation.
 *  libsmbconf - Samba configuration library - Python bindings
 *
 *  Copyright (C) John Mulligan <phlogistonjohn@asynchrono.us> 2022
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _PYSMBCONF_H_
#define _PYSMBCONF_H_

#include "lib/replace/system/python.h"
#include "lib/smbconf/smbconf.h"

typedef struct {
	PyObject_HEAD

	/* C values embedded in our python type */
	TALLOC_CTX * mem_ctx;
	struct smbconf_ctx *conf_ctx;
} py_SMBConf_Object;


#endif /* _PYSMBCONF_H_ */
